<?php
/**
 *   @copyright Copyright (c) 2012 Quality Unit s.r.o.
 *   @package DownloadProtect
 *   @author Juraj Simon
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.postaffiliatepro.com/licenses/license
 */
class Gpf_Common_CallUrlAsync extends Gpf_Tasks_LongTask {

    public function setUrl($url) {
        $params = new stdClass();
        $params->url = $url;
        parent::setParams(serialize($params));
    }

    private function increaseProgress() {
        $progress = $this->getProgress();
        if ($progress == null) {
            $this->setProgress(1);
            return;
        }
        $this->setProgress($this->getProgress() + 1);
    }

    public function execute() {
        $params = unserialize($this->getParams());
        $toUrl = $params->url;
        try {
            Gpf_Log::debug('Curl sending to: ' . $toUrl);
            $ch = curl_init();
            curl_setopt($ch, CURLOPT_URL, $toUrl);
            curl_setopt($ch, CURLOPT_FRESH_CONNECT, true);
            curl_setopt($ch, CURLOPT_TIMEOUT_MS, 10000);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1 );
            curl_setopt($ch, CURLOPT_HTTPHEADER,     array('Content-Type: text/plain'));
            if (curl_exec($ch)===false) {
                Gpf_Log::debug('Curl error: ' . curl_error($ch) . ', code: ' . curl_errno($ch));
                $this->increaseProgress();
                curl_close($ch);
                $this->interrupt(30);
            }
            curl_close($ch);
            Gpf_Log::debug('Curl sending finished');
        } catch (Exception $e) {
            Gpf_Log::error('Failed to forward post to url: ' . $toUrl . '. ' . $e->getMessage());
            $this->increaseProgress();
            $this->interrupt(30);
        }
        $this->setDone();
        $this->forceFinishTask();
    }

    public function getName() {
        return 'Call async curl';
    }
}
