<?php
/**
 *   @copyright Copyright (c) 2007 Quality Unit s.r.o.
 *   @author Viktor Zeman
 *   @package PostAffiliatePro
 *   @since Version 4.5.30
 *   $Id: UpdateManager.class.php 18026 2008-05-14 08:07:20Z mbebjak $
 *
 *   Licensed under the Quality Unit, s.r.o. Dual License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.qualityunit.com/licenses/gpf
 *
 */

/**
 * @package GwtPhpFramework
 */
class Gpf_Log_DeleteLoginsHistoryTask extends Gpf_Log_DeleteTaskBase {

    public function getName() {
        return $this->_('Delete Logins History');
    }

    protected function execute() {
        if (Gpf_Settings::get(Gpf_Settings_Gpf::AUTO_DELETE_LOGINSHISTORY) <= 0) {
            $this->updatePlannedTaskParams();
            $this->setDone();
            return;
        }
        do {
            $this->checkInterruption();
            $statement = $this->deleteLoginshistory();
        } while ($statement->affectedRows() > 0);
        $this->updatePlannedTaskParams();
        $this->setDone();
    }

    private function deleteLoginshistory() {
        $delete = new Gpf_SqlBuilder_DeleteBuilder();
        $delete->from->add(Gpf_Db_Table_LoginsHistory::getName());
        $date = $this->getDateFromParams($this->getParams());
        $delete->where->add(Gpf_Db_Table_LoginsHistory::LAST_REQUEST, '<', $this->getLastDate(Gpf_Settings::get(Gpf_Settings_Gpf::AUTO_DELETE_LOGINSHISTORY), $date)->toDateTime());
        $delete->limit->set('', Gpf_Log_DeleteTaskBase::DELETE_LIMIT);
        return $delete->delete();
    }

    private function updatePlannedTaskParams() {
        $plannedTask = new Gpf_Db_PlannedTask();
        $plannedTask->setClassName('Gpf_Log_DeleteLoginsHistoryTask');
        try {
            $plannedTask->loadFromData(array(Gpf_Db_Table_PlannedTasks::CLASSNAME));
            $plannedTask->setParams($this->getSerializedDateParams(Gpf_Common_DateUtils::now()));
            $plannedTask->update(array(Gpf_Db_Table_PlannedTasks::PARAMS));
        } catch (Gpf_Exception $e) {
            Gpf_Log::error('Error during updating planned DeleteClicksTask: ' . $e->getMessage());
        }
    }
}
