<?php
/**
 *   @copyright Copyright (c) 2012 Quality Unit s.r.o.
 *   @author Juraj Simon
 *   @package LiveAgentPro
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.qualityunit.com/licenses/license
 */

class Gpf_RestApi_Data extends Gpf_Object{
    
    const HANDLER_FIELD_NAME = 'handler';
    
    private $params = array();
    private $method;
    
    public function setRequestMethod($method) {
        $this->method = $method;
    }
    
    public function getRequestMethod() {
        return $this->method;
    }
    
    public function getAllParamsAsString() {
        return var_export($this->params, true);
    }
    
    public function setParam($name, $value) {
        $this->params[$name] = $value;
    }
    
    private function getMandatoryFields() {
        return array(self::HANDLER_FIELD_NAME);
    }
    
    public function getParam($name) {
        if (!array_key_exists($name, $this->params)) {
            return null;
        }
        return $this->params[$name];
    }
    
    public function setHandleString($handlerString) {
        $this->setParam(self::HANDLER_FIELD_NAME, $handlerString);
    }
    
    public function getHandleString() {
        return $this->getParam(self::HANDLER_FIELD_NAME);
    }
    
    private function fieldExists($searchField, array $array) {
        foreach ($array as $arrayField => $value) {
            if (strtolower($searchField) == strtolower($arrayField)) {
                return true;
            }
        }
        return false;
    }
    
    public function fillFromRequestVar($requestVar) {
        foreach($this->getMandatoryFields() as $mandatoryField) {
            if (!$this->fieldExists($mandatoryField, $requestVar)) {
                throw new Gpf_RestApi_BadHandlerException(sprintf('Field %s is missing', $mandatoryField));
            }
        }
        foreach($requestVar as $name => $value) {
            $this->setParam($name, $value);
        }
    }
}
?>
