<?php
/**
 *   @copyright Copyright (c) 2012 Quality Unit s.r.o.
 *   @author Juraj Simon
 *   @package LiveAgentPro
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.qualityunit.com/licenses/license
 */

class Gpf_RestApi_Doc_CallPrinter {
    private $returnFieldCounter = 1;
    private $urlFieldCounter = 1;
    
    protected function printBigRoleIcon($roleId) {
        return Gpf_RestApi_Doc_RoleIconPrinter::printBigRoleIcon($roleId);
    }

    public function printCall($call, $roleId = null) {
        if ($roleId != '' && $call['allowedRole']!=$roleId) {
            return;
        }
        echo '<a name="'.$call['entityitemid'].'"></a>';
        echo '<h3>' . $call['desc']  . '</h3>';
        echo 'Avaliable from application version: ' . $call['fromversion'] . '<br>';
        if (count($call['examplelinks']) > 0) {
            echo '<div style="font-size:14px; padding-top:0px; margin:0px;">Avaliable examples: ';
            foreach ($call['examplelinks'] as $exampleLink) {
                echo "<a href='".$exampleLink['link']."' target='_blank' title='".$exampleLink['name']."'>".$exampleLink['name']."</a> ";
            }
            echo '</div>';
        }
        echo '<br>';
        echo '<b>Allowed role</b><br>';
        if ($call['allowedRole'] == null) {
            echo "This call has no role-specific restrictions.";
        } else {
            echo $this->printBigRoleIcon($call['allowedRole']);
        }
        echo '<br>';
        echo '<br>';
        echo '<b>Example call</b>';
        echo '<div class="'.Gpf_RestApi_Doc_Styles::EXAMPLE_CALL_URL.'">';
        echo '<div class="'.Gpf_RestApi_Doc_Styles::CALL_TYPE_ICON.'">' . $call['type'] . '</div>';


        $call['dataParams'][Dp_api::KEY_FIELD_NAME] = array(Gpf_RestApi_LatestHandler::TYPE => Gpf_RestApi_LatestHandler::TYPE_TEXT, Gpf_RestApi_LatestHandler::MANDATORY => true, Gpf_RestApi_LatestHandler::DESCRIPTION => 'API key');

        if (count($call['dataParams']) > 0) {
            $additionalParamsString = $this->getAdditionalParamsString($call['dataParams'], $call['type']);
        } else {
            $additionalParamsString = '';
        }
        echo '&nbsp;&nbsp;http://example.com/api/' . $call['exampleCall'] . $additionalParamsString . '</div>';
        if (strtolower(trim($call['type'])) == 'put') {
            echo '<span style="font-size:14px;"><i>Note: You must send parameters for this call as PUT fields.<br></i></span>';
        }
        if (strtolower(trim($call['type'])) == 'post') {
            echo '<span style="font-size:14px;"><i>Note: You must send parameters for this call as POST fields.<br></i></span>';
        }
        echo '<br>';
        echo '<p class="'.Gpf_RestApi_Doc_Styles::RETURN_FIELDS_HEADER.'">Mandatory parameters</p>';
        echo '<table cellspacing="0" cellpadding="0" class="'.Gpf_RestApi_Doc_Styles::RETURN_FIELDS_TABLE.'">';
        echo '<tr><th class="'.Gpf_RestApi_Doc_Styles::RETURN_FIELDS_TABLE_HEADER.'">Name</th><th class="'.Gpf_RestApi_Doc_Styles::RETURN_FIELDS_TABLE_HEADER.'">Type</th><th class="'.Gpf_RestApi_Doc_Styles::RETURN_FIELDS_TABLE_HEADER.'">Description</th></tr>';
        if (count($call['exampleCallFields']) > 0 || $this->getDataFieldsCount($call['dataParams'], 'M') > 0) {
            foreach ($call['exampleCallFields'] as $field) {
                echo '<tr><td class="'.$this->getReturnFieldStyle($this->urlFieldCounter).'">' . $field['name'] . '</td><td class="'.$this->getReturnFieldStyle($this->urlFieldCounter).'">text</td><td class="'.$this->getReturnFieldStyle($this->urlFieldCounter).'">' . $field['desc'] . '</td></tr>';
                $this->urlFieldCounter ++;
            }
            if ($this->getDataFieldsCount($call['dataParams'], 'M') > 0) {
                $this->printDataParams($call['dataParams'], 'M');
            }
        } else {
            echo '<tr><td colspan="5" class="'.Gpf_RestApi_Doc_Styles::RETURN_FIELDS_TABLE_CELL_ODD.'"><i>none</i></td></tr>';
        }
        echo '</table>';

        if ($this->getDataFieldsCount($call['dataParams'], 'O') > 0) {
            echo '<br>';
            echo '<p class="'.Gpf_RestApi_Doc_Styles::RETURN_FIELDS_HEADER.'">Optional parameters</p>';
            echo '<table cellspacing="0" cellpadding="0" class="'.Gpf_RestApi_Doc_Styles::RETURN_FIELDS_TABLE.'">';
            echo '<tr><th class="'.Gpf_RestApi_Doc_Styles::RETURN_FIELDS_TABLE_HEADER.'">Name</th><th class="'.Gpf_RestApi_Doc_Styles::RETURN_FIELDS_TABLE_HEADER.'">Type</th><th class="'.Gpf_RestApi_Doc_Styles::RETURN_FIELDS_TABLE_HEADER.'">Description</th></tr>';
            $this->printDataParams($call['dataParams'], 'O');
            echo '</table>';
        }
        echo '<br><br>';
        if ($call['paramnote'] != '') {
            echo "<i>Note: " . $call['paramnote'] . "</i><br><br>";
        }
        echo '<b>Call response description</b><br><br>';
        echo '<p class="'.Gpf_RestApi_Doc_Styles::RETURN_FIELDS_HEADER.'">Return fields</p>';
        echo '<table cellspacing="0" cellpadding="0" class="'.Gpf_RestApi_Doc_Styles::RETURN_FIELDS_TABLE.'">';
        echo '<tr><th class="'.Gpf_RestApi_Doc_Styles::RETURN_FIELDS_TABLE_HEADER.'">Name</th><th class="'.Gpf_RestApi_Doc_Styles::RETURN_FIELDS_TABLE_HEADER.'">Type</th><th class="'.Gpf_RestApi_Doc_Styles::RETURN_FIELDS_TABLE_HEADER.'">Description</th></tr>';
        foreach ($call['returnFields'] as $field) {
            $this->printReturnField($field);

        }
        echo '</table>';
        echo '<br>';
        Gpf_RestApi_Doc_ExamplePrinter::printExampleTable('Example responses', $call['returnExampleXml'], $call['returnExampleJson']);
        echo '<br><br>';
    }

    private function getDataFieldsCount($dataParams, $type = null) {
        $mCount = 0;
        $oCount = 0;
        foreach ($dataParams as $paramName => $dataParam) {
            if ($dataParam[Gpf_RestApi_LatestHandler::MANDATORY] === true) {
                $mCount ++;
            } else {
                $oCount ++;
            }
        }
        if ($type == 'M') {
            return $mCount;
        } else if ($type == 'O') {
            return $oCount;
        }
        return $oCount + $mCount;
    }

    private function getAdditionalParamsString($dataParams, $type) {
        if (strtolower(trim($type)) == 'post' || strtolower(trim($type)) == 'put') {
            return;
        }
        $out = '?';
        foreach ($dataParams as $paramName => $dataParam) {
            if ($dataParam[Gpf_RestApi_LatestHandler::MANDATORY]) {
                $out.='<span style="color:#C94444;">'.$paramName.'=<i>[value]</i></span>&';
            }
        }
        return substr($out, 0, -1);
    }

    private function getDataParamDescription($dataParam) {
        if ($dataParam[Gpf_RestApi_LatestHandler::TYPE] == Gpf_RestApi_LatestHandler::TYPE_CONSTLIST) {
            $def = $dataParam[Gpf_RestApi_LatestHandler::DEFINITION];
            $out = '(Possible values: ';
            foreach ($def->getAllconsts() as $const) {
                $out .= '<i>' . $const . '</i> - ' . $def->getConstDescription($const) . ', ';
            }
            return $dataParam[Gpf_RestApi_LatestHandler::DESCRIPTION] . ' ' . substr($out, 0, -2) . ')';
        }
        return $dataParam[Gpf_RestApi_LatestHandler::DESCRIPTION];
    }

    private function printDataParams($dataParams, $type = 'M') {
        $mParams = array();
        $oParams = array();

        foreach ($dataParams as $paramName => $dataParam) {
            if ($dataParam[Gpf_RestApi_LatestHandler::MANDATORY] === true && $type == 'M') {
                $mParams[] = '<tr><td class="'.$this->getReturnFieldStyle($this->urlFieldCounter).'">'.$paramName. '</td><td class="'.$this->getReturnFieldStyle($this->urlFieldCounter).'">'. $dataParam[Gpf_RestApi_LatestHandler::TYPE] . '</td><td class="'.$this->getReturnFieldStyle($this->urlFieldCounter).'">'.$this->getDataParamDescription($dataParam).'</td></tr>';
            }
            if ($dataParam[Gpf_RestApi_LatestHandler::MANDATORY] === false && $type == 'O') {
                $mParams[] = '<tr><td class="'.$this->getReturnFieldStyle($this->urlFieldCounter).'">'.$paramName. '</td><td class="'.$this->getReturnFieldStyle($this->urlFieldCounter).'">'. $dataParam[Gpf_RestApi_LatestHandler::TYPE] . '</td><td class="'.$this->getReturnFieldStyle($this->urlFieldCounter).'">'.$this->getDataParamDescription($dataParam).'</td></tr>';
            }
            $this->urlFieldCounter++;
        }
        if (count($mParams) > 0) {
            foreach ($mParams as $mParam) {
                echo $mParam;
            }
        }
        if (count($oParams) > 0) {
            foreach ($oParams as $oParam) {
                echo $oParam;
            }
        }
    }

    private function getReturnFieldStyle($count) {
        if ($count % 2 == 0) {
            return Gpf_RestApi_Doc_Styles::RETURN_FIELDS_TABLE_CELL_EVEN;
        }
        return Gpf_RestApi_Doc_Styles::RETURN_FIELDS_TABLE_CELL_ODD;
    }

    private function printReturnField($field) {
        $style = $this->getReturnFieldStyle($this->returnFieldCounter);
        echo '<tr><td class="'.$style.'">' . $field['name'] . '</td><td class="'.$style.'">' . $field['type'] . '</td><td class="'.$style.'">' . $field['desc'] . '</td>';
        $this->returnFieldCounter++;
        if (array_key_exists('columns', $field)) {
            echo '<tr><td colspan="3" class="'.Gpf_RestApi_Doc_Styles::RETURN_FIELDS_TABLE_FIELD_HEADER.'">Field <u>'.$field['name'] . '</u> has '.count($field['columns']).' columns: </td></tr>';
            foreach ($field['columns'] as $column) {
                $this->printReturnField($column);
            }
        }
    }
}
