<?php
/**
 *   @copyright Copyright (c) 2012 Quality Unit s.r.o.
 *   @author Juraj Simon
 *   @package LiveAgentPro
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.qualityunit.com/licenses/license
 */

class Gpf_RestApi_Doc_Generator {

    protected $apiClasses = array();
    protected $apiStructure = array();
    protected $entityStructure = array();
    protected $allowedRole;

    protected function getApiPath() {
    }

    public function setRole($roleId) {
        $this->allowedRole = $roleId;
    }

    private function parseApiDir() {
        $languagesIterator = new Gpf_Io_DirectoryIterator($this->getApiPath(), 'php', true);
        foreach ($languagesIterator as $fullName => $file) {
            $className = str_replace(array('../include/', '.class.php'), '', $fullName);
            $className = str_replace('/', '_', $className);
            try {
                $this->apiClasses[$className] = new ReflectionClass($className);
            } catch  (Exception $e) {
                //do nothing
            }
        }
    }

    protected function getApiPrefix() {
        return 'Gpf_RestApi';
    }

    protected function parseVersions() {
        foreach ($this->apiClasses as $apiClass) {
            $result = preg_match('/^'.$this->getApiPrefix().'_V([0-9]+)_Handler$/i', $apiClass->getName(),$matches);
            if ($result && count($matches)>1) {
                $v = $matches[1];
                $this->apiStructure[$v] = array();
            }
        }
    }

    private function addEntity($version, $entity) {
        $version = strtolower(trim($version));
        $entity = strtolower(trim($entity));
        if (!array_key_exists($version, $this->entityStructure)) {
            $this->entityStructure[$version] = array();
        }
        if (!array_key_exists($entity, $this->entityStructure[$version])) {
            $this->entityStructure[$version][$entity] = array();
        }
    }

    private function addEntityId($version, $entity, $id) {
        $version = strtolower(trim($version));
        $entity = strtolower(trim($entity));
        $this->entityStructure[$version][$entity][] = $id;
    }

    private function parseHandlers() {
        foreach ($this->apiClasses as $apiClass) {
            $classHeader = $apiClass->getDocComment();
            $entity = trim(strtolower($this->getEntity($classHeader)));
            if ($entity != '') {
                $v = $this->getVersionFromClassName($apiClass->getName());
                $this->addEntity($v, $entity);
                $entityId = md5($apiClass->getName());
                $this->addEntityId($v, $entity, $entityId);
                $className = $apiClass->getName();
                $class = new $className;
                $returnMethod = $apiClass->getMethod('getResponseObject');
                $exampleApiCallData = $this->getExampleApiCall($apiClass->getName());
                if (!array_key_exists($entity, $this->apiStructure[$v])) {
                    $this->apiStructure[$v][$entity] = array();
                }
                $this->apiStructure[$v][$entity][] = array('className' => $apiClass->getName(),
                        'desc' => $this->getDescription($classHeader),
                        'type' => strtoupper($class->getName()),
                        'returnFields' => $this->getReturnFields($apiClass),
                        'returnExampleXml' => $this->getExampleXmlResponse($this->getResponseObject($apiClass)),
                        'returnExampleJson' => $this->getExampleJsonResponse($this->getResponseObject($apiClass)),
                        'parentClass' => $this->getPrevioursHandler($apiClass->getName()),
                        'exampleCall' => $this->getApiExampleCallString($exampleApiCallData),
                        'exampleCallFields' => $this->getApiExampleCallFiels($exampleApiCallData),
                        'entityitemid' => $entityId,
                        'dataParams' => $class->getDataParamsList(),
                        'fromversion' => $this->getFromVersion($classHeader),
                        'examplelinks' => Gpf_RestApi_Doc_ClassHeaderParser::getExamplesLinks($classHeader),
                        'paramnote' => Gpf_RestApi_Doc_ClassHeaderParser::getParamNote($classHeader),
                        'allowedRole' => $class->getAllowedRole()
                );
            }
        }
    }

    private function getParemNote() {

    }

    /**
     * @param $handlerName - actual handler
     * @param $nextHandler - handler that will be called from actual handler (its sub-hendler in code)
     */
    private function getExampleApiCall($handlerName, $nextHandlerName = '') {
        $class = new $handlerName;
        $previoursHandlerName = $this->getPrevioursHandler($handlerName);
        if ($this->isResponseHandler($class)) {
            return $this->getExampleApiCall($previoursHandlerName, $handlerName);
        }
        $params = $class->getParams();
        if (count($params) == 0) {
            if ($previoursHandlerName == null) {
                return array();
            } else {
                return $this->getExampleApiCall($previoursHandlerName, $handlerName);
            }
        }
        $callStack = array();
        $nextHandlerClass = new $nextHandlerName;
        //in case of POST call (or call that does not rquire ID - *AllHandler) we do not want to add params to api call URL
        if (($this->isResponseHandler($nextHandlerClass) && $nextHandlerClass instanceof Gpf_RestApi_LatestHandler) ||
                ($this->requireNoIdentifier($class, $nextHandlerClass))) {
            $params = array();
        }
        foreach ($params as $paramName => $paramArr) {
            $param = $paramArr[0];
            $paramDesc = $paramArr[1];
            if ($paramDesc == null) {
                $rClass = $this->apiClasses[$handlerName];
                if ($rClass->hasMethod('getParamName')) {
                    $paramDesc = $this->getMethodDesc($rClass->getMethod('getParamName'));
                }
            }
            if ($paramName == Gpf_RestApi_Handler::NEXT_HANDLER_PARAM_NAME &&
                    $this->handlerHasSubHandlerWithName($class, $nextHandlerName)) {
                if (!$this->isResponseHandler(new $nextHandlerName)) {
                    $callStack[] = array('name' => $nextHandlerClass->getName(), 'desc' => null);
                }
            } else {
                $callStack[] = array('name' => '['.$paramName.']', 'desc' => $paramDesc);
            }
        }
        if ($previoursHandlerName == null) {
            return $callStack;
        }
        return array_merge($this->getExampleApiCall($previoursHandlerName, $handlerName), $callStack);
    }

    private function requireNoIdentifier($handlerClass, $nextHandlerClass) {
        foreach ($handlerClass->getSubHandlers() as $name => $subHandler) {
            if ($name == $nextHandlerClass->getName() . Gpf_RestApi_Handler_IdentifierOnly::SELF_NO_IDENTITNY) {
                return true;
            }
        }
        return false;
    }

    private function getMethodDesc(ReflectionMethod $method) {
        $header = $method->getDocComment();
        $result = preg_match('/@desc\s(.*)/i', $header, $matches);
        if ($result && count($matches) > 1) {
            return $matches[1];
        }
        return '';
    }

    private function getApiExampleCallFiels($callStack) {
        $out = array();
        foreach ($callStack as $callItem) {
            $name = $callItem['name'];
            $desc = $callItem['desc'];
            if (substr($name, 0, 1) != '[') {
                continue;
            }
            $out[] = array('name' => $callItem['name'],  'desc' => $callItem['desc']);
        }
        return $out;
    }

    private function getApiExampleCallString($callStack) {
        $out = '';
        foreach ($callStack as $callItem) {
            if (substr($callItem['name'], 0, 1) == '[') {
                $out .= '<span style="color:#5640E6;">'.$callItem['name'] . '</span>/';
            } else {
                $out .= $callItem['name'] . '/';
            }
        }
        return substr($out, 0, -1);
    }

    private function handlerHasSubHandlerWithName(Gpf_RestApi_Handler $handlerClass, $nextHandlerName) {
        foreach ($handlerClass->getSubHandlers() as $subHandler) {
            if (get_class($subHandler) == $nextHandlerName) {
                return true;
            }
        }
        return false;
    }

    private function isResponseHandler($class) {
        return $class instanceof Gpf_RestApi_GetHandler || $class instanceof Gpf_RestApi_PutHandler || $class instanceof Gpf_RestApi_PostHandler || $class instanceof Gpf_RestApi_DeleteHandler;
    }

    private function getPrevioursHandler($handlerName) {
        foreach ($this->apiClasses as $apiClass) {
            if ($this->isHandler($apiClass->getName())) {
                if ($apiClass->isAbstract() || trim(strtolower($handlerName)) == trim(strtolower($apiClass->getName()))) {
                    continue;
                }
                $className = $apiClass->getName();
                $class = new $className;
                $handlers = $class->getSubHandlers();
                foreach ($handlers as $handler) {
                    if (get_class($handler) == $handlerName) {
                        return $className;
                    }
                }
            }
        }
        return null;
    }

    private function isHandler($className) {
        $result = preg_match('/_Handler$/i', $className, $matches);
        if ($result && count($matches) > 0) {
            return true;
        }
        return false;
    }

    private function getResponseObject(ReflectionClass $handlerClass) {
        $rMethod = $handlerClass->getMethod('getResponseObject');
        preg_match('/@return\s(.*)/i', $rMethod->getDocComment(), $matches);

        $retClass = trim($matches[1]);
        return new $retClass;
    }

    private function getReturnFields(ReflectionClass $handlerClass) {
        $response = $this->getResponseObject($handlerClass);

        $fiels = array();
        foreach ($response->getFields() as $field) {
            $fields[] = $this->getRespFieldItem($field);
        }
        return $fields;
    }

    private function getRespFieldItem(Gpf_RestApi_Response_FieldBase $field) {
        $fieldItem = array('name' => $field->getName(),
                'type' => $field->getType(),
                'desc' => $field->getDescription());
        if ($field instanceof Gpf_RestApi_Response_ConstListField) {
            $fieldItem['desc'] .= ' (Possible values: ' . $this->getConstListfieldDescription($field) . ')';
        }
        if ($field instanceof Gpf_RestApi_Response_RecordsetField) {
            $columns = $field->getHeaderDefinition();
            foreach ($columns as $col) {
                $fieldItem['columns'][] = $this->getRespFieldItem($col);
            }
        }
        return $fieldItem;
    }

    private function getConstListfieldDescription(Gpf_RestApi_Response_ConstListField $field) {
        $out = '';
        foreach ($field->getAllPosibleValues() as $value) {
            $out .= '<i>'.$value . '</i> - ' . $field->getconstDescription($value) . ', ';
        }
        return substr($out, 0, -2);
    }

    private function getColumns($object) {
        $cols = array();
        if ($object instanceof Gpf_DbEngine_Table) {
            foreach ($object->getColumns() as $column) {
                $cols[] = array('name' => $column->getName(), 'type' => $this->getTypeFromColumn($column->getType()));
            }
        }
        return $cols;
    }

    private function getTypeFromColumn($columnType) {
        switch ($columnType) {
            case Gpf_DbEngine_Column::TYPE_DATE:
                return Gpf_RestApi_Response_DateTimeField::TYPE;
            case Gpf_DbEngine_Column::TYPE_NUMBER:
                return Gpf_RestApi_Response_IntField::TYPE;
            case Gpf_DbEngine_Column::TYPE_STRING:
                return Gpf_RestApi_Response_TextField::TYPE;
        }
    }

    private function getVersionFromClassName($className) {
        $result = preg_match('/^'.$this->getApiPrefix().'_V([0-9]+)/i', $className, $matches);
        if ($result && count($matches)>1) {
            return $matches[1];
        }
        return null;
    }

    private function getDescription($descriptionString) {
        $result = preg_match('/@desc (.*)/i', $descriptionString, $matches);
        if ($result && count($matches) > 1) {
            return $matches[1];
        }
        return null;
    }

    private function getFromVersion($descriptionString) {
        $result = preg_match('/@fromversion\s(.*)/i', $descriptionString, $matches);
        if ($result && count($matches) > 1) {
            return $matches[1];
        }
        return null;
    }

    private function getEntity($descriptionString) {
        $result = preg_match('/@docentity\s(.*)/i', $descriptionString, $matches);
        if ($result && count($matches) > 1) {
            return $matches[1];
        }
        return null;
    }

    protected function printSmallroleIcon($roleId) {
        return Gpf_RestApi_Doc_RoleIconPrinter::printSmallRoleIcon($roleId);
    }

    protected function printBigroleIcon($roleId) {
        return Gpf_RestApi_Doc_RoleIconPrinter::printBigRoleIcon($roleId);
    }

    /*private function getRoleName($roleId) {
     if ($roleId == Pap_Application::ROLETYPE_AFFILIATE) {
    return 'Affiliate'
    }
    }*/

    private function printRegister() {
        echo '<a href="#dataformats" title="Format specifications of all possible datafields used in API">Data fields formats</a><br/>';
        echo '<a href="#returnmsgs" title="Return messages formats">Return messages formats</a><br/>';
        echo '<a href="#errormsgs" title="Example of common error messages">Error messages formats</a><br/>';
        foreach ($this->apiStructure as $version => $entities) {
            echo '<a href="#apiv'.$version.'" title="Api ver. '.$version.' complete reference">Api v.' . $version . '</a><br/>';
            foreach ($entities as $entity => $calls) {
                if (!$this->containsAllowedCalls($calls)) {
                    continue;
                }
                echo str_repeat('&nbsp;', 4) . '<a href="#apiv'.$version.'_'.$entity.'">' . strtoupper(substr($entity, 0, 1)) . substr($entity, 1) . '</a><br/>';
                foreach ($calls as $call) {
                    if ($this->allowedRole != '' && $call['allowedRole']!=$this->allowedRole) {
                        continue;
                    }
                    echo str_repeat('&nbsp;', 8).'<a href="#'.$call['entityitemid'].'">' .$call['desc'].'</a>&nbsp;'.$this->printSmallRoleIcon($call['allowedRole']).'<br/>';
                }
            }
        }
    }

    private function containsAllowedCalls($calls) {
        if ($this->allowedRole == null) {
            return true;
        }
        foreach ($calls as $call) {
            if ($call['allowedRole']==$this->allowedRole) {
                return true;
            }
        }
        return false;
    }

    public function printAsHtml($apiStructure) {
        echo Gpf_RestApi_Doc_Styles::getStyles();
        $this->printRegister();
        echo "<br/>";
        $tableStyle = Gpf_RestApi_Doc_Styles::RETURN_FIELDS_TABLE;
        $tableHeaderStyle = Gpf_RestApi_Doc_Styles::RETURN_FIELDS_TABLE_HEADER;
        $tableEven = Gpf_RestApi_Doc_Styles::RETURN_FIELDS_TABLE_CELL_EVEN;
        $tableOdd = Gpf_RestApi_Doc_Styles::RETURN_FIELDS_TABLE_CELL_ODD;
        echo <<<EOT
        <a name="dataformats"></a>
        <h2>Datafield formats</h2><br/>
        <table cellspacing="0" cellpadding="0" class="$tableStyle">
        <tr><th class="$tableHeaderStyle">Type</th><th class="$tableHeaderStyle">Description</th></tr>
        <tr><td class="$tableEven">datetime</td><td class="$tableEven">valid format: YYYY-MM-DD HH:MM:SS. <i>Note: LiveAgent uses "America/Los_Angeles" timezone by default.</i></td></tr>
        <tr><td class="$tableEven">text</td><td class="$tableEven">could by any string or empty</td></tr>
        <tr><td class="$tableEven">constlist</td><td class="$tableEven">could by only one instance of finite set. Possible values are named in specifice field description.</td></tr>
        <tr><td class="$tableEven">int</td><td class="$tableEven">integer value greater or equal to zero.</td></tr>
        <tr><td class="$tableEven">list</td><td class="$tableEven">list of entities. Every entity has its specific column definition.</td></tr>
        </table>
        <br/><br/>

EOT;
        echo '<a name="returnmsgs"></a>';
        echo '<h2>Return messages formats</h2><br/>';
        echo 'Return messages are in JSON format by default. But you can change this to XML by adding <b>.xml</b> at the end of call URL.';
        echo 'This will give response in JSON:<br/>';
        echo '<div class="'.Gpf_RestApi_Doc_Styles::EXAMPLE_CALL_URL.'">';
        echo '<div class="'.Gpf_RestApi_Doc_Styles::CALL_TYPE_ICON.'">GET</div>&nbsp;&nbsp;http://example.com/api/agents/john@agent.com';
        echo '</div><br/>';
        echo 'We add <b>.xml</b> to call URL and we have response in XML format<br/>';
        echo '<div class="'.Gpf_RestApi_Doc_Styles::EXAMPLE_CALL_URL.'">';
        echo '<div class="'.Gpf_RestApi_Doc_Styles::CALL_TYPE_ICON.'">GET</div>&nbsp;&nbsp;http://example.com/api/agents/john@agent.com<span style="color:#C94444;">.xml</span>';
        echo '</div>';
        echo '<br/>';
        echo '<i>Note: if y ou have <a href="http://www.php.net/manual/en/book.yaml.php" target="_blank">PHP YAML</a> extension enabled in your PHP, you could also use <b>.yaml</b> to get response in YAML format.</i>';
        echo '<br/><br/>';
        echo '<a name="errormsgs"></a>';
        echo '<h2>Error messages formats</h2><br/>';
        $e1 = new Gpf_RestApi_Response_Error(new Gpf_RestApi_ProcessingException(500, 'Example error message'));
        $e1->set(Gpf_RestApi_Response_Error::DEBUG_MESSAGE, 'debug info');
        echo '<b>400 - Bad request error</b><br/>';
        echo 'You can get this in case your athentification with API key failed because of problem with API key in application.<br/><br/>';
        echo '<b>401 - Forbidden</b><br/>';
        echo 'You can get this in case of failed authentification with wrong key<br/><br/>';
        echo '<b>404 - Not found error</b><br/>';
        echo 'You can get this in case "agnet with email agent@email.com not found"<br/><br/>';
        echo '<b>500 - Common porcessing error</b><br/>';
        echo 'This is the most common error type. You can get this in case "parameter message is missing" or "unable to load conversation"<br/><br/>';
        echo '<b>503 - Service Unavailable</b><br/>';
        echo 'You can get this in case application is unavaliable to process requests - in case of updating and/or installing.<br/><br/>';

        $this->printErrorMessageExample($e1);
        echo '<br/>';
        foreach ($apiStructure as $version => $versionEntities) {
            echo '<a name="apiv'.$version.'">';
            echo '<h1>API version ' . $version . '</h1>';
            $this->printVersionEntities($versionEntities, $version);
        }
    }

    private function printErrorMessageExample($err) {
        $xml = new Gpf_RestApi_Response_Encoder_Xml();
        $json = new Gpf_RestApi_Response_Encoder_Json();
        Gpf_RestApi_Doc_ExamplePrinter::printExampleTable('Error response example', $xml->encode($err), $json->encode($err));
    }

    private function printVersionEntities($versionEntities, $version) {
        foreach ($versionEntities as $entity => $entityCalls) {
            if (!$this->containsAllowedCalls($entityCalls)) {
                continue;
            }
            echo '<br/>';
            echo '<a name="apiv'.$version . '_' . $entity.'"></a>';
            echo '<h2>' . strtoupper(substr($entity, 0, 1)) . substr($entity, 1) . '</h2><br/>';
            $this->printEntityCalls($entityCalls);
        }
    }

    /**
     * @return Gpf_RestApi_Doc_CallPrinter
     */
    protected function getCallPrinter() {
        return new Gpf_RestApi_Doc_CallPrinter();
    }

    private function printEntityCalls($versionCalls) {
        $callPrinter = $this->getCallPrinter();
        foreach ($versionCalls as $call) {
            $callPrinter->printCall($call, $this->allowedRole);
        }
    }

    protected function getExampleJsonResponse(Gpf_RestApi_Response_Base $reponse) {
    }

    protected function getExampleXmlResponse(Gpf_RestApi_Response_Base $reponse) {
    }

    public function generate() {
        $this->parseApiDir();
        $this->parseVersions();
        $this->parseHandlers();

        $this->printAsHtml($this->apiStructure);
    }
}
?>
