<?php
/**
 *   @copyright Copyright (c) 2012 Quality Unit s.r.o.
 *   @author Juraj Simon
 *   @package LiveAgentPro
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.qualityunit.com/licenses/license
 */

class Gpf_RestApi_Doc_Xml {
    public static function formatXmlString($xml) {
    
        // add marker linefeeds to aid the pretty-tokeniser (adds a linefeed between all tag-end boundaries)
        $xml = preg_replace('/(>)(<)(\/*)/', "$1\n$2$3", $xml);
    
        // now indent the tags
        $token      = strtok($xml, "\n");
        $result     = ''; // holds formatted version as it is built
        $pad        = 0; // initial indent
        $matches    = array(); // returns from preg_matches()
    
        // scan each line and adjust indent based on opening/closing tags
        while ($token !== false) :
    
        // test for the various tag states
    
        // 1. open and closing tags on same line - no change
        if (preg_match('/.+<\/\w[^>]*>$/', $token, $matches)) :
        $indent=0;
        // 2. closing tag - outdent now
        elseif (preg_match('/^<\/\w/', $token, $matches)) :
        $pad--;
        // 3. opening tag - don't pad this one, only subsequent tags
        elseif (preg_match('/^<\w[^>]*[^\/]>.*$/', $token, $matches)) :
        $indent=1;
        // 4. no indentation needed
        else :
        $indent = 0;
        endif;
    
        // pad the line with the required number of leading spaces
        $line    = str_pad($token, strlen($token)+$pad, ' ', STR_PAD_LEFT);
        $result .= $line . "\n"; // add to the cumulative result, with linefeed
        $token   = strtok("\n"); // get the next token
        $pad    += $indent; // update the pad size for subsequent lines
        endwhile;
    
        return self::xml_highlight($result);
    }
    
    public static function xml_highlight($s) {
        $s = htmlspecialchars($s);
        $s = preg_replace("#&lt;([/]*?)(.*)([\s]*?)&gt;#sU",
                "<font color=\"#0000FF\">&lt;\\1\\2\\3&gt;</font>",$s);
        $s = preg_replace("#&lt;([\?])(.*)([\?])&gt;#sU",
                "<font color=\"#800000\">&lt;\\1\\2\\3&gt;</font>",$s);
        $s = preg_replace("#&lt;([^\s\?/=])(.*)([\[\s/]|&gt;)#iU",
                "&lt;<font color=\"#808000\">\\1\\2</font>\\3",$s);
        $s = preg_replace("#&lt;([/])([^\s]*?)([\s\]]*?)&gt;#iU",
                "&lt;\\1<font color=\"#808000\">\\2</font>\\3&gt;",$s);
        $s = preg_replace("#([^\s]*?)\=(&quot;|')(.*)(&quot;|')#isU",
                "<font color=\"#800080\">\\1</font>=<font color=\"#FF00FF\">\\2\\3\\4</font>",$s);
        $s = preg_replace("#&lt;(.*)(\[)(.*)(\])&gt;#isU",
                "&lt;\\1<font color=\"#800080\">\\2\\3\\4</font>&gt;",$s);
        return $s;
    }
}
