<?php
/**
 *   @copyright Copyright (c) 2012 Quality Unit s.r.o.
 *   @author Juraj Simon
 *   @package LiveAgentPro
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.qualityunit.com/licenses/license
 */

abstract class Gpf_RestApi_Handler extends Gpf_Object {

    const NEXT_HANDLER_PARAM_NAME = 'nextHandler';

    const SUB_HANDLER = '_subHandler';

    protected $nextHandlerName;

    protected $allowedRoleTypeId = null;

    /**
     * @var Gpf_RestApi_Data
     */
    protected $data;

    protected $subHandlers = array();
    protected $params = array();
    protected $optionalParams = array();

    public function __construct() {
        $this->setAllowedRole();
        $this->addParams();
        $this->addSubhandlers();
    }

    public function getAllowedRole() {
        return $this->allowedRoleTypeId;
    }

    protected function setAllowedRole() {
    }

    protected function addParams() {
    }

    protected function addSubhandlers() {
    }

    protected function addParam (Gpf_RestApi_Param_Base $param, $name = null, $description = null) {
        if ($name === null) {
            $this->params[] = array($param, $description);
        } else {
            $this->params[$name] = array($param, $description);
        }
    }

    /**
     * @return Gpf_RestApi_Param_Base
     */
    protected function getParam ($id) {
        if (array_key_exists($id, $this->params)) {
            return $this->params[$id][0];
        }
        throw new Gpf_Exception('Param with id ' . $id . ' does not exist in ' . get_class($this));
    }

    protected function getParamValueAndUpdateHandlerString(Gpf_RestApi_Param_Base $param) {
        $value = $param->getValue($this->data->getHandleString());
        $this->data->setHandleString($param->getHandlerStringWithoutMe($this->data->getHandleString()));
        return $value;
    }

    public abstract function getName();

    protected abstract function handleSelf();

    protected function handleSpecificHandlers($prefix) {
        Gpf_Log::debug('handling handlers ' . $prefix);
        foreach ($this->subHandlers as $name => $subHandler) {
            if (strpos($name, $prefix) !== false && strtolower($subHandler->getName() == strtolower($this->nextHandlerName))) {
                $subHandler->setData($this->data);
                $result = $subHandler->handle();
                if ($result !== null && is_object($result)) {
                    return $result;
                }
            }
        }
    }

    protected function handleSubHandlers() {
        if ($this->nextHandlerName == null) {
            return;
        }
        return $this->handleSpecificHandlers(self::SUB_HANDLER);
    }

    /**
     * @desc used to gen api doc
     */
    public function getSubHandlers() {
        return $this->subHandlers;
    }

    /**
     * @desc used to gen api doc
     */
    public function getParams() {
        return $this->params;
    }

    protected function addSubHandler(Gpf_RestApi_Handler $handler) {
        $handlerName = $handler->getName() . get_class($handler) . self::SUB_HANDLER;
        $this->subHandlers[$handlerName] = $handler;
    }

    public function handle() {
        Gpf_Log::debug('Handling ' . get_class($this));
        if ($this->allowedRoleTypeId !== null && $this->allowedRoleTypeId != Gpf_Session::getAuthUser()->getRoleTypeId()) {
            return;
        }
        $result = $this->handleSelf();
        if ($result !== null && is_object($result)) {
            return $result;
        }
        $result = $this->handleSubHandlers();
        if ($result !== null && is_object($result)) {
            return $result;
        }
        return null;
    }

    public function setData(Gpf_RestApi_Data $data) {
        $this->data = $data;
    }
}
?>
