<?php
/**
 *   @copyright Copyright (c) 2012 Quality Unit s.r.o.
 *   @author Juraj Simon
 *   @package LiveAgentPro
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.qualityunit.com/licenses/license
 */

abstract class Gpf_RestApi_LatestHandler extends Gpf_RestApi_Handler {
     
    const MANDATORY = 'mandatory';
    const TYPE = 'type';
    const DESCRIPTION = 'desc';
    const DEFINITION = 'def';
     
    const TYPE_DATE_TIME = Gpf_RestApi_Response_DateTimeField::TYPE;
    const TYPE_TEXT = Gpf_RestApi_Response_TextField::TYPE;
    const TYPE_CONSTLIST = Gpf_RestApi_Response_ConstListField::TYPE;

    protected $dataPramsList = array();
     
    protected function addDataParam($name, $type, $mandatory = false, $description = null, $definition = null) {
        $this->dataPramsList[$name] = array(
                self::TYPE => $type,
                self::MANDATORY => $mandatory,
                self::DESCRIPTION => $description,
                self::DEFINITION => $definition
        );
    }
     
    /**
     * @throws Gpf_RestApi_Param_WrongFormatException
     */
    protected function checkDataParams() {
        foreach ($this->dataPramsList as $paramName => $dataParam) {
            $isMandatory = $dataParam[self::MANDATORY];
            $type = $dataParam[self::TYPE];
            if ($this->data->getParam($paramName) == null && $isMandatory === true) {
                throw new Gpf_RestApi_ProcessingException(500, $this->_('Parameter %s not found or empty.', $paramName));
            }
            if ($this->data->getParam($paramName)!= '' && !$this->isValid($this->data->getParam($paramName), $type)) {
                throw new Gpf_RestApi_ProcessingException(500, $this->_('Parameter %s has invalid format.', $paramName));
            }
            if ($type == self::TYPE_CONSTLIST && $isMandatory === true &&
                    !in_array($this->data->getParam($paramName), $dataParam[self::DEFINITION]->getAllconsts())) {
                throw new Gpf_RestApi_ProcessingException(500, $this->_('Parameter %s has invalid value.', $paramName));
            }
            if ($type == self::TYPE_CONSTLIST && $isMandatory === false && $this->data->getParam($paramName)!= '' &&
                    !in_array($this->data->getParam($paramName), $dataParam[self::DEFINITION]->getAllconsts())) {
                throw new Gpf_RestApi_ProcessingException(500, $this->_('Parameter %s has invalid value.', $paramName));
            }
        }
    }
     
    private function isValid($value, $type) {
        switch ($type) {
            case self::TYPE_DATE_TIME:
                return $this->dateIsValid($value);
        }
        return true;
    }
     
    private function dateIsValid($date) {
        return preg_match('/^\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2}$/', $date);
    }
     
    /**
     * @desc used for api doc gen
     */
    public function getDataParamsList() {
        return $this->dataPramsList;
    }
}
?>
