<?php
/**
 *   @copyright Copyright (c) 2012 Quality Unit s.r.o.
 *   @author Juraj Simon
 *   @package LiveAgentPro
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.qualityunit.com/licenses/license
 */

abstract class Gpf_RestApi_Response_Base implements Gpf_RestApi_Response{
    protected $fields = array();

    public function toArray() {
        $response = array();
        foreach ($this->fields as $field) {
            $fieldInArray = $field->toArray();
            foreach ($fieldInArray as $fName => $fValue) {
                $response[$fName] = $fValue;
            }
        }
        return array('response' => $response);
    }

    /**
     * @desc used for doc gen
     */
    public function getFields() {
        return $this->fields;
    }

    protected function addConstList (Gpf_RestApi_Response_ConstListField $field) {
        $this->fields[$field->getName()] = $field;
    }

    protected function addGroupField (Gpf_RestApi_Response_GroupField $field) {
        $this->fields[$field->getName()] = $field;
    }

    protected function addTextField (Gpf_RestApi_Response_TextField $field) {
        $this->fields[$field->getName()] = $field;
    }

    protected function addDateTimeField (Gpf_RestApi_Response_DateTimeField $field) {
        $this->fields[$field->getName()] = $field;
    }

    protected function addIntField (Gpf_RestApi_Response_IntField $field) {
        $this->fields[$field->getName()] = $field;
    }

    protected function addRecordsetField (Gpf_RestApi_Response_RecordsetField $field) {
        $this->fields[$field->getName()] = $field;
    }

    public function get($name) {
        foreach ($this->fields as $field) {
            if (strtolower($field->getName()) == strtolower($name)) {
                return $field->getValue();
            }
        }
        throw new Gpf_Exception('Response object ' . get_class($this) . ' does not have field named ' . $name);
    }

    public function set($name, $value) {
        foreach ($this->fields as $field) {
            if ($field->getType() == Gpf_RestApi_Response_GroupField::TYPE) {
                try {
                    $field->set($name, $value);
                    return;
                } catch (Gpf_RestApi_Response_FieldNotFoundException $e) {
                }
            }
            if (strtolower($field->getName()) == strtolower($name)) {
                if ($field->getType() == Gpf_RestApi_Response_GroupField::TYPE) {
                    throw new Gpf_Exception('You can not set group field directly');
                }
                $field->setValue($value);
                return;
            }
        }
        throw new Gpf_Exception('Response object ' . get_class($this) . ' does not have field named ' . $name);
    }
}
?>
