<?php
/**
 *   @copyright Copyright (c) 2012 Quality Unit s.r.o.
 *   @author Juraj Simon
 *   @package LiveAgentPro
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.qualityunit.com/licenses/license
 */

class Gpf_RestApi_Response_GroupField extends Gpf_RestApi_Response_FieldBase implements Gpf_RestApi_Response {
    const TYPE = 'group';

    protected $fields = array();

    public function getType() {
        return self::TYPE;
    }

    public function toArray() {
        $response = array();
        foreach ($this->fields as $field) {
            $fieldInArray = $field->toArray();
            foreach ($fieldInArray as $fName => $fValue) {
                $response[$fName] = $fValue;
            }
        }
        return array($this->name => $response);
    }
    
    public function getFields() {
        return $this->fields;
    }
    
    public function set($name, $value) {
        foreach ($this->fields as $field) {
            if ($field->getType() == Gpf_RestApi_Response_GroupField::TYPE) {
                $field->set($name, $value);
            }
            if (strtolower($field->getName()) == strtolower($name)) {
                if ($field->getType() == Gpf_RestApi_Response_GroupField::TYPE) {
                    throw new Gpf_Exception('You can not set group field directly');
                }
                $field->setValue($value);
                return;
            }
        }
        throw new Gpf_RestApi_Response_FieldNotFoundException('Response object ' . get_class($this) . ' does not have field named ' . $name);
    }

    public function addTextField (Gpf_RestApi_Response_TextField $field) {
        $this->fields[$field->getName()] = $field;
    }
    
    public function addConstListField (Gpf_RestApi_Response_ConstListField $field) {
        $this->fields[$field->getName()] = $field;
    }
    
    public function addDateTimeField (Gpf_RestApi_Response_DateTimeField $field) {
        $this->fields[$field->getName()] = $field;
    }

    public function __construct($name, $description = null) {
        $this->name = $name;
        $this->value = null;
        $this->description = $description;
    }
}
?>
