<?php
/**
 *   @copyright Copyright (c) 2012 Quality Unit s.r.o.
 *   @author Juraj Simon
 *   @package LiveAgentPro
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.qualityunit.com/licenses/license
 */

class Gpf_RestApi_Response_RecordsetField extends Gpf_RestApi_Response_FieldBase implements Gpf_RestApi_Response {
    const TYPE = 'list';

    private $headerColumns = array();

    public function addHeaderColumn(Gpf_RestApi_Response_FieldBase $field) {
        $this->headerColumns[$field->getName()] = $field;
    }
    
    public function getHeaderColumnsAsArray() {
        $columns = array();
        foreach ($this->headerColumns as $column) {
            $columns[] = $column->getName();
        }
        return $columns;
    }

    public function getHeaderDefinition() {
        return $this->headerColumns;
    }

    public function getType() {
        return self::TYPE;
    }

    public function __construct($name, $description = null) {
        $this->name = $name;
        $this->value = null;
        $this->description = $description;
    }
        
    public function toArray() {
        $response = array();
        if ($this->value == null) {
            return $response;
        }
        foreach ($this->value as $row) {
            Gpf_Log::debug('Searching in ' . var_export($row, true) . ' for ' . var_export($this->getHeaderDefinition(), true));
            $responseRow = array();
            foreach ($this->getHeaderDefinition() as $columnName => $field) {
                Gpf_Log::debug('field ' . $field->getName());
                if (!array_key_exists($columnName, $row)) {
                    continue;
                }
                $value = $row[$columnName];
                $field->setValue($value);
                if ($field instanceof Gpf_RestApi_Response_RecordsetField) {
                    Gpf_Log::debug('We have recordset here' . $field->getName());
                    $responseRow[$columnName] = array();
                    $fieldArray = $field->toArray();
                     foreach($fieldArray[$columnName] as $name => $item) {
                         $responseRow[$columnName][] = $item;
                     }
                } else {
                    $responseRow[$columnName] = $field->getValue();
                }
            }
            $response[] = $responseRow;
        }
        return array($this->name => $response);
    }
}
?>
