<?php
/**
 *   @copyright Copyright (c) 2011 Quality Unit s.r.o.
 *   @package LiveAgentPro
 *   @author Juraj Simon
 *
 *   Licensed under the Quality Unit, s.r.o. Dual License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.qualityunit.com/licenses/gpf
 */

class Gpf_RestApi_VersionParser extends Gpf_Object {
    public function getVersion($handlerClassPrefix, $handlerClassSuffix) {
        if (array_key_exists('v', $_REQUEST)) {
            return $_REQUEST['v'];
        }
        if (array_key_exists('V', $_REQUEST)) {
            return $_REQUEST['V'];
        }
        $version = $this->getVersionFromHandler();
        if ($version != '') {
            return str_replace(array('V', 'v'), '', $version);
        }
        return $this->getLatestVersion($handlerClassPrefix, $handlerClassSuffix);
    }

    private function getVersionFromHandler() {
        $matchCount = preg_match('/(V[0-9]+)\/.*/i', $this->getHandler(), $matches);
        if ($matchCount == 0) {
            return null;
        }
        return $matches[1];
    }

    private function getHandler() {
        if (array_key_exists('handler', $_REQUEST)) {
            return $_REQUEST['handler'];
        }
        return '';
    }

    private function getLatestVersion($handlerClassPrefix, $handlerClassSuffix) {
        for ($i=2; $i<=999; $i++) {
            try {
                $class = $handlerClassPrefix.$i.$handlerClassSuffix;
                if (!class_exists($class)) {
                    break;
                }
                $handler = new $class;
            } catch (Gpf_ClassNotDefined $e) {
                break;
            }
        }
        return $i - 1;
    }
}
?>
