<?php
/**
 *   @copyright Copyright (c) 2007 Quality Unit s.r.o.
 *   @author Maros Galik
 *   @package PostAffiliatePro
 *   @since Version 1.0.0
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.postaffiliatepro.com/licenses/license
 *
 */

/**
 * @package PostAffiliatePro
 */
class Pap_Common_AuthUser extends Gpf_Db_AuthUser {

    private $roleId;

    public function setRoleId($roleId) {
        $this->roleId = $roleId;
    }

    protected function beforeSaveCheck() {
        try {
            parent::beforeSaveCheck();
        } catch (Gpf_DbEngine_Row_PasswordConstraintException $e) {
            if(!$this->isMasterMerchant($this->get(Gpf_Db_Table_AuthUsers::ID))) {
                throw $e;
            }
        }
    }

    private function isMasterMerchant($authId) {
        return $this->getRoleId($authId) == Pap_Application::DEFAULT_ROLE_MERCHANT;
    }

    private function getRoleId($authId) {
        if ($this->roleId != null) {
            return $this->roleId;
        }

        if ($authId == null) {
            return null;
        }
        $select = new Gpf_SqlBuilder_SelectBuilder();
        $select->select->add(Gpf_Db_Table_Users::ROLEID);
        $select->from->add(Gpf_Db_Table_AuthUsers::getName(), 'au');
        $select->from->addInnerJoin(Gpf_Db_Table_Users::getName(), 'u', 'au.' . Gpf_Db_Table_AuthUsers::ID . '=u.' . Gpf_Db_Table_Users::AUTHID);
        $select->where->add('au.' . Gpf_Db_Table_AuthUsers::ID, '=', $authId);

        try {
            return $select->getOneRow()->get(Gpf_Db_Table_Users::ROLEID);
        } catch (Gpf_DbEngine_NoRowException $e) {
            return null;
        }
    }
}

?>
