<?php
/**
 *   @copyright Copyright (c) 2007 Quality Unit s.r.o.
 *   @package PostAffiliatePro
 *   @author Michal Bebjak
 *   @since Version 1.0.0
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.postaffiliatepro.com/licenses/license
 *
 */

/**
 * @package PostAffiliatePro
 */
class Pap_Common_Banner_Image extends Pap_Common_Banner {

    protected function getBannerCode(Pap_Common_User $user, $flags, $data1 = '', $data2 = '', $isCached = false) {
        $imageUrl = $this->getImageUrl();
        $imageUrl = $this->changeActualDomainUrlHttpToHttps($imageUrl);
        $description = $this->getDescription($user);
        $format = $this->getBannerFormat();

        $format = Pap_Common_UserFields::replaceCustomConstantInText('image_src', $imageUrl, $format);
        $format = Pap_Common_UserFields::replaceCustomConstantInText('image_name', basename($imageUrl), $format);
        $format = Pap_Common_UserFields::replaceCustomConstantInText('alt', $description, $format);
        $format = Pap_Common_UserFields::replaceCustomConstantInText(Pap_Db_Table_Banners::SEOSTRING, $this->getSeoString(), $format);

        $format = $this->replaceUrlConstants($format, $user, $flags, '', $data1, $data2, $isCached);
        $format = $this->replaceUserConstants($format, $user);
        $format = $this->replaceWidthHeightConstants($format, $flags);

        return $format;
    }

    public function getImageUrl() {
        return $this->getData(1);
    }

    public function setImageUrl($url) {
        $this->setData(1, $url);
    }

    public static function getBannerFormat() {
        return Gpf_Settings::get(Pap_Settings::IMAGE_BANNER_FORMAT_SETTING_NAME);
    }

    protected function setDetectedSize($size){
        $this->setData(4, $size);
    }

    protected function getDetectedSize(){
        return $this->getData(4);
    }

    protected function beforeSaveAction() {
        $this->updateImageFileName();
        $this->detectImageSize();
    }

    protected function updateImageFileName($dataNumber = null) {
        if ($dataNumber != null && is_int($dataNumber)) {
            $fileUrl = $this->getData($dataNumber);
        } else {
            $fileUrl = $this->getImageUrl();
        }

        $newFileUrl = $this->updateFileName($fileUrl, $dataNumber);

        if ($newFileUrl == '') {
            return;
        }

        if ($dataNumber != null && is_int($dataNumber)) {
            $this->setData($dataNumber, $newFileUrl);
        } else {
            $this->setImageUrl($newFileUrl);
        }
    }

    private function changeActualDomainUrlHttpToHttps($url) {
        if((@$_SERVER['HTTPS'] == 'on') && (strpos($url, 'https') === false)){
            $url = str_ireplace('http'.substr(Gpf_Paths::getInstance()->getFullDomainUrl(),strpos(Gpf_Paths::getInstance()->getFullDomainUrl(), ':')), 'https'.substr(Gpf_Paths::getInstance()->getFullDomainUrl(),strpos(Gpf_Paths::getInstance()->getFullDomainUrl(), ':')), $url);
        }
        return $url;
    }

    private function detectImageSize(){
        $image = $this->encodeImageUrlForGetImageSize($this->getImageUrl());
        if (($size = @getimagesize($image)) !== false) {
            $this->setDetectedSize($size[0].'x'.$size[1]);
        } else {
            $this->setDetectedSize(Gpf_DbEngine_Row::NULL);
        }
    }
    
    /**
     * @return String
     */
    protected function encodeImageUrlForGetImageSize($url) {
    	$url = urldecode($url);
        $url = str_replace(' ', '%20', $url);
        return $url;
    }

    protected function setUndefinedSize(){
        if($this->getDetectedSize() != null){
            $size = explode('x',$this->getDetectedSize());
            $this->width = $size[0];
            $this->height = $size[1];
        }
    }
}

?>
