<?php
/**
 *   @copyright Copyright (c) 2007 Quality Unit s.r.o.
 *   @package PostAffiliatePro
 *   @author Milos Jancovic
 *   @since Version 1.0.0
 *   $Id: Banner.class.php 16622 2008-03-21 09:39:50Z aharsani $
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.postaffiliatepro.com/licenses/license
 *
 */

/**
 * @package PostAffiliatePro
 */
abstract class Pap_Common_Rule extends Gpf_Object {
    const DATE_ACTUAL_MONTH = 'TM';
    const DATE_ACTUAL_YEAR = 'TY';
    const DATE_ALL_UNPAID_COMMISSIONS = 'AUC';
    const DATE_LAST_WEEK = 'LW';
    const DATE_LAST_TWO_WEEKS = 'L2W';
    const DATE_LAST_MONTH = 'LM';
    const DATE_ALL_TIME = 'AT';
    const DATE_SINCE_DAY_OF_LAST_MONTH = 'SD';
    const DATE_LAST_YEAR = 'LY';
    const DATE_LAST_7DAYS = 'L7D';
    const DATE_LAST_30DAYS = 'L30D';
    const DATE_LAST_90DAYS = 'L90D';

    const BONUS_TYPE_AMOUNT = '$';
    const BONUS_TYPE_PERCENTAGE = '%';


    /**
     * @var Pap_Model_Rule
     */
    private $modelRule;
    private $condition = null;

    public function __construct(Pap_Model_Rule $modelRule) {
        $this->modelRule = $modelRule;
    }
    
    /**
     * 
     * @return Pap_Model_Rule
     */
    public function getModel() {
        return $this->modelRule;
    }

    /**
     * @return Pap_Common_Condition
     */
    private function createCondition() {
        return Pap_Common_Condition::create($this);
    }
    
	/**
     * @return Pap_Db_Transaction
     */
    public function getTransaction() {
        return $this->getModel()->getTransaction();
    }

    public function setTransactionsWhere(Gpf_SqlBuilder_WhereClause $where, $transactionTableAlias = 't') {
        $this->setSqlDateRange($where, $transactionTableAlias);
        $where->add($transactionTableAlias . '.'. Pap_Db_Table_Rules::CAMPAIGN_ID, '=', $this->getCampaignId());
    }
    
    public function getCampaignId() {
        return $this->modelRule->getCampaignId();
    }
    
    public function getId() {
        return $this->modelRule->getId();
    }
    
    public function getWhat() {
        return $this->modelRule->getWhat();
    }
    
    public function getStatus(){
        return $this->modelRule->getStatus();
    }
    
    public function getDate(){
        return $this->modelRule->getDate();
    }
    
    public function getSince(){
        return $this->modelRule->getSince();
    }
    
    public function getEquation(){
        return $this->modelRule->getEquation();
    }
    
    public function getEquationValue1() {
        return $this->modelRule->getEquationValue1();
    }
    
    public function getEquationValue2(){
        return $this->modelRule->getEquationValue2();
    }
    
    public function getAction(){
        return $this->modelRule->getAction();
    }
    
    private function setSqlDateRange(Gpf_SqlBuilder_WhereClause $where, $transactionTableAlias = 't') {
        if ($this->getDate() == self::DATE_ALL_UNPAID_COMMISSIONS) {
            $where->add($transactionTableAlias.'.'.Pap_Db_Table_Transactions::PAYOUT_STATUS, '=', Pap_Common_Constants::PSTATUS_UNPAID);
        }

        //TODO: implement - use dateRange object
        //$where->addDateRange($transactionTableAlias.'.'.Pap_Db_Table_Transactions::DATE_INSERTED, $dateRange);
        
        $dateRange = $this->getDateRange(new Gpf_DateTime());
        if($dateRange->isAllTime()) {
            return;
        }
        $where->add($transactionTableAlias.'.'.Pap_Db_Table_Transactions::DATE_INSERTED, '>=', $dateRange->getFrom()->toDateTime());
        $where->add($transactionTableAlias.'.'.Pap_Db_Table_Transactions::DATE_INSERTED, '<=', $dateRange->getTo()->toDateTime());
    }    

    /**
     * @return Pap_Common_Condition
     */
    public function getCondition() {
        if ($this->condition === null) {
            $this->condition = $this->createCondition();
        }
        return $this->condition;
    }

    public function isConditionValid() {
        return $this->getCondition()->isValid();
    }

    public function executeAction() {
        if ($this->isConditionValid()) {
            $this->getActionObject()->execute();
        } else {
            $logger = Pap_Logger::create(Pap_Common_Constants::TYPE_ACTION);
            $logger->log('Condition of rule is not valid - skipping', Gpf_Log::DEBUG);
        }
    }

    abstract protected function getActionObject();

    /**
     *
     * @return Gpf_DateTime_Range
     */
    public function getDateRange(Gpf_DateTime $now = null) {
        if($now === null) {
            $now = new Gpf_DateTime();
        }
        if ($this->getDate() == self::DATE_ALL_TIME
        || $this->getDate() == self::DATE_ALL_UNPAID_COMMISSIONS) {
            $range = new Gpf_DateTime_Range();
            return $range;
        }

        if ($this->getDate() == self::DATE_SINCE_DAY_OF_LAST_MONTH) {
            $from = $now->getMonthStart();
            $from->addMonth(-1);
            $from->addDay($this->getSince() - 1);
            $to = $now->getMonthStart();
            $to->addDay(-1);
            return new Gpf_DateTime_Range($from, $to);
        }
        $filter = new Gpf_SqlBuilder_Filter();
        $result = $filter->decodeDatePreset($this->getDate());
        return new Gpf_DateTime_Range(new Gpf_DateTime($result['dateFrom']), new Gpf_DateTime($result['dateTo']));
    }

    public function getString() {
        return $this->getCondition()->getText() . $this->getActionObject()->getString();
    }
}

?>
