<?php
/**
 *   @copyright Copyright (c) 2007 Quality Unit s.r.o.
 *   @package PostAffiliatePro
 *   @author Maros Fric
 *   @since Version 1.0.0
 *   $Id: ClickImpression.class.php 43677 2013-02-08 12:49:10Z vzeman $
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.postaffiliatepro.com/licenses/license
 *
 */

/**
 * @package PostAffiliatePro
 */
abstract class Pap_Db_ClickImpression extends Gpf_DbEngine_Row {
	
	const STATUS_RAW = 'R';
	const STATUS_UNIQUE = 'U';
	const STATUS_DECLINED = 'D';
	
    private $rawCountChange = 0;
    private $uniqueCountChange = 0;

    public function addRawCountChage($count = 1) {
        $this->rawCountChange = $this->rawCountChange + $count;
    }

    public function addUniqueCountChage($count = 1) {
        $this->uniqueCountChange = $this->uniqueCountChange + $count;
    }

    protected function init() {
        $this->setAccountId(Gpf_Db_Account::DEFAULT_ACCOUNT_ID);
        $this->setBannerId('');
        $this->setCampaignId('');
        $this->setUserId('');
        $this->setParentBannerId('');
        $this->setCountryCode('');
        $this->setChannel('');
        $this->setData1('');
        $this->setData2('');
        parent::init();
    }
    
    public function setRaw($value) {
        $this->set(Pap_Db_Table_ClicksImpressions::RAW, $value);
    }

    public function setUnique($value) {
        $this->set(Pap_Db_Table_ClicksImpressions::UNIQUE, $value);
    }
    
    public function getRaw() {
        return $this->get(Pap_Db_Table_ClicksImpressions::RAW);
    }

    public function getUnique() {
        return $this->get(Pap_Db_Table_ClicksImpressions::UNIQUE);
    }

    public function setTime($time) {
        $this->set(Pap_Stats_Table::DATEINSERTED, $time);
    }
        
    public function getTime() {
        return $this->get(Pap_Stats_Table::DATEINSERTED);
    }

    public function setAccountId($id) {
        $this->set(Pap_Stats_Table::ACCOUNTID, $id);
    }

    public function getAccountId() {
        return $this->get(Pap_Stats_Table::ACCOUNTID);
    }

    public function getUserId() {
        return $this->get(Pap_Stats_Table::USERID);
    }
    
    public function setUserId($id) {
        $this->set(Pap_Stats_Table::USERID, $id);
    }
    
    public function getCampaignId() {
        return $this->get(Pap_Stats_Table::CAMPAIGNID);
    }

    public function setCampaignId($id) {
        $this->set(Pap_Stats_Table::CAMPAIGNID, $id);
    }
    
    public function getBannerId() {
        return $this->get(Pap_Stats_Table::BANNERID);
    }

    public function setBannerId($id) {
        $this->set(Pap_Stats_Table::BANNERID, $id);
    }

    public function setParentBannerId($id) {
        $this->set(Pap_Stats_Table::PARENTBANNERID, $id);
    }

    public function setCountryCode($code) {
        $this->set(Pap_Stats_Table::COUNTRYCODE, $code);
    }
    
    public function getCountryCode() {
        return $this->get(Pap_Stats_Table::COUNTRYCODE);
    }

    public function setData1($value) {
        $this->set(Pap_Stats_Table::CDATA1, $value);
    }

    public function setData2($value) {
        $this->set(Pap_Stats_Table::CDATA2, $value);
    }
    
    public function setChannel($value) {
        $this->set(Pap_Stats_Table::CHANNEL, $value);
    }

    public function addRaw() {
        $this->addRawCount(1);
    }

    public function addRawCount($count) {
        $this->setRaw($this->getRaw()+$count);
    }
        
    public function addUnique() {
        $this->addUniqueCount(1);
    }
    
    public function addUniqueCount($count) {
        $this->setUnique($this->getUnique()+$count);
    }
    
    public function mergeWith(Pap_Db_ClickImpression $clickImpression) {
        $this->addRawCount($clickImpression->getRaw());
        $this->addUniqueCount($clickImpression->getUnique());
    }

    public function insertEmptyCountsIfNotExists() {
        $innerSelect = new Gpf_SqlBuilder_SelectBuilder();
        $innerSelect->select->add('count(*)', 'count');
        $innerSelect->from->add($this->getTable()->name());
        $innerSelect->where->add(Pap_Stats_Table::ACCOUNTID, '=', $this->getAccountId());
        $innerSelect->where->add(Pap_Db_Table_RawClicks::USERID, '=', $this->getUserId());
        $innerSelect->where->add(Pap_Db_Table_RawClicks::CAMPAIGNID, '=', $this->getCampaignId());
        $innerSelect->where->add(Pap_Db_Table_RawClicks::BANNERID, '=', $this->getBannerId());
        $innerSelect->where->add(Pap_Db_Table_RawClicks::PARENTBANNERID, '=', $this->get(Pap_Stats_Table::PARENTBANNERID));
        $innerSelect->where->add(Pap_Db_Table_RawClicks::COUNTRYCODE, '=', $this->getCountryCode());
        $innerSelect->where->add(Pap_Stats_Table::CDATA1, '=', $this->get(Pap_Stats_Table::CDATA1));
        $innerSelect->where->add(Pap_Stats_Table::CDATA2, '=', $this->get(Pap_Stats_Table::CDATA2));
        $innerSelect->where->add(Pap_Db_Table_RawClicks::CHANNEL, '=', $this->get(Pap_Stats_Table::CHANNEL));
        $innerSelect->where->add(Pap_Stats_Table::DATEINSERTED, '=', $this->getTime());

        $select = new Gpf_SqlBuilder_SelectBuilder();
        $select->select->add('\''.$this->getAccountId().'\'', Pap_Stats_Table::ACCOUNTID);
        $select->select->add('\''.$this->getUserId().'\'', Pap_Db_Table_RawClicks::USERID);
        $select->select->add('\''.$this->getCampaignId().'\'', Pap_Db_Table_RawClicks::CAMPAIGNID);
        $select->select->add('\''.$this->getBannerId().'\'', Pap_Db_Table_RawClicks::BANNERID);
        $select->select->add('\''.$this->get(Pap_Stats_Table::PARENTBANNERID).'\'', Pap_Db_Table_RawClicks::PARENTBANNERID);
        $select->select->add('\''.$this->getCountryCode().'\'', Pap_Db_Table_RawClicks::COUNTRYCODE);
        $select->select->add('\''.$this->get(Pap_Stats_Table::CDATA1).'\'', Pap_Stats_Table::CDATA1);
        $select->select->add('\''.$this->get(Pap_Stats_Table::CDATA2).'\'', Pap_Stats_Table::CDATA2);
        $select->select->add('\''.$this->get(Pap_Stats_Table::CHANNEL).'\'', Pap_Db_Table_RawClicks::CHANNEL);
        $select->select->add('\''.$this->getTime().'\'', Pap_Stats_Table::DATEINSERTED);
        $select->from->addSubselect($innerSelect, 'innerSelect');
        $select->where->add('innerSelect.count', '=', 0);

        $insert = new Gpf_SqlBuilder_InsertBuilder();
        $insert->setTable($this->getTable());
        $insert->fromSelect($select);
        $insert->execute();
    }

    protected function addUpdateCountColumns(Gpf_SqlBuilder_UpdateBuilder $update) {
        $update->set->add(Pap_Db_Table_Clicks::RAW, Pap_Db_Table_Clicks::RAW . '+' . $this->rawCountChange, false);
        $update->set->add(Pap_Db_Table_Clicks::UNIQUE, Pap_Db_Table_Clicks::UNIQUE . '+' . $this->uniqueCountChange, false);
        return $update;
    }

    /**
     * @throws Gpf_DbEngine_NoRowException
     */
    public function updateCountChanges() {
        $update = new Gpf_SqlBuilder_UpdateBuilder();
        $update = $this->addUpdateCountColumns($update);
        $update->from->add($this->getTable()->name());
        $update->where->add(Pap_Stats_Table::ACCOUNTID, '=', $this->getAccountId());
        $update->where->add(Pap_Db_Table_RawClicks::USERID, '=', $this->getUserId());
        $update->where->add(Pap_Db_Table_RawClicks::CAMPAIGNID, '=', $this->getCampaignId());
        $update->where->add(Pap_Db_Table_RawClicks::BANNERID, '=', $this->getBannerId());
        $update->where->add(Pap_Db_Table_RawClicks::PARENTBANNERID, '=', $this->get(Pap_Stats_Table::PARENTBANNERID));
        $update->where->add(Pap_Db_Table_RawClicks::COUNTRYCODE, '=', $this->getCountryCode());
        $update->where->add(Pap_Stats_Table::CDATA1, '=', $this->get(Pap_Stats_Table::CDATA1));
        $update->where->add(Pap_Stats_Table::CDATA2, '=', $this->get(Pap_Stats_Table::CDATA2));
        $update->where->add(Pap_Stats_Table::CHANNEL, '=', $this->get(Pap_Stats_Table::CHANNEL));
        $update->where->add(Pap_Stats_Table::DATEINSERTED, '=', $this->getTime());
        $update->limit->set(1);

        $statement = $update->update();

        if ($statement->affectedRows() == 0) {
           throw new Gpf_DbEngine_NoRowException($update);
        }
    }
}
?>
