<?php
/**
 *   @copyright Copyright (c) 2007 Quality Unit s.r.o.
 *   @author Milos Jancovic
 *   @package PostAffiliatePro
 *   @since Version 1.0.0
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.postaffiliatepro.com/licenses/license
 *
 */

/**
 * @package PostAffiliatePro plugins
 */
class Pap_Features_AffiliateNetwork_AccountStatusCheck extends Gpf_Plugins_Handler {

    /**
     * @var Pap_Features_AffiliateNetwork_AccountStatusCheck
     */
    private static $instance;

    /**
     * @return Pap_Features_AffiliateNetwork_AccountStatusCheck
     */
    public static function getHandlerInstance() {
        if (self::$instance == null) {
            self::$instance = new Pap_Features_AffiliateNetwork_AccountStatusCheck();
        }
        return self::$instance;
    }

    protected function __construct() {
    }

    private $accounts = array();

    public function buildCampaignsWhere(Pap_Affiliates_Promo_SelectBuilderCompoundFilter $compound) {
        $selectBuilder = $compound->getSelectBuilder();
        $selectBuilder->from->addInnerJoin(Gpf_Db_Table_Accounts::getName(), 'ac',
            'ac.'.Gpf_Db_Table_Accounts::ID .'=c.'.Pap_Db_Table_Campaigns::ACCOUNTID);
        $selectBuilder->where->add('ac.'.Gpf_Db_Table_Accounts::STATUS, '=', Gpf_Db_Account::APPROVED);
    }

    public function buildBannersWhere(Gpf_SqlBuilder_SelectBuilder $selectBuilder) {
        $selectBuilder->from->addInnerJoin(Gpf_Db_Table_Accounts::getName(), 'ac',
            'ac.'.Gpf_Db_Table_Accounts::ID .'=b.'.Pap_Db_Table_Banners::ACCOUNT_ID);
        $selectBuilder->where->add('ac.'.Gpf_Db_Table_Accounts::STATUS, '=', Gpf_Db_Account::APPROVED);
    }

    public function checkAccountStatus(Pap_Contexts_Tracking $context) {
        $accountId = $context->getAccountId();
        $context->debug('Checking account '.$accountId);
        if (!$context->getDoTrackerSave()) {
            $context->debug('Saving already stopped. SKIPPING');
            return;
        }
        if(!array_key_exists($accountId, $this->accounts)) {
            $this->accounts[$accountId] = false;
            try {
                $account = $this->loadAccountFromDb($accountId);
                if ($account->getStatus() == Gpf_Db_Account::APPROVED) {
                    $context->debug('Account approved.');
                    $this->accounts[$accountId] = true;
                } else {
                    $context->debug('Account is not approved. STOPPING ');
                }
            } catch (Gpf_Exception $e) {
                $context->debug('Account does not exist. STOPPING ');
            }
        }
        $context->setDoTrackerSave($this->accounts[$accountId]);
    }

    /**
     * @throws Gpf_Exception
     * @return Pap_Account
     */
    protected function loadAccountFromDb($accountId) {
        $account = new Pap_Account();
        $account->setId($accountId);
        $account->load();
        return $account;
    }
}
?>