<?php
/**
 *   @copyright Copyright (c) 2007 Quality Unit s.r.o.
 *   @author Milos Jancovic
 *   @package PostAffiliatePro
 *   @since Version 1.0.0
 *   $Id: ActionParser.class.php 16620 2008-03-21 09:21:07Z aharsani $
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.postaffiliatepro.com/licenses/license
 *
 */

/**
 * @package PostAffiliatePro
 */
class Pap_Features_AffiliateNetwork_AccountVariables extends Pap_Features_AffiliateNetwork_Variables {

    /**
     * @var Pap_Account
     */
    private $account;

    public function setAccount(Pap_Account $account) {
        $this->account = $account;
    }

    protected function initVariables() {
        $this->createVariable('accountid', $this->_('Account ID'));
        $this->createVariable('accountname', $this->_('Account name'));
        $this->createVariable('accountstatus', $this->_('Account status'));
        $this->createVariable('accountemail', $this->_('Account email'));
        $this->createVariable('accountcreated', $this->_('Account created'));
        $this->createVariable('agreement', $this->_('Agreement'));
        $this->createDynamicFieldVariables();
    }

    /**
     * @return array $key = id, $value = value
     */
    public function toValues() {
        if (is_null($this->account)) {
            return array();
        }
        $values = array();
        $values['accountid'] = $this->account->getId();
        $values['accountname'] = $this->account->getName();
        $values['accountstatus'] = Pap_Common_Constants::getStatusAsText($this->account->getStatus());
        $values['accountemail'] = $this->account->getEmail();
        $date = new Gpf_DateTime($this->account->getDateinserted());
        $values['accountcreated'] = $date->toLocaleDate();
        $values['agreement'] = $this->account->get('agreement');
        $values = $this->loadDynamicFieldValues($values);
        return $values;
    }

    private function loadDynamicFieldValues(array $values) {
        $fields = $this->createAccountFieldsForm()->loadFields();
        foreach ($fields as $record){
            $code = $record->get(Gpf_Db_Table_FormFields::CODE);
            $values[$code] = $this->account->get($code);
        }
        return $values;
    }

    private function createDynamicFieldVariables() {
        $fields = $this->createAccountFieldsForm()->loadFields();
        foreach ($fields as $record){
            $this->createVariable($record->get(Gpf_Db_Table_FormFields::CODE), $record->get(Gpf_Db_Table_FormFields::NAME));
        }
    }

    protected function createAccountFieldsForm() {
        return new Pap_Features_AffiliateNetwork_AccountFieldsForm();
    }
}
?>