<?php
/**
 *   @copyright Copyright (c) 2007 Quality Unit s.r.o.
 *   @package PostAffiliatePro
 *   @author Milos Jancovic
 *   @since Version 1.0.0
 *   $Id: CurrenciesGrid.class.php 20370 2008-08-29 09:11:06Z mbebjak $
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.postaffiliatepro.com/licenses/license
 *
 */

/**
 * @package PostAffiliatePro
 */
class Pap_Features_AffiliateNetwork_AccountingGrid extends Gpf_View_GridService implements Gpf_View_Grid_HasRowFilter {

    const INVOICE_PERIOD = 'invoice_period';
    const LAST_INVOICE = 'last_invoice';

    private $lastInvoiceDate;

    /**
     * @service invoice read
     * @return Gpf_Rpc_Serializable
     */
    public function getRows(Gpf_Rpc_Params $params) {
        return parent::getRows($params);
    }

    protected function initViewColumns() {
        $this->addViewColumn(Pap_Db_Table_Invoices::DATE_CREATED, $this->_("Date"), true);
        $this->addViewColumn(Pap_Db_Table_Invoices::STATUS, $this->_("Paid"), true);
        $this->addViewColumn(Pap_Db_Table_Invoices::AMOUNT, $this->_("Amount"), true);
        $this->addViewColumn(self::INVOICE_PERIOD, $this->_("Invoice period"), true);
        $this->addViewColumn(Pap_Db_Table_Invoices::MERCHANT_NOTE, $this->_("Invoice note"), true);
        $this->addViewColumn(Pap_Db_Table_Invoices::NUMBER, $this->_("Invoice number"), true);
        $this->addViewColumn(self::ACTIONS, $this->_("Actions"));
    }

    protected function initDataColumns() {
        $this->setKeyDataColumn(Pap_Db_Table_Invoices::ID);
        $this->addDataColumn(Pap_Db_Table_Invoices::DATE_CREATED);
        $this->addDataColumn(Pap_Db_Table_Invoices::DUE_DATE);
        $this->addDataColumn(Pap_Db_Table_Invoices::AMOUNT);
        $this->addDataColumn(Pap_Db_Table_Invoices::DATE_FROM);
        $this->addDataColumn(Pap_Db_Table_Invoices::DATE_TO);
        $this->addDataColumn(Pap_Db_Table_Invoices::MERCHANT_NOTE);
        $this->addDataColumn(Pap_Db_Table_Invoices::SYSTEM_NOTE);
        $this->addDataColumn(Pap_Db_Table_Invoices::STATUS);
        $this->addDataColumn(Pap_Db_Table_Invoices::DATE_PAID);
        $this->addDataColumn(Pap_Db_Table_Invoices::NUMBER);
    }

    protected function initDefaultView() {
        $this->addDefaultViewColumn(Pap_Db_Table_Invoices::DATE_CREATED, '1', 'A');
        $this->addDefaultViewColumn(Pap_Db_Table_Invoices::STATUS);
        $this->addDefaultViewColumn(Pap_Db_Table_Invoices::AMOUNT);
        $this->addDefaultViewColumn(self::INVOICE_PERIOD);
        $this->addDefaultViewColumn(Pap_Db_Table_Invoices::MERCHANT_NOTE);
        $this->addDefaultViewColumn(Pap_Db_Table_Invoices::NUMBER);
        $this->addDefaultViewColumn(self::ACTIONS);
    }

    protected function buildFrom() {
        $this->_selectBuilder->from->add(Pap_Db_Table_Invoices::getName());
    }

    protected function buildWhere() {
        $this->getAccountFilter()->addTo($this->_selectBuilder->where);
        $this->buildFilter();
        $this->setDateCreated($this->_selectBuilder->where);
    }

    /**
     * @anonym
     * @service
     * @param Gpf_Rpc_Params $params
     */
    public function getCustomFilterFields(Gpf_Rpc_Params $params) {
        $filterFields = new Gpf_View_CustomFilterFields();
        $filterFields->addNumberField(Pap_Db_Table_Invoices::AMOUNT, $this->_("Amount"));
        $filterFields->addNumberField(Pap_Db_Table_Invoices::NUMBER, $this->_("Invoice number"));
        return $filterFields->getRecordSet();
    }

    /**
     * @return Gpf_Data_RecordSet
     */
    protected function initResult() {
        $result = parent::initResult();
        $result->getHeader()->add(self::LAST_INVOICE);
        return $result;
    }

    /**
     * @param $row
     * @return DataRow or null
     */
    public function filterRow(Gpf_Data_Row $row) {
        $row->add(self::LAST_INVOICE, Gpf::NO);
        if ($row->get(Pap_Db_Table_Invoices::DATE_CREATED) == $this->getLastInvoiceDateCreated()) {
            $row->set(self::LAST_INVOICE, Gpf::YES);
        }
        return $row;
    }

    /**
     * @param $where
     */
    private function setDateCreated(Gpf_SqlBuilder_WhereClause $where) {
        foreach ($this->filters->getFilter('datetime') as $dateFilter){
            $dateFilter->setCode(Pap_Db_Table_Invoices::DATE_CREATED);
            $dateFilter->addTo($where);
        }
    }

    /**
     * @return Gpf_SqlBuilder_Filter
     */
    protected function getAccountFilter() {
        $accountFilter = $this->filters->getFilter('accountid');
        if (count($accountFilter) === 0) {
            throw new Gpf_Exception('Accountid is missing!');
        }
        return $accountFilter[0];
    }

    /**
     * @throws Gpf_DbEngine_NoRowException
     * @return Date String
     */
    protected function loadLastInvoiceDateCreated() {
        return Pap_Db_Table_Invoices::getInstance()->getLastInvoiceDateCreated($this->getAccountFilter()->getValue());
    }

    /**
     * @return String
     */
    private function getLastInvoiceDateCreated() {
        if (is_null($this->lastInvoiceDate)) {
            try {
                $this->lastInvoiceDate = $this->loadLastInvoiceDateCreated();
            } catch (Gpf_DbEngine_NoRowException $e) {
                $this->lastInvoiceDate = '';
            }
        }
        return $this->lastInvoiceDate;
    }
}
?>