<?php
/**
 *   @copyright Copyright (c) 2007 Quality Unit s.r.o.
 *   @package PostAffiliatePro
 *   @author Milos Jancovic
 *   @since Version 1.0.0
 *   $Id: CustomerForm.class.php 19572 2008-08-01 16:43:19Z mjancovic $
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.postaffiliatepro.com/licenses/license
 *
 */

/**
 * @package PostAffiliatePro
 */
class Pap_Features_AffiliateNetwork_AccountsForm extends Gpf_View_FormService {

    const USER_SETTING_APPLY_VAT = 'network_account_apply_vat';

	/**
	 * @var Pap_Merchants_User
	 */
	protected $merchant;

    /**
     * @service account add
     * @param $fields
     * @return Gpf_Rpc_Form
     */
    public function add(Gpf_Rpc_Params $params) {
        $form = parent::add($params);
        Gpf_Plugins_Engine::extensionPoint('PostAffiliateNetwork.AccountsForm.afterSave', $form);
        $this->saveInvoicingOptions($form);
        return $form;
    }

    /**
     * @service account write
     * @param $fields
     * @return Gpf_Rpc_Form
     */
    public function save(Gpf_Rpc_Params $params) {
        $form = parent::save($params);
        $form = $this->saveInvoicingOptions($form);
        Gpf_Plugins_Engine::extensionPoint('PostAffiliateNetwork.AccountsForm.afterSave', $form);
        return $form;
    }

    // @service account_vat_options write

    /**
     * @param Gpf_Rpc_Form $form
     * @return unknown_type
     */
    private function saveInvoicingOptions(Gpf_Rpc_Form $form) {
        $accountId = $this->getFieldValue($form, 'Id');
        if (Gpf_Session::getAuthUser()->hasPrivilege(Pap_Privileges::ACCOUNT_VAT_OPTIONS, Pap_Privileges::P_WRITE)) {
            Gpf_Db_Table_UserAttributes::setSetting(self::USER_SETTING_APPLY_VAT, $this->getFieldValue($form, 'applyVatInvoicing'), $accountId);
            Gpf_Db_Table_UserAttributes::setSetting(Pap_Settings::VAT_PERCENTAGE_SETTING_NAME, $this->getFieldValue($form, 'vatPercentage'), $accountId);
        } else {
            $form->addField('applyVatInvoicing', $this->getApplyVatInvoicingSetting($accountId));
            $form->addField('vatPercentage', $this->getVatPercentageInvoicingSetting($accountId));
        }
        return $form;
    }

    private function loadInvoicingOptions(Gpf_Rpc_Form $form) {
        $accountId = $this->getFieldValue($form, 'Id');
        $form->addField('applyVatInvoicing', $this->getApplyVatInvoicingSetting($accountId));
        $form->addField('vatPercentage', $this->getVatPercentageInvoicingSetting($accountId));
        return $form;
    }

    private function getApplyVatInvoicingSetting($accountId) {
        try {
            return Gpf_Db_Table_UserAttributes::getSetting(self::USER_SETTING_APPLY_VAT, $accountId);
        } catch (Gpf_DbEngine_NoRowException $e) {
            return '';
        }
    }

    private function getVatPercentageInvoicingSetting($accountId) {
        try {
            $vatPercentage = Gpf_Db_Table_UserAttributes::getSetting(Pap_Settings::VAT_PERCENTAGE_SETTING_NAME, $accountId);
            if ($vatPercentage == '') {
                $vatPercentage = Gpf_Settings::get(Pap_Settings::VAT_PERCENTAGE_SETTING_NAME);
            }
        } catch (Gpf_DbEngine_NoRowException $e) {
            $vatPercentage = Gpf_Settings::get(Pap_Settings::VAT_PERCENTAGE_SETTING_NAME);
        }
        return $vatPercentage;
    }

    private function getFieldValue(Gpf_Rpc_Form $form, $fieldName) {
        if($form->existsField($fieldName)) {
            return $form->getFieldValue($fieldName);
        }
        return '';
    }

    /**
     * @service account write
     * @return Gpf_Rpc_Action
     */
    public function saveFields(Gpf_Rpc_Params $params) {
        return parent::saveFields($params);
    }

    /**
     * @service account read
     * @param Gpf_Rpc_Params $params
     * @return Gpf_Rpc_Form
     */
    public function load(Gpf_Rpc_Params $params) {
        $form = parent::load($params);
        Gpf_Plugins_Engine::extensionPoint('PostAffiliateNetwork.AccountsForm.load', $form);
        $this->loadInvoicingOptions($form);
        return $form;
    }

    /**
     * @service account delete
     * @param $ids
     * @return Gpf_Rpc_Action
     */
    public function deleteRows(Gpf_Rpc_Params $params) {
        return parent::deleteRows($params);
    }

    /**
     * @return Pap_Account
     */
    protected function createDbRowObject() {
    	$this->merchant = new Pap_Merchants_User();
        return new Pap_Account();
    }

    /**
     * @return string
     */
    protected function getDbRowObjectName() {
        return $this->_("Account");
    }

    protected function setDefaultDbRowObjectValues(Gpf_DbEngine_RowBase $dbRow) {
        $dbRow->setApplication(Gpf_Application::getInstance()->getCode());
        $dbRow->setStatus(Pap_Common_Constants::STATUS_PENDING);
        $this->merchant->setStatus(Gpf_Db_User::PENDING);
    }

    protected function fillAdd(Gpf_Rpc_Form $form, Gpf_DbEngine_RowBase $dbRow) {
        parent::fillAdd($form, $dbRow);
        $actualDate = Gpf_Common_DateUtils::getDateTime(time());
        $this->merchant->setFirstName($form->getFieldValue('firstname'));
        $this->merchant->setLastName($form->getFieldValue('lastname'));
        $this->merchant->setUserName($dbRow->getEmail());
        $this->merchant->setRoleId(Gpf_Settings::get(Pap_Features_AffiliateNetwork_Definition::DEFAULT_NETWORK_MERCHANT_ROLE_SETTING_NAME));
        $this->merchant->setDateInserted($actualDate);
        $this->merchant->setData('6', $this->merchant->getUserName());
        $this->merchant->setPassword(Gpf_Common_String::generatePassword(8));
        if ($form->existsField(Gpf_Db_Table_Accounts::STATUS)) {
        	$this->merchant->setStatus($form->getFieldValue(Gpf_Db_Table_Accounts::STATUS));
        }
    }

    protected function checkBeforeSave(Gpf_DbEngine_RowBase $row, Gpf_Rpc_Form $form, $operationType = self::EDIT) {
    	$form->addValidator(new Gpf_Rpc_Form_Validator_EmailValidator(), 'email', $this->_('Account name'));
    	if (!$form->validate()) {
    		return false;
    	}
    	return true;
    }

    protected function addRow(Gpf_Rpc_Form $form, Gpf_DbEngine_RowBase $row) {
        parent::addRow($form, $row);
        $this->merchant->setAccountId($row->getId());
        $this->merchant->insert();

        $createDefaultCampaignPrivate = false;
        if ($form->existsField('defaultCampaignPrivate') && $form->getFieldValue('defaultCampaignPrivate') == Gpf::YES) {
            $createDefaultCampaignPrivate = true;
        }

        $this->createDefaultCampaign($row, $createDefaultCampaignPrivate);
        $this->sendEmails($row);
    }

    private function createDefaultCampaign(Pap_Account $account, $createDefaultCampaignPrivate = false) {
        if ($createDefaultCampaignPrivate) {
            Pap_Db_Table_Campaigns::createDefaultCampaign($account->getId(), $account->getName().$this->getDefaultCampaignSuffix(), null, Pap_Common_Campaign::CAMPAIGN_TYPE_ON_INVITATION);
            return;
        }
        Pap_Db_Table_Campaigns::createDefaultCampaign($account->getId(), $account->getName().$this->getDefaultCampaignSuffix());
    }
    
    private function getDefaultCampaignSuffix() {
        return $this->_(' - default campaign');
    }
    
    /**
     * @throws Gpf_Exception
     */
    protected function deleteRow(Gpf_DbEngine_RowBase $row) {
        if ($row->getPrimaryKeyValue() == Gpf_Db_Account::DEFAULT_ACCOUNT_ID) {
            throw new Gpf_Exception($this->_('Could not delete default account'));
        }

        $merchantIdsSet = $this->getMerchantIdsForAccount($row);

        $this->checkLoggedMerchant($merchantIdsSet);

        foreach ($merchantIdsSet->getAllRowsIterator() as $merchantRow) {
            $merchant = new Pap_Merchants_User();
            $merchant->setId($merchantRow->get('userid'));
            $merchant->load();
            $merchant->delete();
        }

        $row->delete();
    }

    /**
     * @throws Gpf_Exception
     * @param Gpf_SqlBuilder_SelectBuilder $merchantIdsSet
     */
    private function checkLoggedMerchant(Gpf_SqlBuilder_SelectBuilder $merchantIdsSet) {
        foreach ($merchantIdsSet->getAllRowsIterator() as $row) {
            if ($row->get('userid') == Gpf_Session::getAuthUser()->getPapUserId()) {
                throw new Gpf_Exception($this->_('Could not delete account with logged merchant'));
            }
        }
    }

    /**
     * @param Pap_Account $account
     * @return Gpf_SqlBuilder_SelectBuilder
     */
    private function getMerchantIdsForAccount(Pap_Account $account) {
        $sql = new Gpf_SqlBuilder_SelectBuilder();
        $sql->select->add('pu.'.Pap_Db_Table_Users::ID, 'userid');
        $sql->from->add(Gpf_Db_Table_AuthUsers::getName(), 'au');
        $sql->from->addInnerJoin(Gpf_Db_Table_Users::getName(), 'qu',
			'au.'.Gpf_Db_Table_AuthUsers::ID.'=qu.'.Gpf_Db_Table_Users::AUTHID);
        $sql->from->addInnerJoin(Pap_Db_Table_Users::getName(), 'pu',
			'qu.'.Gpf_Db_Table_Users::ID.'=pu.'.Pap_Db_Table_Users::ACCOUNTUSERID);
        $sql->where->add('qu.'.Gpf_Db_Table_Users::ACCOUNTID, '=', $account->getPrimaryKeyValue());
        return $sql;
    }

    /**
     * @service account write
     * @return Gpf_Rpc_Action
     */
    public function changeStatus(Gpf_Rpc_Params $params) {
        $action = new Gpf_Rpc_Action($params, $this->_("Selected account(s) status is changed"),$this->_("Failed to change status for selected account(s)"));

        foreach ($action->getIds() as $id){
            try {
                $this->changeAccountStatus($id, $action->getParam('status'), $action->getParam('account_note'), $action->getParam('system_note'));
                $action->addOk();
            } catch(Gpf_DbEngine_NoRowException $e) {
                $action->addError();
            }
        }

        return $action;
    }

    /**
     * Load default agreement between merchant and network owner
     *
     * @service
     * @anonym
     * @return Gpf_Rpc_Action
     */
    public function getDefaultAgreement(Gpf_Rpc_Params $params) {
        $data = new Gpf_Rpc_Data($params);
        try {
           $data->setValue(Pap_Features_AffiliateNetwork_Definition::MERCHANT_AGREEMENT,
            Gpf_Settings::get(Pap_Features_AffiliateNetwork_Definition::MERCHANT_AGREEMENT));
        } catch (Gpf_Exception $e) {
            $data->setValue(Pap_Features_AffiliateNetwork_Definition::MERCHANT_AGREEMENT, '');
        }
        return $data;
    }

    private function changeAccountStatus($id, $status, $accountNote, $systemNote) {
        $account = $this->createDbRowObject();
        $account->setId($id);
        $account->load();
        if ($account->getStatus() == $status) {
            return;
        }
        $account->setStatus($status);
        $account->setAccountNote($accountNote);
        $account->setSystemNote($systemNote);
        $account->save();
    }


	protected function sendEmails(Pap_Account $account) {
		if ($this->isPossibleSentBeforeApprovalEmail($account)) {
			$this->sendEmailToNewAccountBeforeApproval($account);
		}
		if ($account->getStatus() === Pap_Common_Constants::STATUS_APPROVED) {
			$this->sendApprovedEmailToNewAccount($account);
		}
	}

	protected function sendApprovedEmailToNewAccount(Pap_Account $account) {
		$this->sendEmailToNewAccount(new Pap_Features_AffiliateNetwork_Mail_AccountNewAccountSignupApproved(), $account);
	}

	protected function sendEmailToNewAccountBeforeApproval(Pap_Account $account) {
		$this->sendEmailToNewAccount(new Pap_Features_AffiliateNetwork_Mail_AccountNewAccountSignupBeforeApproval(), $account);
	}

	private function sendEmailToNewAccount(Pap_Features_AffiliateNetwork_Mail_AccountSignupBase $mail, Pap_Account $account) {
		$mail->setUser($this->merchant);
		$mail->addRecipient($account->getEmail());
		$this->sendEmail($mail, $account);
	}

	protected function sendEmail(Pap_Features_AffiliateNetwork_Mail_Account $mail, Pap_Account $account) {
		$mail->setAccount($account);
		$mail->send();
	}

	private function isPossibleSentBeforeApprovalEmail(Pap_Account $account) {
		return Gpf_Settings::get(Pap_Features_AffiliateNetwork_Definition::NOTIFICATION_ON_NEW_ACCOUNT_SIGNUP_BEFORE_APPROVAL) === Gpf::YES &&
		$account->getStatus() === Pap_Common_Constants::STATUS_PENDING;
	}
}

?>