<?php
/**
 *   @copyright Copyright (c) 2007 Quality Unit s.r.o.
 *   @package PostAffiliatePro
 *   @author Milos Jancovic
 *   @since Version 1.0.0
 *   $Id: CurrenciesGrid.class.php 20370 2008-08-29 09:11:06Z mbebjak $
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.postaffiliatepro.com/licenses/license
 *
 */

/**
 * @package PostAffiliatePro
 */
class Pap_Features_AffiliateNetwork_AccountsGrid extends Pap_Common_StatsGrid {

    const NETWORK_FEES = 'fees';
    const NETWORK_PAYMENTS = 'payments';
    const NETWORK_BALLANCE = 'ballance';
    const NETWORK_COMMISSIONS = 'netcommissions';

    public function __construct() {
        parent::__construct(Pap_Stats_Table::ACCOUNTID, 'a', Gpf_Db_Table_Accounts::ID);
    }

    /**
     * @service account read
     * @return Gpf_Rpc_Serializable
     */
    public function getRows(Gpf_Rpc_Params $params) {
        return parent::getRows($params);
    }

    protected function initViewColumns() {
        $this->addViewColumn(Gpf_Db_Table_Accounts::ID, $this->_("ID"), true);
        $this->addViewColumn(Gpf_Db_Table_Accounts::NAME, $this->_("Name"), true);
        $this->addViewColumn(Gpf_Db_Table_Accounts::STATUS, $this->_("Status"), true);
        $this->addViewColumn(Gpf_Db_Table_Accounts::EMAIL, $this->_("Email"), true);
        $this->addViewColumn(self::NETWORK_COMMISSIONS, $this->_("Commissions"), false, Gpf_View_ViewColumn::TYPE_CURRENCY);
        $this->addViewColumn(self::NETWORK_FEES, $this->_("Fees"), true, Gpf_View_ViewColumn::TYPE_CURRENCY);
        $this->addViewColumn(self::NETWORK_PAYMENTS, $this->_("Payments"), true, Gpf_View_ViewColumn::TYPE_CURRENCY);
        $this->addViewColumn(self::NETWORK_BALLANCE, $this->_("Ballance"), true, Gpf_View_ViewColumn::TYPE_CURRENCY);
        $this->addViewColumn(self::IMPRESSIONS_RAW, $this->_("Impressions raw/unique"), false);
        $this->addViewColumn(self::CLICKS_RAW, $this->_("Clicks raw/unique"), false);
        $this->addViewColumn(self::CTR_RAW, $this->_("CTR raw/unique"), false);
        $this->addViewColumn(self::SALES_COUNT, $this->_("Sales count"), false);
        $this->addViewColumn(self::SALES_TOTAL, $this->_("Sales total"), false);
        $this->addViewColumn(Gpf_Db_Table_Accounts::ACCOUNT_NOTE, $this->_("Account note"), false);
        $this->addViewColumn(Gpf_Db_Table_Accounts::SYSTEM_NOTE, $this->_("System note"), false);
        $this->addDynamicFields();
        $this->addAllActionsViewColumns();

        $this->addViewColumn(self::ACTIONS, $this->_("Actions"));
    }

    protected function addDynamicFields() {
        $formFields = Gpf_Db_Table_FormFields::getInstance();
        $fields = $formFields->getFieldsNoRpc('accountForm', array(Gpf_Db_FormField::STATUS_MANDATORY,
        Gpf_Db_FormField::STATUS_OPTIONAL,
        Gpf_Db_FormField::STATUS_HIDDEN,
        Gpf_Db_FormField::STATUS_READ_ONLY));
        foreach ($fields as $field) {
            $this->addViewColumn($field->get('code'), $this->_localize($field->get('name')), true);
        }
    }

    protected function initDataColumns() {
        $this->setKeyDataColumn('a.'.Gpf_Db_Table_Accounts::ID);
        $this->addDataColumn(Gpf_Db_Table_Accounts::ID, 'a.'.Gpf_Db_Table_Accounts::ID);
        $this->addDataColumn(Gpf_Db_Table_Accounts::NAME, 'a.'.Gpf_Db_Table_Accounts::NAME);
        $this->addDataColumn(Gpf_Db_Table_Accounts::STATUS, 'a.'.Gpf_Db_Table_Accounts::STATUS);
        $this->addDataColumn(Gpf_Db_Table_Accounts::EMAIL, 'a.'.Gpf_Db_Table_Accounts::EMAIL);
        $this->addDataColumn(Gpf_Db_Table_Accounts::ACCOUNT_NOTE, 'a.'.Gpf_Db_Table_Accounts::ACCOUNT_NOTE);
        $this->addDataColumn(Gpf_Db_Table_Accounts::SYSTEM_NOTE, 'a.'.Gpf_Db_Table_Accounts::SYSTEM_NOTE);
        for ($i = 1; $i <= 25; $i++) {
            $this->addDataColumn('data'.$i, 'a.data'.$i);
        }
        
        $this->addDataColumn(self::NETWORK_FEES, 'IF(f.amount is NULL, 0, f.amount)');
        $this->addDataColumn(self::NETWORK_PAYMENTS, 'IF(p.payments is NULL, 0, p.payments)');
        $this->addDataColumn(self::NETWORK_BALLANCE, 'IF(p.payments is NULL, 0, p.payments) - (IF(ac.amount is NULL, 0, ac.amount) + IF(tc.'.Pap_Db_Table_Transactions::COMMISSION.' is NULL, 0, tc.'.Pap_Db_Table_Transactions::COMMISSION.')) - IF(f.amount is NULL, 0, f.amount)');
        $this->addDataColumn(self::NETWORK_COMMISSIONS, 'IF(ac.amount is NULL, 0, ac.amount) + IF(tc.'.Pap_Db_Table_Transactions::COMMISSION.' is NULL, 0, tc.'.Pap_Db_Table_Transactions::COMMISSION.')');

        $this->initStatColumns();
    }

    protected function initDefaultView() {
        $this->addDefaultViewColumn(Gpf_Db_Table_Accounts::ID);
        $this->addDefaultViewColumn(Gpf_Db_Table_Accounts::NAME, '1', 'A');
        $this->addDefaultViewColumn(Gpf_Db_Table_Accounts::STATUS);
        $this->addDefaultViewColumn(Gpf_Db_Table_Accounts::EMAIL);
        $this->addDefaultViewColumn(self::ACTIONS);
    }

    protected function initResult() {
        $result = parent::initResult();
        foreach (array(self::NETWORK_FEES, self::NETWORK_BALLANCE, self::NETWORK_PAYMENTS) as $columnName) {
            if ($this->isColumnRequired($columnName)) {
                $result->getHeader()->add($columnName);
            }
        }
        return $result;
    }

    protected function buildFrom() {
        $this->_selectBuilder->from->add(Gpf_Db_Table_Accounts::getName(), 'a');

        if ($this->areColumnsRequiredOrInFilter(array(self::NETWORK_FEES, self::NETWORK_BALLANCE))) {
            $feesSelect = $this->getAccountingSelect(array(Pap_Db_Accounting::TYPE_FEE_FIXED, Pap_Db_Accounting::TYPE_FEE_PERCENTAGE));
            $this->_selectBuilder->from->addLeftJoin('('.$feesSelect->toString().')',
                    'f', 'f.'.self::GROUP_COLUMN_ALIAS.'=a.'. Gpf_Db_Table_Accounts::ID);
        }

        if ($this->areColumnsRequiredOrInFilter(array(self::NETWORK_COMMISSIONS, self::NETWORK_BALLANCE))) {
            $accountingCommissionSelect = $this->getAccountingSelect(Pap_Db_Accounting::TYPE_COMMISSSION);
            $this->_selectBuilder->from->addLeftJoin('('.$accountingCommissionSelect->toString().')',
                    'ac', 'ac.'.self::GROUP_COLUMN_ALIAS.'=a.'. Gpf_Db_Table_Accounts::ID);


            $transactionStats = new Pap_Stats_Computer_TransactionsStatsBuilder($this->getStatsParameters(), $this->groupColumn, self::GROUP_COLUMN_ALIAS, true, true);
            $transactionStats->getTransactionsSelect()->from->addLeftJoin(
                '('.Pap_Db_Table_Invoices::getInstance()->getLastAccountingCommissionSelect()->toString().')',
                'ld', Pap_Db_Table_Invoices::ACCOUNTID.'=ld.aid');
            $transactionStats->getTransactionsWhereClause()->add(Pap_Db_Table_Transactions::R_STATUS, '=', Pap_Common_Constants::STATUS_APPROVED);

            $lastDateWhere = new Gpf_SqlBuilder_CompoundWhereCondition();
            $lastDateWhere->add(Pap_Db_Table_Transactions::DATE_APPROVED, '>', 'ld.lastDate', 'OR', false);
            $lastDateWhere->add('ld.lastDate', 'IS', 'NULL', 'OR', false);
            $transactionStats->getTransactionsWhereClause()->addCondition($lastDateWhere);

            $this->_selectBuilder->from->addLeftJoin('('.$transactionStats->getStatsSelect()->toString().')',
                    'tc', 'tc.'.self::GROUP_COLUMN_ALIAS.'=a.'. Gpf_Db_Table_Accounts::ID);
        }

        if ($this->areColumnsRequiredOrInFilter(array(self::NETWORK_PAYMENTS, self::NETWORK_BALLANCE))) {
            $paymentsSelect = new Gpf_SqlBuilder_SelectBuilder();
            $paymentsSelect->select->add(Pap_Db_Table_Invoices::ACCOUNTID, self::GROUP_COLUMN_ALIAS);
            $paymentsSelect->select->add('sum('.Pap_Db_Table_Invoices::AMOUNT.')', 'payments');
            $paymentsSelect->from->add(Pap_Db_Table_Invoices::getName());
            $paymentsSelect->where->add(Pap_Db_Table_Invoices::STATUS, '=', Pap_Common_Constants::PSTATUS_PAID);
            $paymentsSelect->groupBy->add(Pap_Db_Table_Invoices::ACCOUNTID);
            $this->addDateFilterTo($paymentsSelect->where, Pap_Db_Table_Invoices::DATE_PAID);

            $this->_selectBuilder->from->addLeftJoin('('.$paymentsSelect->toString().')',
                    'p', 'p.'.self::GROUP_COLUMN_ALIAS.'=a.'. Gpf_Db_Table_Accounts::ID);
        }

        $this->buildStatsFrom();
    }

    private function getAccountingSelect($type) {
        $select = new Gpf_SqlBuilder_SelectBuilder();
        $select->select->add(Pap_Db_Table_Accountings::ACCOUNTID, self::GROUP_COLUMN_ALIAS);
        $select->select->add('sum('.Pap_Db_Table_Accountings::AMOUNT.')', 'amount');
        $select->from->add(Pap_Db_Table_Accountings::getName());
        if (is_array($type)) {
            $condition = new Gpf_SqlBuilder_CompoundWhereCondition();
            foreach ($type as $typeItem) {
                $condition->add(Pap_Db_Table_Accountings::TYPE, '=', $typeItem, 'OR');
            }
            $select->where->addCondition($condition);
        } else {
            $select->where->add(Pap_Db_Table_Accountings::TYPE, '=', $type);
        }
        $select->groupBy->add(Pap_Db_Table_Accountings::ACCOUNTID);
        $this->addDateFilterTo($select->where, Pap_Db_Table_Accountings::DATE_INSTERTED);
        return $select;
    }

    private function addDateFilterTo(Gpf_SqlBuilder_WhereClause $where, $dateColumnName) {
        foreach ($this->filters->getFilter('datetime') as $filter) {
            $filterClone = clone $filter;
            $filterClone->setCode($dateColumnName);
            $filterClone->addTo($where);
        }
    }

    protected function buildWhere() {
        parent::buildWhere();
        $this->_selectBuilder->where->add(Gpf_Db_Table_Accounts::ID, '<>', Gpf_Db_Account::DEFAULT_ACCOUNT_ID);
    }

    protected function addSelect($sql, $alias) {
        if (in_array($alias, array(self::NETWORK_FEES, self::NETWORK_BALLANCE, self::NETWORK_PAYMENTS)) && !$this->isColumnRequired($alias)) {
            return;
        }
        parent::addSelect($sql, $alias);
    }

    protected function addFilter(Gpf_SqlBuilder_Filter $filter) {
        if ($filter->getCode() == self::SEARCH_FILTER) {
            $condition = new Gpf_SqlBuilder_CompoundWhereCondition();
            $condition->add(Gpf_Db_Table_Accounts::ID, 'LIKE', '%'.$filter->getValue().'%', 'OR');
            $condition->add(Gpf_Db_Table_Accounts::NAME, 'LIKE', '%'.$filter->getValue().'%', 'OR');
            $condition->add(Gpf_Db_Table_Accounts::STATUS, 'LIKE', '%'.$filter->getValue().'%', 'OR');
            $condition->add(Gpf_Db_Table_Accounts::EMAIL, 'LIKE', '%'.$filter->getValue().'%', 'OR');
            $this->_selectBuilder->where->addCondition($condition);
        }
    }

    /**
     * @anonym
     * @service
     * @param Gpf_Rpc_Params $params
     */
    public function getCustomFilterFields(Gpf_Rpc_Params $params) {
        $filterFields = new Gpf_View_CustomFilterFields();

        $filterFields->addStringField(Gpf_Db_Table_Accounts::ID, $this->_("ID"));
        $filterFields->addStringField(Gpf_Db_Table_Accounts::NAME, $this->_("Name"));
        $filterFields->addStringField(Gpf_Db_Table_Accounts::EMAIL, $this->_("Email"));

        $filterFields->addNumberField(self::NETWORK_FEES, $this->_("Fees"));
        $filterFields->addNumberField(self::NETWORK_PAYMENTS, $this->_("Payments"));
        $filterFields->addNumberField(self::NETWORK_BALLANCE, $this->_("Ballance"));

        $formFields = Gpf_Db_Table_FormFields::getInstance();
        $fields = $formFields->getFieldsNoRpc('accountForm', array(Gpf_Db_FormField::STATUS_MANDATORY,
        Gpf_Db_FormField::STATUS_OPTIONAL,
        Gpf_Db_FormField::STATUS_HIDDEN,
        Gpf_Db_FormField::STATUS_READ_ONLY));
        foreach ($fields as $field) {
            $type = $field->get('type');
            if ($type == Gpf_Db_FormField::TYPE_NUMBER) {
                $filterFields->addNumberField($field->get('code'), $this->_localize($field->get('name')));
            } else {
                $filterFields->addStringField($field->get('code'), $this->_localize($field->get('name')));
            }
        }

        $this->addStatCustomFilterFields($filterFields);

        return $filterFields->getRecordSet();
    }

    /**
     * @return Pap_Stats_Params
     */
    protected function getStatsParameters() {
        $params = parent::getStatsParameters();
        return $this->addParamsWithDateRangeFilter($params);
    }

    /**
     * @param $select
     * @return Gpf_SqlBuilder_SelectBuilder
     */
    protected function createCountSelect(Gpf_SqlBuilder_SelectBuilder $select) {
        $preffixes = $select->where->getUniqueTablePreffixes();
        $preffixes['a'] = 'a';

        $count = new Gpf_SqlBuilder_SelectBuilder();
        $count->select->add('count(*)', 'count');
        $count->from = clone $select->from;
        $count->from->prune($preffixes);
        $count->where = $select->where;
        $count->groupBy = $select->groupBy;
        $count->having = $select->having;
        return $count;
    }
}
?>