<?php
/**
 *   @copyright Copyright (c) 2007 Quality Unit s.r.o.
 *   @author Milos Jancovic
 *   @package PostAffiliatePro
 *   @since Version 1.0.0
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.postaffiliatepro.com/licenses/license
 *
 */

class Pap_Features_AffiliateNetwork_Definition extends Gpf_Plugins_Definition  {

    const NETWORK_MERCHANT_ROLE = 'net_merc';

    const NETWORK_INVOICE = 'network_invoice';
    const NETWORK_PROFORMA_INVOICE = 'network_proforma_invoice';
    const ACCOUNT_POST_SIGNUP_TYPE = 'account_post_signup_type';
    const ACCOUNT_POST_SIGNUP_URL = 'account_post_signup_url';
    const ACCOUNT_APPROVAL = 'account_approval';
    const MERCHANT_AGREEMENT = 'merchant_agreement';
    const NOTIFICATION_ON_NEW_ACCOUNT_SIGNUP = 'notification_on_new_account_signup';
    const AFF_NOTIFICATION_ON_CAMPAIGNS_SUSPENDED = 'aff_notification_on_campaigns_suspended';
    const NOTIFICATION_ON_NEW_ACCOUNT_SIGNUP_BEFORE_APPROVAL = 'acc_notif_on_new_account_signup_before_approval';
    const NOTIFICATION_ON_REQUEST_PAYMENT = 'account_notification_on_request_payment';
    const NOTIFICATION_ON_INVOICE_PAID = 'account_notification_on_invoice_paid';
    const NOTIFICATION_ON_SUSPENDED = 'account_notification_on_suspended';
    const NOTIFICATION_ON_DECLINED = 'account_notification_on_declined';
    const FORCE_MERCHANT_AGREEMENT_ACCEPTANCE_SETTING_NAME = "forceMerchantAgreementAcceptance";
    const DEFAULT_NETWORK_MERCHANT_ROLE_SETTING_NAME = 'default_network_merchant_role';

    public function __construct() {
        $this->codeName = 'AffiliateNetwork';
        $this->name = $this->_('Affiliate network');
        $this->description = $this->_('AffiliateNetworkDescription');
        $this->version = '1.0.0';
        $this->pluginType = self::PLUGIN_TYPE_FEATURE;

        $this->addImplementation('PostAffiliate.merchant.menu', 'Pap_Features_AffiliateNetwork_Main', 'addToMenu');
        $this->addImplementation('PostAffiliate.Application.registerRolePrivileges', 'Pap_Features_AffiliateNetwork_Main', 'registerNetMerPrivileges');

        $this->addImplementation('Accounts.initConstraints', 'Pap_Features_AffiliateNetwork_Main', 'initConstraints');

        $this->addImplementation('AffiliateNetwork.modifyWhere', 'Pap_Features_AffiliateNetwork_Main', 'modifyWhere');
        $this->addImplementation('AffiliateNetwork.modifyFrom', 'Pap_Features_AffiliateNetwork_Main', 'modifyFrom');
        $this->addImplementation('AffiliateNetwork.directLinksModifyWhere', 'Pap_Features_AffiliateNetwork_Main', 'directLinksModifyWhere');

        $this->addImplementation('TransactionsGrid.initViewColumns', 'Pap_Features_AffiliateNetwork_Main', 'modifyViewColumnsTransactionsGrid');
        $this->addImplementation('TransactionsGrid.initDataColumns', 'Pap_Features_AffiliateNetwork_Main', 'modifyDataColumnsTransactionsGrid');
        $this->addImplementation('TransactionsGrid.initSearchAffiliateCondition', 'Pap_Features_AffiliateNetwork_Main', 'initSearchAffiliateCondition');

        $this->addImplementation('PostAffiliate.AdminsGrid.initViewColumns', 'Pap_Features_AffiliateNetwork_Main', 'initViewColumnsAdminsGrid');
        $this->addImplementation('PostAffiliate.AdminsGrid.initDefaultView', 'Pap_Features_AffiliateNetwork_Main', 'initDefaultViewAdminsGrid');

        $this->insertImplementations('PostAffiliate.CampaignsGrid');

        $this->insertImplementations('PostAffiliate.BannersGrid');

        $this->insertImplementations('PostAffiliate.VisitorAffiliatesGrid');

        $this->addImplementation('PostAffiliate.Pap_Db_Table_ClicksImpressions.getStatsSelect', 'Pap_Features_AffiliateNetwork_Main', 'statsClicksImpressionsModifyWhere');
        $this->addImplementation('Gpf_Report_OnlineUsers.buildWhere', 'Pap_Features_AffiliateNetwork_Main', 'buildWhereOnlineUsers');
        $this->addImplementation('Gpf_Report_OnlineUsersGadget.getOnlineRolesCount', 'Pap_Features_AffiliateNetwork_Main', 'getOnlineRolesCount');
        $this->addImplementation('Gpf_Report_LoginsHistory.buildWhere', 'Pap_Features_AffiliateNetwork_Main', 'buildWhereLoginsHistory');
        $this->addImplementation('Gpf_Common_UserRichListBox.createSelectBuilder', 'Pap_Features_AffiliateNetwork_Main', 'createSelectBuilderUserRichListBox');
        $this->addImplementation('Gpf_Role_RoleUsersGrid.buildWhere', 'Pap_Features_AffiliateNetwork_Main', 'modifyWhere');
        $this->addImplementation('Gpf_Auth_UserForm.save', 'Pap_Features_AffiliateNetwork_Main', 'saveNetworkOwner');
        $this->addImplementation('Pap_Db_Table_Campaigns.getCampaignsSelect', 'Pap_Features_AffiliateNetwork_Main', 'modifyCampaignsList');

        $this->addImplementation('PostAffiliate.ClicksGrid.initRawClicksSelect', 'Pap_Features_AffiliateNetwork_Main', 'initRawClicksSelectClicksGrid');
        $this->addImplementation('PostAffiliate.ClicksGrid.initDataColumns', 'Pap_Features_AffiliateNetwork_Main', 'initDataColumnsClicksGrid');

        $this->addImplementation('PostAffiliate.Campaigns.getCampaignsSelect', 'Pap_Features_AffiliateNetwork_Main', 'createSelectBuilderCampaignRichListBox');

        $this->addImplementation('PostAffiliate.Pap_Stats_Computer_Base.initSelectBuilder', 'Pap_Features_AffiliateNetwork_Main', 'initStatsSelectBuilder');
        $this->addImplementation('Pap_Stats_Computer_TransactionsStatsBuilder.buildWhere', 'Pap_Features_AffiliateNetwork_Main', 'initStatsSelectBuildWhere');

        $this->addImplementation('PostAffiliate.BannerListbox.getBannerSelect', 'Pap_Features_AffiliateNetwork_Main', 'initBannersSelectBuilder');

        $this->addImplementation('PostAffiliatePro.Pap_Mail_Reports_Report.setAccountId', 'Pap_Features_AffiliateNetwork_Main', 'setMailReportAccountId');

        $this->addImplementation('PostAffiliate.iconSet', 'Pap_Features_AffiliateNetwork_Main', 'addIcon');

        $this->addImplementation('PostAffiliate.Affiliates.Campaigns.buildWhere', 'Pap_Features_AffiliateNetwork_AccountStatusCheck', 'buildCampaignsWhere');
        $this->addImplementation('PostAffiliate.BannersGrid.buildWhere', 'Pap_Features_AffiliateNetwork_AccountStatusCheck', 'buildBannersWhere');
        $this->addImplementation('Pap_Common_Campaign_CampaignForAffiliateRichListBox.getCampaignRecordSetForAffiliate', 'Pap_Features_AffiliateNetwork_AccountStatusCheck', 'buildCampaignsWhere');
        $this->addImplementation('Tracker.click.recognizeParameters', 'Pap_Features_AffiliateNetwork_AccountStatusCheck', 'checkAccountStatus');
        $this->addImplementation('Pap_Tracking_Action_ActionProcessor.processAccount', 'Pap_Features_AffiliateNetwork_AccountStatusCheck', 'checkAccountStatus');

        $this->addImplementation('Core.defineSettings', 'Pap_Features_AffiliateNetwork_Main', 'initSettings');

        $this->addImplementation('PostAffiliate.EmailNotificationsForm.save', 'Pap_Features_AffiliateNetwork_Main', 'saveEmailNotifications');
        $this->addImplementation('PostAffiliate.EmailNotificationsForm.load', 'Pap_Features_AffiliateNetwork_Main', 'loadEmailNotifications');

        $this->addImplementation('PostAffiliate.Pap_Merchants_Config_AffEmailNotificationsForm.save', 'Pap_Features_AffiliateNetwork_Main', 'saveAffEmailNotifications');
        $this->addImplementation('PostAffiliate.Pap_Merchants_Config_AffEmailNotificationsForm.load', 'Pap_Features_AffiliateNetwork_Main', 'loadAffEmailNotifications');

        $this->addImplementation('PostAffiliate.Pap_Account.onStatusChange', 'Pap_Features_AffiliateNetwork_Main', 'onStatusChange');

        $this->addImplementation('Core.loadSetting', 'Pap_Features_AffiliateNetwork_Main', 'loadSetting');

        $this->addImplementation('PostAffiliate.ApplicationSettings.loadSetting', 'Pap_Features_AffiliateNetwork_Main', 'addApplicationSettings');

        $this->addImplementation('Pap_Common_Reports_DailyReportGrid.buildFrom', 'Pap_Features_AffiliateNetwork_Main', 'addAccountFilter');

        $this->addImplementation('Gpf_Role_RoleForm.afterDeleteRows', 'Pap_Features_AffiliateNetwork_Main', 'clearDefaultRole');

        $this->addImplementation('PostAffiliate.Transaction.refundChargeback', 'Pap_Features_AffiliateNetwork_Main', 'processRefundChargeback');
        $this->addImplementation('PostAffiliate.TransactionForm.changeTransactionStatus', 'Pap_Features_AffiliateNetwork_Main', 'processChangeTransactionStatus');

        $this->addImplementation('PostAffiliate.User.afterInsert', 'Pap_Features_AffiliateNetwork_Main', 'afterNewUserInsert');
    }

    public function onActivate() {
        if (!Gpf_Plugins_Engine::getInstance()->getConfiguration()->isPluginActive('MultipleMerchants')) {
            Gpf_Plugins_Engine::getInstance()->activate('MultipleMerchants', true);
        }
        $this->setup();
    }

    public function onDeactivate() {
        throw new Gpf_Exception($this->_("Affiliate network feature can't be deactivated"));
    }

    protected function setup() {
        $this->createDefaultNetworkRole();
        $this->setupMailTemplates();
    }

    private function createDefaultNetworkRole() {
        $networkMerchant = new Gpf_Db_Role();
        $networkMerchant->setId(self::NETWORK_MERCHANT_ROLE);
        $networkMerchant->setName('Default network merchant');
        $networkMerchant->setRoleType(Pap_Application::ROLETYPE_MERCHANT);
        try {
            $networkMerchant->insert();
        } catch (Gpf_DbEngine_DuplicateEntryException $e) {
        }

        $Merchant = new Gpf_Db_Role();
        $Merchant->setId(Pap_Application::DEFAULT_ROLE_MERCHANT);
        $Merchant->load();
        $Merchant->setName('Network Owner');
        $Merchant->save();
    }

    private function setupMailTemplates() {
        $this->setupMailTemplate(new Pap_Features_AffiliateNetwork_Mail_RequestPayment());
        $this->setupMailTemplate(new Pap_Features_AffiliateNetwork_Mail_InvoicePaid());
        $this->setupMailTemplate(new Pap_Features_AffiliateNetwork_Mail_AccountSuspendedMerchant());
        $this->setupMailTemplate(new Pap_Features_AffiliateNetwork_Mail_AccountDeclinedMerchant());
        $this->setupMailTemplate(new Pap_Features_AffiliateNetwork_Mail_CampaignSuspendedAffiliate());
        $this->setupMailTemplate(new Pap_Features_AffiliateNetwork_Mail_MerchantNewAccountSignup());
        $this->setupMailTemplate(new Pap_Features_AffiliateNetwork_Mail_AccountNewAccountSignupApproved());
        $this->setupMailTemplate(new Pap_Features_AffiliateNetwork_Mail_AccountNewAccountSignupBeforeApproval());
    }

    private function setupMailTemplate(Gpf_Mail_Template $mailTemplate) {
        $mailTemplate->setup(Gpf_Application::getInstance()->getAccountId());
    }

    private function insertImplementations($extensionPointPrefix) {
        $this->addImplementation($extensionPointPrefix . '.initDataColumns', 'Pap_Features_AffiliateNetwork_Main', 'initDataColumns');
        $this->addImplementation($extensionPointPrefix . '.initViewColumns', 'Pap_Features_AffiliateNetwork_Main', 'initViewColumns');
        $this->addImplementation($extensionPointPrefix . '.initDefaultView', 'Pap_Features_AffiliateNetwork_Main', 'initDefaultView');
    }
}
?>