<?php
/**
 *   @copyright Copyright (c) 2007 Quality Unit s.r.o.
 *   @package PostAffiliatePro
 *   @author Milos Jancovic
 *   @since Version 1.0.0
 *   $Id: CustomerForm.class.php 19572 2008-08-01 16:43:19Z mjancovic $
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.postaffiliatepro.com/licenses/license
 *
 */

/**
 * @package PostAffiliatePro
 */
class Pap_Features_AffiliateNetwork_Invoice extends Gpf_Object {

	/**
	 * @var Pap_Features_AffiliateNetwork_AccountVariables
	 */
	private $accountVariables;
	/**
	 * @var Pap_Features_AffiliateNetwork_InvoiceVariables
	 */
	private $invoiceVariables;

    private $fixedFee;
    private $percentageFee;
    private $commissions;

	public function __construct() {
		$this->accountVariables = new Pap_Features_AffiliateNetwork_AccountVariables();
		$this->invoiceVariables = new Pap_Features_AffiliateNetwork_InvoiceVariables();
	}

	public function setAccount(Pap_Account $account) {
		$this->accountVariables->setAccount($account);
	}

	public function setInvoice(Pap_Db_Invoice $invoice) {
		$this->invoiceVariables->setInvoice($invoice);
	}

    public function setFeesCommissions($fixedFee, $percentageFee, $commissions) {
        $this->invoiceVariables->setFeesCommissions($fixedFee, $percentageFee, $commissions);
    }

	public function generate($invoiceFormat) {
		$template = new Gpf_Templates_Template($invoiceFormat, '', Gpf_Templates_Template::FETCH_TEXT);
		$this->assignInvoiceVariables($template);
		return $template->getHTML();
	}

	/**
	 * @return Gpf_Data_RecordSet
	 */
	public function getVariables() {
		$variables = $this->accountVariables->toShalowRecordSet();
		$variables->add(array('date',$this->_('Date')));
		$variables->add(array('time',$this->_('Time')));
		$this->addVariables($variables, $this->accountVariables);
		$this->addVariables($variables, $this->invoiceVariables);
		return $variables;
	}

	private function assignInvoiceVariables(Gpf_Templates_Template $template) {
		$template->assign('date', Gpf_Common_DateUtils::getDateInLocaleFormat());
		$template->assign('time', Gpf_Common_DateUtils::getTimeInLocaleFormat());
		$this->assignVariables($template, $this->accountVariables);
		$this->assignVariables($template, $this->invoiceVariables);
	}

	private function assignVariables(Gpf_Templates_Template $template, Pap_Features_AffiliateNetwork_Variables $variables) {
		foreach ($variables->toValues() as $id => $value) {
			$template->assign($id, $value);
		}
	}

	private function addVariables(Gpf_Data_RecordSet $variablesRecordSet, Pap_Features_AffiliateNetwork_Variables $variables) {
		foreach ($variables as $variable) {
			$variablesRecordSet->addRecord($variable);
		}
	}
}
?>