<?php
/**
 *   @copyright Copyright (c) 2007 Quality Unit s.r.o.
 *   @package PostAffiliatePro
 *   @author Milos Jancovic
 *   @since Version 1.0.0
 *   $Id: CustomerForm.class.php 19572 2008-08-01 16:43:19Z mjancovic $
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.postaffiliatepro.com/licenses/license
 *
 */

/**
 * @package PostAffiliatePro
 */
class Pap_Features_AffiliateNetwork_InvoiceForm extends Pap_Features_AffiliateNetwork_NetworkInvoiceForm {

    const FORMFIELD_FEE_FIXED = 'fixedFee';
    const FORMFIELD_FEE_PERCENTAGE = 'percentageFee';
    const FORMFIELD_COMMISSIONS = 'commissions';

    /**
     * @var Pap_Db_Accounting
     */
    protected $feeFixed;

    /**
     * @var Pap_Db_Accounting
     */
    protected $feePercentage;

    /**
     * @var Pap_Db_Accounting
     */
    protected $commission;

    /**
     * @service invoice add
     * @param $fields
     * @return Gpf_Rpc_Form
     */
    public function add(Gpf_Rpc_Params $params) {
        return parent::add($params);
    }

    /**
     * @service invoice read
     * @param $fields
     * @return Gpf_Rpc_Form
     */
    public function send_mail(Gpf_Rpc_Params $params) {
        return parent::send_mail($params);
    }

    /**
     * @service invoice write
     * @param Gpf_Rpc_Params $params
     * @return Gpf_Rpc_Form
     */
    public function markaspaid(Gpf_Rpc_Params $params) {
        return $this->save($params);
    }

    /**
     * @service invoice read
     * @param $fields
     * @return Gpf_Rpc_Form
     */
    public function init(Gpf_Rpc_Params $params) {
        $form = new Gpf_Rpc_Form($params);
        $this->processDateFieldsToDateTimeFields($form);        
        
        $transactionsStatsParams = $this->createStatsParams($form);
        $nextInvoiceDateFrom = $this->getNextInvoiceDateFrom($form->getFieldValue('accountid'));
        $this->initDateForStatsParams($transactionsStatsParams, $form, $nextInvoiceDateFrom);
        $transactionStats = new Pap_Stats_Transactions($transactionsStatsParams);

        $form->addField('datefrom', $nextInvoiceDateFrom);
        $form->addField('commissions', $this->computeCommissions($transactionStats));
        $form->addField('balance', $this->computeBalance(new Pap_Features_AffiliateNetwork_Stats_AccountingComputer($this->createStatsParams($form)),
        new Pap_Features_AffiliateNetwork_Stats_InvoiceComputer($this->createStatsParams($form))));
        $form->addField('duedate', $this->computeDueDate($form));
        return $form;
    }

    private function computeDueDate(Gpf_Rpc_Form $form){
        $todate = new Gpf_DateTime($form->getFieldValue('todate'));
        $todate->addDay(14);
        return $todate->toDate();
    }

    /**
     * @service invoice read
     * @param Gpf_Rpc_Params $params
     * @return Gpf_Rpc_Form
     */
    public function load(Gpf_Rpc_Params $params) {
        return parent::load($params);
    }

    /**
     * @service invoice delete
     * @param $ids
     * @return Gpf_Rpc_Action
     */
    public function deleteRows(Gpf_Rpc_Params $params) {
        return parent::deleteRows($params);
    }
    
    /**
     * @service invoice read
     * @return Gpf_Rpc_Action
     */
    public function getNextInvoiceNumber(Gpf_Rpc_Params $params) {
        $data = new Gpf_Rpc_Data($params);

        $data->setValue('nextInvoiceNumber', $this->getNextNumber($this->getInvoiceNumbersArray()));
        return $data;
    }
    
    /**
     * @return array
     */
    private function getInvoiceNumbersArray() {
        $select = new Gpf_SqlBuilder_SelectBuilder();
        $select->select->add(Pap_Db_Table_Invoices::NUMBER);
        $select->from->add(Pap_Db_Table_Invoices::getName());
        $invoiceCollection = $select->getAllRowsIterator();
        $numbers = array();
        foreach ($invoiceCollection as $invoice) {
            $numbers[] = $invoice->get(Pap_Db_Table_Invoices::NUMBER);
        }
        return $numbers;
    }
    
    protected function getNextNumber(array $numbers) {
        if (count($numbers) == 0) {
            return 1;
        }
        $newValue = max($numbers);
        $lastValue = $newValue;
        $newValue++;
        
        if (strlen($lastValue) > strlen($newValue)) {
            $prefix = substr($lastValue, 0, strlen($lastValue)-strlen($newValue));
            $newValue = $prefix.$newValue;
        }
        
        return $newValue;
    }

    /**
     * @return String
     */
    protected function computeBalance(Pap_Features_AffiliateNetwork_Stats_AccountingComputer $accountingStats,
    Pap_Features_AffiliateNetwork_Stats_InvoiceComputer $invoiceStats) {
        return - $accountingStats->getAmount()->getCommissions() -        
        $accountingStats->getAmount()->getFees() +
        $invoiceStats->getAmount()->getPaid();
    }
    
    protected function computeCommissions(Pap_Stats_Transactions $transactionStats) {
    	return $transactionStats->getCommission()->getApproved() + $transactionStats->getCommission()->getPaid();
    }

    protected function loadRow(Gpf_DbEngine_RowBase $row) {
        try {
            $row->load();
        } catch (Gpf_DbEngine_NoRow $e) {
            throw new Gpf_Exception($this->_("%s does not exist", $this->getDbRowObjectName()));
        }
    }

    protected function fillAdd(Gpf_Rpc_Form $form, Gpf_DbEngine_RowBase $dbRow) {
    	$this->processDateFieldsToDateTimeFields($form);
        parent::fillAdd($form, $dbRow);
        $fixedFee = $form->getFieldValue(self::FORMFIELD_FEE_FIXED);
        $percentageFee = $form->getFieldValue(self::FORMFIELD_FEE_PERCENTAGE);
        $commissions = $form->getFieldValue(self::FORMFIELD_COMMISSIONS);
        $dbRow->setProformaText($this->generateInvoiceText($dbRow, Gpf_Settings::get(Pap_Features_AffiliateNetwork_Definition::NETWORK_PROFORMA_INVOICE), $fixedFee, $percentageFee, $commissions));
        $this->feeFixed = $this->createAccountingRow($form->getFieldValue('accountid'), Pap_Db_Accounting::TYPE_FEE_FIXED);
        $this->feeFixed->setAmount($fixedFee);
        
        $this->feePercentage = $this->createAccountingRow($form->getFieldValue('accountid'), Pap_Db_Accounting::TYPE_FEE_PERCENTAGE);
        $this->feePercentage->setAmount($percentageFee);

        $this->commission = $this->createAccountingRow($form->getFieldValue('accountid'), Pap_Db_Accounting::TYPE_COMMISSSION);
        $this->commission->setAmount($commissions);
    }

    protected function addRow(Gpf_Rpc_Form $form, Gpf_DbEngine_RowBase $row) {
        parent::addRow($form, $row);
        $this->insertAccountRow($this->feeFixed, $row);
        $this->insertAccountRow($this->feePercentage, $row);
        $this->insertAccountRow($this->commission, $row);
        $this->sendRequestPaymentMail($row);
    }

    protected function fillSave(Gpf_Rpc_Form $form, Gpf_DbEngine_RowBase $dbRow) {
    	$this->processDateFieldsToDateTimeFields($form);
        parent::fillSave($form, $dbRow);
        $dbRow->setStatus(Pap_Common_Constants::PSTATUS_PAID);
        $dbRow->setInvoiceText($this->generateInvoiceText($dbRow, Gpf_Settings::get(Pap_Features_AffiliateNetwork_Definition::NETWORK_INVOICE)));

        Gpf_Plugins_Engine::extensionPoint('PostAffiliate.InvoiceForm.markaspaid', $dbRow);
    }

    protected function updateRow(Gpf_Rpc_Form $form, Gpf_DbEngine_RowBase $row) {
        parent::updateRow($form, $row);
        $this->sendInvoicePaidMail($row);
    }

    /**
     * @param $form
     * @return Pap_Stats_Params
     */
    protected function createStatsParams(Gpf_Rpc_Form $form) {
        $statsParams = new Pap_Stats_Params();
        $statsParams->setAccountId($form->getFieldValue('accountid'));
        return $statsParams;
    }
    
    protected function initDateForStatsParams(Pap_Stats_Params $statsParams, Gpf_Rpc_Form $form, $nextInvoiceDateFrom) {
    	$statsParams->setDateApprovedFrom(new Gpf_DateTime($nextInvoiceDateFrom));
        try {
            $statsParams->setDateApprovedTo(new Gpf_DateTime($form->getFieldValue('todate')));
        } catch (Gpf_Data_RecordSetNoRowException $e) {
        }
    }

    /**
     * @return String
     */
    protected function getNextInvoiceDateFrom($accountId) {
        try {
            return Pap_Db_Table_Invoices::getInstance()->getNextInvoiceDateFrom($accountId);
        } catch (Gpf_DbEngine_NoRowException $e) {
        }
        $accountDateInserted = new Gpf_DateTime(Gpf_Common_DateUtils::getTimestamp($this->getAccountDateInserted($accountId)));
        Gpf_Plugins_Engine::extensionPoint('PostAffiliate.InvoiceForm.getNextInvoiceDateFrom', $accountDateInserted);
        return $accountDateInserted->toDate();
    }

    /**
     * @return String
     */
    private function getAccountDateInserted($accountId) {
        try {
            return Gpf_Common_DateUtils::getOnlyDatePart(Pap_Account::getAccount($accountId)->getDateinserted());
        } catch (Gpf_Exception $e) {
        }
        return '';
    }

    /**
     * @param $accountid
     * @param $type
     * @return Pap_Db_Accounting
     */
    protected function createAccountingRow($accountid, $type) {
        $accounting = new Pap_Db_Accounting();
        $accounting->setAccountId($accountid);
        $accounting->setType($type);
        return $accounting;
    }

    /**
     * @throws Gpf_DbEngine_Row_ConstraintException
     * @param $accounting
     * @param $invoice
     */
    protected function insertAccountRow(Pap_Db_Accounting $accounting, Pap_Db_Invoice $invoice) {
        $accounting->setInvoiceId($invoice->getId());
        $accounting->insert();
    }

    protected function sendRequestPaymentMail(Pap_Db_Invoice $invoice) {
    	if (Gpf_Settings::get(Pap_Features_AffiliateNetwork_Definition::NOTIFICATION_ON_REQUEST_PAYMENT) === Gpf::YES) {
        	$this->sendInvoiceMail(new Pap_Features_AffiliateNetwork_Mail_RequestPayment(), $invoice);
    	}
    }

    protected function sendInvoicePaidMail(Pap_Db_Invoice $invoice) {
    	if (Gpf_Settings::get(Pap_Features_AffiliateNetwork_Definition::NOTIFICATION_ON_INVOICE_PAID) === Gpf::YES) {
        	$this->sendInvoiceMail(new Pap_Features_AffiliateNetwork_Mail_InvoicePaid(), $invoice);
    	}
    }

    /**
     * @param $invoice
     * @param $invoiceFormat
     * @return String
     */
    protected function generateInvoiceText(Pap_Db_Invoice $invoice, $invoiceFormat, $fixedFee = null, $percentageFee = null, $commissions = null) {
        $invoiceText = new Pap_Features_AffiliateNetwork_Invoice();
        try {
            $invoiceText->setAccount(Pap_Account::getAccount($invoice->getAccountId()));
        } catch (Gpf_Exception $e) {
        }
        $invoiceText->setInvoice($invoice);
        $invoiceText->setFeesCommissions($fixedFee, $percentageFee, $commissions);
        return $invoiceText->generate($invoiceFormat);
    }
    
    protected function processDateFieldsToDateTimeFields(Gpf_Rpc_Form $form) {    	
    	$this->processDateTime($form, 'datefrom');
    	$this->processDateTime($form, 'datepaid', '23:59:59');
    	$this->processDateTime($form, 'dateto', '23:59:59');
    	$this->processDateTime($form, 'todate', '23:59:59');
    	$this->processDateTime($form, 'duedate', '23:59:59');
    }

    private function processDateTime(Gpf_Rpc_Form $form, $fieldName, $time = '00:00:00') {
        if ($form->existsField($fieldName)) {
    		$form->setField($fieldName, $form->getFieldValue($fieldName) . ' '. $time);
    	}
    }
}
?>