<?php
/**
 *   @copyright Copyright (c) 2007 Quality Unit s.r.o.
 *   @author Milos Jancovic
 *   @package PostAffiliatePro
 *   @since Version 1.0.0
 *   $Id: ActionParser.class.php 16620 2008-03-21 09:21:07Z aharsani $
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.postaffiliatepro.com/licenses/license
 *
 */

/**
 * @package PostAffiliatePro
 */
class Pap_Features_AffiliateNetwork_InvoiceVariables extends Pap_Features_AffiliateNetwork_Variables {

    /**
     * @var Pap_Db_Invoice
     */
    private $invoice;

    private $fixedFee;
    private $percentageFee;
    private $commissions;

    public function setInvoice(Pap_Db_Invoice $invoice) {
        $this->invoice = $invoice;
    }

    public function setFeesCommissions($fixedFee, $percentageFee, $commissions) {
        $this->fixedFee = $fixedFee;
        $this->percentageFee = $percentageFee;
        $this->commissions = $commissions;
    }

    protected function initVariables() {
        $this->createVariable('duedate', $this->_('Due date'));
        $this->createVariable('datefrom', $this->_('Date from'));
        $this->createVariable('dateto', $this->_('Date to'));
        $this->createVariable('invoicenumber', $this->_('Invoice number'));
        $this->createVariable('amount', $this->_('Amount'));
        $this->createVariable('commissions', $this->_('Commissions'));
        $this->createVariable('fee_fixed', $this->_('Fixed fee'));
        $this->createVariable('fee_percentage', $this->_('Percentage fee'));
        $this->createVariable('merchantnote', $this->_('Merchant note'));
        $this->createVariable('vat_percentage', $this->_('VAT percentage'));
        $this->createVariable('payment_vat_part', $this->_('VAT part'));
        $this->createVariable('payment_incl_vat', $this->_('Payment incl. VAT'));
        $this->createVariable('payment_excl_vat', $this->_('Payment excl. VAT'));
    }

    /**
     * @return array $key = id, $value = value
     */
    public function toValues() {
        if (is_null($this->invoice)) {
            return array();
        }
        $values = array();
        $values['duedate'] = $this->invoice->getDueDate();
        $values['datefrom'] = $this->invoice->getDateFrom();
        $values['dateto'] = $this->invoice->getDateTo();
        $values['invoicenumber'] = $this->invoice->getNumber();
        $values['amount'] = $this->invoice->getAmount();
        $values['commissions'] = $this->getCommissions();
        $values['fee_fixed'] = $this->getFixedFee();
        $values['fee_percentage'] = $this->getPercentageFee();
        $values['merchantnote'] = $this->invoice->getMerchantNote();
        $vatPercentage = $this->getAccountVatPercentage($this->invoice->getAccountId());
        $values['vat_percentage'] = $vatPercentage;
        $vatPart = ($this->invoice->getAmount() * $vatPercentage) / 100;
        $values['payment_vat_part'] = $vatPart;
        $values['payment_incl_vat'] = $this->invoice->getAmount() + $vatPart;
        $values['payment_excl_vat'] = $this->invoice->getAmount() - $vatPart;
        return $values;
    }

    private function getAccountVatPercentage($accountId) {
        try {
            if (Gpf_Db_Table_UserAttributes::getSetting(Pap_Features_AffiliateNetwork_AccountsForm::USER_SETTING_APPLY_VAT, $accountId) != Gpf::YES) {
                return 0;
            }
        } catch (Gpf_DbEngine_NoRowException $e) {
            return 0;
        }
        try {
            $vatPercentage = Gpf_Db_Table_UserAttributes::getSetting(Pap_Settings::VAT_PERCENTAGE_SETTING_NAME, $accountId);
            if ($vatPercentage == '') {
                $vatPercentage = Gpf_Settings::get(Pap_Settings::VAT_PERCENTAGE_SETTING_NAME);
            }
        } catch (Gpf_DbEngine_NoRowException $e) {
            $vatPercentage = Gpf_Settings::get(Pap_Settings::VAT_PERCENTAGE_SETTING_NAME);
        }
        return $vatPercentage;
    }

    private function getCommissions() {
        return $this->commissions !== null ? $this->commissions: $this->getAccountingValue(Pap_Db_Accounting::TYPE_COMMISSSION);
    }

    private function getFixedFee() {
        return $this->fixedFee !== null ? $this->fixedFee: $this->getAccountingValue(Pap_Db_Accounting::TYPE_FEE_FIXED);
    }

    private function getPercentageFee() {
        return $this->percentageFee !== null ? $this->percentageFee: $this->getAccountingValue(Pap_Db_Accounting::TYPE_FEE_PERCENTAGE);
    }

    private function getAccountingValue($type) {
        $accounting = new Pap_Db_Accounting();
        $accounting->setType($type);
        $accounting->setInvoiceId($this->invoice->getId());
        try {
            $accounting->loadFromData(array(Pap_Db_Table_Accountings::INVOICEID, Pap_Db_Table_Accountings::TYPE));
        } catch (Gpf_DbEngine_NoRowException $e) {
            return 0;
        }
        return $accounting->getAmount();
    }
}
?>