<?php
/**
 *   @copyright Copyright (c) 2007 Quality Unit s.r.o.
 *   @author Milos Jancovic
 *   @package PostAffiliatePro
 *   @since Version 1.0.0
 *   $Id: UpdateManager.class.php 18026 2008-05-14 08:07:20Z mbebjak $
 *
 *   Licensed under the Quality Unit, s.r.o. Dual License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.qualityunit.com/licenses/gpf
 *
 */

/**
 * @package PostAffiliatePro
 */
class Pap_Features_AffiliateNetwork_Mail_CampaignsSuspendedSendTask extends Gpf_Tasks_LongTask {

	/**
	 * @var Pap_Account
	 */
	private $account;

    public function setAccount(Pap_Account $account) {
        $this->account = $account;
        $this->setParams(serialize($account));
    }

	public function getName() {
		return $this->_('Campaigns suspended send');
	}

	protected function execute() {
	    $this->account = unserialize($this->getParams());

		foreach ($this->getAffiliatesIdIterator() as $row) {
			$this->processRow($row);
		}
	}

	protected function processRow(Gpf_Data_Record $row) {
		if ($this->isPending($row->get(Pap_Stats_Table::USERID), $this->_('Send campaigns suspended mail'))) {
			try {
				$this->sendEmail($this->getAffiliate($row->get(Pap_Stats_Table::USERID)));
			} catch (Gpf_Exception $e) {
			}
			$this->setDone();
		}
	}

	/**
	 * @throws Gpf_DbEngine_NoRowException
	 * @param $affiliateId
	 * @return Pap_Common_User
	 */
	protected function getAffiliate($affiliateId) {
		return Pap_Affiliates_User::getUserById($affiliateId);
	}

	/**
	 * @return Pap_Features_AffiliateNetwork_Mail_CampaignSuspendedAffiliate
	 */
	protected function createMail() {
		return new Pap_Features_AffiliateNetwork_Mail_CampaignSuspendedAffiliate();
	}

	/**
	 * @param $affiliateId
	 * @return array
	 */
	protected function getAffiliatePromotesCampaigns($affiliateId) {
		$campaign = new Pap_Db_Campaign();
		$suspendedCampaigns = array();
		foreach ($this->getAffiliateCampaignsIterator($affiliateId) as $row) {
			$campaign->fillFromRecord($row);
			$suspendedCampaigns[] = $campaign->toArray();
		}
		return $suspendedCampaigns;
	}

	/**
	 * @param $affiliate
	 */
	protected function sendEmail(Pap_Common_User $affiliate) {
		$mail = $this->createMail();
		$mail->setUser($affiliate);
		$mail->setSuspendedCampaigns($this->getAffiliatePromotesCampaigns($affiliate->getId()));
		$mail->addRecipient($affiliate->getEmail());
		$mail->send();
	}

	/**
	 * @return Iterator
	 */
	protected function getAffiliateCampaignsIterator($affiliateId) {
		$unionBuilder = $this->createCampaignsUnion($affiliateId);

		$affiliateCampaigns = new Gpf_SqlBuilder_SelectBuilder();
		$affiliateCampaigns->select->add(Pap_Db_Table_Campaigns::NAME);
		$affiliateCampaigns->from->add(Pap_Db_Table_Campaigns::getName(), 'c');
		$affiliateCampaigns->from->addInnerJoin('(' . $unionBuilder->toString() . ')', 'ac',
		'c.'.Pap_Db_Table_Campaigns::ID . '=ac.' . Pap_Stats_Table::CAMPAIGNID);

		return $affiliateCampaigns->getAllRowsIterator();
	}

	/**
	 * @param $affiliateId
	 * @return Gpf_SqlBuilder_UnionBuilder
	 */
	protected function createCampaignsUnion($affiliateId) {
		$unionBuilder = new Gpf_SqlBuilder_UnionBuilder();
		$unionBuilder->addSelect($this->createCampaignsStatsSelect(Pap_Db_Table_Impressions::getInstance(), $affiliateId));
		$unionBuilder->addSelect($this->createCampaignsStatsSelect(Pap_Db_Table_Clicks::getInstance(), $affiliateId));
		$unionBuilder->addSelect($this->createCampaignsStatsSelect(Pap_Db_Table_Transactions::getInstance(), $affiliateId));
		return $unionBuilder;
	}

	/**
	 * @param $table
	 * @param $affiliateId
	 * @return Gpf_SqlBuilder_SelectBuilder
	 */
	protected function createCampaignsStatsSelect(Gpf_DbEngine_Table $table, $affiliateId) {
		$statsSelect = $this->createStatsSelect($table, Pap_Stats_Table::CAMPAIGNID);
		$statsSelect->where->add(Pap_Stats_Table::USERID, '=', $affiliateId);
		return $statsSelect;
	}

	/**
	 * @return Iterator
	 */
	protected function getAffiliatesIdIterator() {
		$unionBuilder = new Gpf_SqlBuilder_UnionBuilder();
		$unionBuilder->addSelect($this->createStatsSelect(Pap_Db_Table_Impressions::getInstance(), Pap_Stats_Table::USERID));
		$unionBuilder->addSelect($this->createStatsSelect(Pap_Db_Table_Clicks::getInstance(), Pap_Stats_Table::USERID));
		$unionBuilder->addSelect($this->createStatsSelect(Pap_Db_Table_Transactions::getInstance(), Pap_Stats_Table::USERID));
		return $unionBuilder->getAllRowsIterator();
	}

	/**
	 * @param $table
	 * @return Gpf_SqlBuilder_SelectBuilder
	 */
	protected function createStatsSelect(Gpf_DbEngine_Table $table, $columnName) {
		$select = new Gpf_SqlBuilder_SelectBuilder();
		$select->select->add($columnName);
		$select->from->add($table->name());
		$this->fillWhereClause($select->where);
		$select->groupBy->add($columnName);
		return $select;
	}

	protected function fillWhereClause(Gpf_SqlBuilder_WhereClause $where) {
		$dateFilter = new Gpf_SqlBuilder_Filter(array(Pap_Stats_Table::DATEINSERTED, 'DP', 'L30D'));
		$dateFilter->addTo($where);
		$where->add(Pap_Stats_Table::ACCOUNTID, '=', $this->account->getId());
	}
}