<?php
/**
 *   @copyright Copyright (c) 2007 Quality Unit s.r.o.
 *   @author Milos Jancovic
 *   @package PostAffiliatePro
 *   @since Version 1.0.0
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.postaffiliatepro.com/licenses/license
 *
 */

/**
 * @package PostAffiliatePro plugins
 */
class Pap_Features_AffiliateNetwork_Main extends Gpf_Plugins_Handler {

    public static function getHandlerInstance() {
        return new Pap_Features_AffiliateNetwork_Main();
    }

    public function addToMenu(Pap_Merchants_Menu $menu) {
        $item = $menu->addItem('Accounts', $this->_('Accounts'));
        $item->addItem('Accounts', $this->_('Account Manager'));
    }

    public function registerNetMerPrivileges(Pap_Application $application) {
        $application->addRolePrivileges(Pap_Features_AffiliateNetwork_Definition::NETWORK_MERCHANT_ROLE, 'Pap_Features_AffiliateNetwork_NetworkMerchantPrivileges');
    }

    public function afterNewUserInsert(Pap_Common_User $user) {
        if ($user->getAccountId() !== Gpf_Db_Account::DEFAULT_ACCOUNT_ID && $user->getType() == Pap_Application::ROLETYPE_MERCHANT) {
            $user->addGadget('C', Gpf_Lang::_runtime('Accounting'), 'content://AccountingGadget',
                Gpf_Db_Table_Gadgets::POSITION_TYPE_SIDEBAR, 540, 1000, 333, 124);
        }
    }

    public function initConstraints(Gpf_Db_Table_Accounts $accounts) {
        $accounts->addDeleteConstraint(new Gpf_DbEngine_CascadeDeleteConstraint(Gpf_Db_Table_Accounts::ID, Pap_Db_Table_Campaigns::ACCOUNTID, new Pap_Db_Campaign()));
        $accounts->addDeleteConstraint(new Gpf_DbEngine_CascadeDeleteConstraint(Gpf_Db_Table_Accounts::ID, Pap_Db_Table_Banners::ACCOUNT_ID, new Pap_Db_Banner()));
    }

    public function modifyWhere(Gpf_Common_SelectBuilderCompoundRecord $context) {
        if ($this->isNetworkMerchant()) {
            $columnPrefix = '';
            if ($context->getParams()->contains('columnPrefix')) {
                $columnPrefix = $context->getParams()->get('columnPrefix') . '.';
            }
            $operator = 'AND';
            if ($context->getParams()->contains('operator')) {
                $operator = $context->getParams()->get('operator');
            }

            $context->getSelectBuilder()->where->add($columnPrefix . Gpf_Db_Table_Accounts::ID, '=', $this->getAccountId(), $operator);
        }
    }

    public function modifyFrom(Gpf_Common_SelectBuilderCompoundRecord $context) {
        if ($this->isMasterMerchant()) {
            $context->getSelectBuilder()->from->addInnerJoin(Gpf_Db_Table_Accounts::getName(), $context->getParams()->get('joinedAlias'),
            $context->getParams()->get('joinedAlias') . '.' . Gpf_Db_Table_Accounts::ID . '=' . $context->getParams()->get('onJoinAlias') . '.' . Gpf_Db_Table_Accounts::ID);
        }
    }

    public function directLinksModifyWhere(Gpf_SqlBuilder_SelectBuilder $select) {
        if ($this->isNetworkMerchant()) {
            $condition = new Gpf_SqlBuilder_CompoundWhereCondition();
            $condition->add('b.' . Gpf_Db_Table_Accounts::ID, '=', $this->getAccountId());
            $condition->add('c.' . Gpf_Db_Table_Accounts::ID, '=', $this->getAccountId(), 'OR');
            $select->where->addCondition($condition);
        }
    }

    public function statsClicksImpressionsModifyWhere(Pap_Stats_StatsSelectContext $statsSelectContext) {
        if ($this->isNetworkMerchant()) {
            $statsSelect = $statsSelectContext->getSelectBuilder();
            $statsSelect->where->add(Gpf_Db_Table_Accounts::ID, '=', $this->getAccountId());
        }
    }

    public function initStatsSelectBuildWhere(Gpf_SqlBuilder_SelectBuilder $transactionsSelect) {
        if ($this->isNetworkMerchant()) {
            $transactionsSelect->where->add(Gpf_Db_Table_Accounts::ID, '=', $this->getAccountId());
        }
    }

    public function addAccountFilter(Gpf_Rpc_FilterCollection $filters) {
        if ($this->isNetworkMerchant()) {
            $filters->add(array(Gpf_Db_Table_Accounts::ID, 'E', $this->getAccountId()));
        }
    }

    public function modifyViewColumnsTransactionsGrid(Pap_Merchants_Transaction_TransactionsGrid $transactionsGrid) {
        if ($this->isNetworkMerchant() && !Gpf_Session::getAuthUser()->hasPrivilege(Pap_Privileges::AFFILIATE, Gpf_Privileges::P_READ)) {
            $transactionsGrid->removeViewColumn(Gpf_Db_Table_AuthUsers::USERNAME);
            $transactionsGrid->removeViewColumn(Pap_Db_Table_Transactions::USER_ID);
        }
    }

    public function modifyDataColumnsTransactionsGrid(Pap_Merchants_Transaction_TransactionsGrid $transactionsGrid) {
        if ($this->isNetworkMerchant() && !Gpf_Session::getAuthUser()->hasPrivilege(Pap_Privileges::AFFILIATE, Gpf_Privileges::P_READ)) {
            $transactionsGrid->removeDataColumn(Gpf_Db_Table_AuthUsers::USERNAME);
            $transactionsGrid->removeDataColumn(Gpf_Db_Table_AuthUsers::FIRSTNAME);
            $transactionsGrid->removeDataColumn(Gpf_Db_Table_AuthUsers::LASTNAME);
            $transactionsGrid->removeDataColumn(Pap_Db_Table_Transactions::USER_ID);
        }
    }

    public function initSearchAffiliateCondition(Pap_Merchants_Transaction_TransactionsGrid $transactionsGrid) {
        if ($this->isNetworkMerchant() && !Gpf_Session::getAuthUser()->hasPrivilege(Pap_Privileges::AFFILIATE, Gpf_Privileges::P_READ)) {
            $transactionsGrid->clearSearchAffiliateCondition();
        }
    }

    public function initViewColumnsAdminsGrid(Pap_Features_MultipleMerchants_AdminsGrid $grid) {
        if (!$this->isMasterMerchant()) {
            return;
        }

        if ($grid->isAccountColumnVisible()) {
            $grid->addViewColumn('account', $this->_('Account'));
        }
        $grid->addViewColumn('role', $this->_('Role'));
    }

    public function initDefaultViewAdminsGrid(Pap_Features_MultipleMerchants_AdminsGrid $grid) {
        if ($this->isMasterMerchant()) {
            $grid->addDefaultViewColumn('account');
            $grid->addDefaultViewColumn('role');
        }
    }

    public function initDataColumns(Gpf_View_GridService $grid) {
        if ($this->isMasterMerchant()) {
            $grid->addDataColumn('account', 'a.' . Gpf_Db_Table_Accounts::NAME);
            $grid->addDataColumn('accountstatus', 'a.' . Gpf_Db_Table_Accounts::STATUS);
        }
    }

    public function initViewColumns(Gpf_View_GridService $grid) {
        if ($this->isMasterMerchant()) {
            $grid->addViewColumn('account', $this->_('Account'), true);
        }
    }

    public function initDefaultView(Gpf_View_GridService  $grid) {
        if ($this->isMasterMerchant()) {
            $grid->addDefaultViewColumn('account');
        }
    }

    public function buildWhereOnlineUsers(Gpf_SqlBuilder_WhereClause $where) {
        if ($this->isNetworkMerchant()) {
            $where->add('u.accountid', '=', $this->getAccountId());
        }
    }

    public function getOnlineRolesCount(Gpf_SqlBuilder_WhereClause $where) {
        if ($this->isNetworkMerchant()) {
            $where->add('u.'. Gpf_Db_Table_Accounts::ID, '=', $this->getAccountId());
        }
    }

    public function buildWhereLoginsHistory(Gpf_SqlBuilder_WhereClause $where) {
        if ($this->isNetworkMerchant()) {
            $where->add('u.accountid', '=', $this->getAccountId());
        }
    }

    public function createSelectBuilderUserRichListBox(Gpf_SqlBuilder_WhereClause $where) {
        if ($this->isNetworkMerchant()) {
            $where->add('users.'.Gpf_Db_Table_Users::ACCOUNTID, '=', $this->getAccountId());
        }
    }

    public function initRawClicksSelectClicksGrid(Gpf_SqlBuilder_SelectBuilder $select) {
        if ($this->isNetworkMerchant()) {
            $select->where->add('subrc.'.Pap_Db_Table_Campaigns::ACCOUNTID, '=', $this->getAccountId());
        }
    }

    public function createSelectBuilderCampaignRichListBox(Gpf_SqlBuilder_SelectBuilder $select) {
        if ($this->isNetworkMerchant()) {
            $select->where->add('c.'.Pap_Db_Table_Campaigns::ACCOUNTID, '=', $this->getAccountId());
        }
    }

    public function initDataColumnsClicksGrid(Pap_Merchants_Reports_ClicksGrid $grid) {
        $grid->addDataColumn("accountid", "rc.accountid");
    }

    public function initStatsSelectBuilder(Pap_Common_Reports_SelectBuilderCompoundParams $selectBuilderCompoundContext) {
        $selectBuilder = $selectBuilderCompoundContext->getSelectBuilder();
        $params = $selectBuilderCompoundContext->getParams();

        if ($this->isMasterMerchant() && $params->isAccountIdDefined()) {
            $selectBuilder->where->add('t.'.Pap_Db_Table_Campaigns::ACCOUNTID, '=', $params->getAccountId());
            return;
        }
        if ($this->isNetworkMerchant()) {
            if ($params->isAccountIdDefined() && Gpf_Session::getAuthUser()->hasPrivilege(Pap_Privileges::ACCOUNT, Gpf_Privileges::P_READ)) {
                $selectBuilder->where->add('t.'.Pap_Db_Table_Campaigns::ACCOUNTID, '=', $params->getAccountId());
                return;
            }
            $selectBuilder->where->add('t.'.Pap_Db_Table_Campaigns::ACCOUNTID, '=', $this->getAccountId());
        }
    }

    public function initBannersSelectBuilder(Gpf_SqlBuilder_SelectBuilder $selectBuilder) {
        if ($this->isNetworkMerchant()) {
            $selectBuilder->where->add('b.'.Pap_Db_Table_Banners::ACCOUNT_ID, '=', Gpf_Session::getAuthUser()->getAccountId());
        }
    }

    public function setMailReportAccountId(Gpf_Data_Record $context) {
        if ($context->get('accountId') !== Gpf_Db_Account::DEFAULT_ACCOUNT_ID) {
            $context->get('statsParams')->setAccountId($context->get('accountId'));
        }
    }

    public function addIcon(Pap_Common_IconSet $iconSet) {
        $iconSet->addIcon('NetworkAccounts', 'icon-networkaccounts');
        $iconSet->addIcon('NetworkAccountAdd', 'icon-networkaccountadd');
        $iconSet->addIcon('NetworkAccountsFields', 'icon-networkaccountfields');
    }

    public function initSettings(Gpf_Settings_Gpf $context) {
        $context->addDbSetting(Pap_Features_AffiliateNetwork_Definition::NETWORK_INVOICE);
        $context->addDbSetting(Pap_Features_AffiliateNetwork_Definition::NETWORK_PROFORMA_INVOICE);
        $context->addDbSetting(Pap_Features_AffiliateNetwork_Definition::ACCOUNT_APPROVAL, 'M');
        $context->addDbSetting(Pap_Features_AffiliateNetwork_Definition::ACCOUNT_POST_SIGNUP_TYPE, 'page');
        $context->addDbSetting(Pap_Features_AffiliateNetwork_Definition::ACCOUNT_POST_SIGNUP_URL, '');
        $context->addDbSetting(Pap_Features_AffiliateNetwork_Definition::MERCHANT_AGREEMENT, '');
        $context->addDbSetting(Pap_Features_AffiliateNetwork_Definition::NOTIFICATION_ON_NEW_ACCOUNT_SIGNUP, Gpf::YES);
        $context->addDbSetting(Pap_Features_AffiliateNetwork_Definition::AFF_NOTIFICATION_ON_CAMPAIGNS_SUSPENDED, Gpf::YES);
        $context->addDbSetting(Pap_Features_AffiliateNetwork_Definition::NOTIFICATION_ON_NEW_ACCOUNT_SIGNUP_BEFORE_APPROVAL, Gpf::YES);
        $context->addDbSetting(Pap_Features_AffiliateNetwork_Definition::NOTIFICATION_ON_REQUEST_PAYMENT, Gpf::YES);
        $context->addDbSetting(Pap_Features_AffiliateNetwork_Definition::NOTIFICATION_ON_INVOICE_PAID, Gpf::YES);
        $context->addDbSetting(Pap_Features_AffiliateNetwork_Definition::NOTIFICATION_ON_SUSPENDED, Gpf::YES);
        $context->addDbSetting(Pap_Features_AffiliateNetwork_Definition::NOTIFICATION_ON_DECLINED, Gpf::YES);
        $context->addDbSetting(Pap_Features_AffiliateNetwork_Definition::FORCE_MERCHANT_AGREEMENT_ACCEPTANCE_SETTING_NAME, Gpf::NO);
        $context->addDbSetting(Pap_Features_AffiliateNetwork_Definition::DEFAULT_NETWORK_MERCHANT_ROLE_SETTING_NAME, Pap_Features_AffiliateNetwork_Definition::NETWORK_MERCHANT_ROLE);
    }

    public function saveEmailNotifications(Gpf_Rpc_Form $form) {
        if ($this->isMasterMerchant() && $form->existsField(Pap_Features_AffiliateNetwork_Definition::NOTIFICATION_ON_NEW_ACCOUNT_SIGNUP)) {
            Gpf_Settings::set(Pap_Features_AffiliateNetwork_Definition::NOTIFICATION_ON_NEW_ACCOUNT_SIGNUP,
            $form->getFieldValue(Pap_Features_AffiliateNetwork_Definition::NOTIFICATION_ON_NEW_ACCOUNT_SIGNUP));
        }
    }

    public function loadEmailNotifications(Gpf_Rpc_Form $form) {
        if ($this->isMasterMerchant()) {
            $form->setField(Pap_Features_AffiliateNetwork_Definition::NOTIFICATION_ON_NEW_ACCOUNT_SIGNUP,
            Gpf_Settings::get(Pap_Features_AffiliateNetwork_Definition::NOTIFICATION_ON_NEW_ACCOUNT_SIGNUP));
        }
    }

    public function saveAffEmailNotifications(Gpf_Rpc_Form $form) {
        Gpf_Settings::set(Pap_Features_AffiliateNetwork_Definition::AFF_NOTIFICATION_ON_CAMPAIGNS_SUSPENDED,
        $form->getFieldValue(Pap_Features_AffiliateNetwork_Definition::AFF_NOTIFICATION_ON_CAMPAIGNS_SUSPENDED));
    }

    public function loadAffEmailNotifications(Gpf_Rpc_Form $form) {
        $form->setField(Pap_Features_AffiliateNetwork_Definition::AFF_NOTIFICATION_ON_CAMPAIGNS_SUSPENDED,
        Gpf_Settings::get(Pap_Features_AffiliateNetwork_Definition::AFF_NOTIFICATION_ON_CAMPAIGNS_SUSPENDED));
    }

    public function onStatusChange(Pap_account $account) {
        if ($account->getStatus() === Pap_Account::SUSPENDED) {
            $this->onAccountSuspended($account);
            return;
        }
        if ($account->getStatus() === Pap_account::DECLINED) {
            $this->onAccountDeclined($account);
            return;
        }
        if ($account->getStatus() === Pap_account::APPROVED) {
            $this->onAccountApproved($account);
            return;
        }
    }

    public function loadSetting(Gpf_ApplicationSettings $context) {
        if ($this->isNetworkMerchant()) {
            try {
                $account = Pap_Account::getAccount($this->getAccountId());
                $context->addValue('accountstatus', $account->getStatus());
            } catch (Gpf_DbEngine_NoRowException $e) {
                $context->addValue('accountstatus', Pap_account::DECLINED);
            }
        }
    }

    public function modifyCampaignsList(Gpf_SqlBuilder_SelectBuilder $selectBuilder) {
        $dbUser = new Gpf_Db_User();
        $dbUser->setId(Gpf_Session::getAuthUser()->getAccountUserId());
        $dbUser->load();
        if($dbUser->getRoleId() == Pap_Application::DEFAULT_ROLE_MERCHANT) {
            $selectBuilder->where = new Gpf_SqlBuilder_WhereClause();
        }
    }

    public function processChangeTransactionStatus(Pap_Common_Transaction $transaction) {
        $this->checkTransactionAccount($transaction);
    }

    public function processRefundChargeback(Pap_Common_Transaction $transaction) {
        $this->checkTransactionAccount($transaction);
    }

    private function checkTransactionAccount(Pap_Common_Transaction $transaction) {
        if ($this->isNetworkMerchant() && $transaction->getAccountId() != $this->getAccountId()) {
            throw new Gpf_Exception("The transaction is assigned to another account as logged user!");
        }
    }

    public function saveNetworkOwner(Gpf_Db_AuthUser $dbRow) {
        $gpfUser = new Gpf_Db_User();
        $gpfUser->setAuthId($dbRow->getId());
        $gpfUser->setAccountId(Gpf_Session::getAuthUser()->getAccountId());
        $gpfUser->setRoleId(Pap_Application::DEFAULT_ROLE_MERCHANT);
        try {
            $gpfUser->loadFromData(array(Gpf_Db_Table_Users::AUTHID, Gpf_Db_Table_Users::ACCOUNTID, Gpf_Db_Table_Users::ROLEID));
        } catch (Exception $e) {
            return;
        }

        $account = new Pap_Account();
        $account->setId(Gpf_Session::getAuthUser()->getAccountId());
        $account->load();

        $account->setName($dbRow->getFirstName() . ' ' . $dbRow->getLastName());
        $account->setEmail($dbRow->getUsername());
        $account->save();
    }

    public function clearDefaultRole(Gpf_Rpc_Action $action) {
        foreach ($action->getIds() as $id) {
            if ($id != Gpf_Settings::get(Pap_Features_AffiliateNetwork_Definition::DEFAULT_NETWORK_MERCHANT_ROLE_SETTING_NAME)) {
                continue;
            }
            try {
                $role = new Gpf_Db_Role();
                $role->setId($id);
                $role->load();
            } catch (Gpf_DbEngine_NoRowException $e) {
                Gpf_Settings::set(Pap_Features_AffiliateNetwork_Definition::DEFAULT_NETWORK_MERCHANT_ROLE_SETTING_NAME, Pap_Features_AffiliateNetwork_Definition::NETWORK_MERCHANT_ROLE);
            }
        }
    }

    protected function onAccountApproved(Pap_Account $account) {
        try {
            $this->processAccountApproved($account->getMerchant(), $account);
        } catch (Gpf_DbEngine_NoRowException $e) {
        }
    }

    protected function onAccountSuspended(Pap_Account $account) {
        if (Gpf_Settings::get(Pap_Features_AffiliateNetwork_Definition::NOTIFICATION_ON_SUSPENDED) === Gpf::YES) {
            $this->sendAccountMail(new Pap_Features_AffiliateNetwork_Mail_AccountSuspendedMerchant(), $account);
        }
        if (Gpf_Settings::get(Pap_Features_AffiliateNetwork_Definition::AFF_NOTIFICATION_ON_CAMPAIGNS_SUSPENDED) === Gpf::YES) {
            $this->sendSuspendedCampaignsToAffiliates($account);
        }
    }

    protected function onAccountDeclined(Pap_Account $account) {
        if (Gpf_Settings::get(Pap_Features_AffiliateNetwork_Definition::NOTIFICATION_ON_DECLINED) === Gpf::YES) {
            $this->sendAccountMail(new Pap_Features_AffiliateNetwork_Mail_AccountDeclinedMerchant(), $account);
        }
    }

    protected function sendSuspendedCampaignsToAffiliates(Pap_Account $account) {
        $campaignsSuspendedSendTask = new Pap_Features_AffiliateNetwork_Mail_CampaignsSuspendedSendTask();
        $campaignsSuspendedSendTask->setAccount($account);
        $campaignsSuspendedSendTask->run();
    }

    protected function sendAccountMail(Pap_Features_AffiliateNetwork_Mail_Account $mail, Pap_Account $account) {
        $mail->setAccount($account);
        $mail->addRecipient($account->getEmail());
        $mail->send();
    }

    protected function processAccountApproved(Pap_Common_User $accountMerchant, Pap_Account $account) {
        $accountMerchant->setStatus($account->getStatus());
        if ($accountMerchant->isFirstChangeStatus()) {
            $accountMerchant->save();
            $this->sendSignupApprovedEmail($accountMerchant, $account);
        }
    }

    protected function sendSignupApprovedEmail(Pap_Common_User $accountMerchant, Pap_Account $account) {
        $mail = new Pap_Features_AffiliateNetwork_Mail_AccountNewAccountSignupApproved();
        $mail->setUser($accountMerchant);
        $this->sendAccountMail($mail, $account);
    }

    private function isMasterMerchant() {
        return Gpf_Session::getAuthUser()->isMasterMerchant();
    }

    private function isNetworkMerchant() {
        return Gpf_Session::getAuthUser()->isNetworkMerchant();
    }

    private function getAccountId() {
        return Gpf_Session::getAuthUser()->getAccountId();
    }

    public function addApplicationSettings(Pap_ApplicationSettings $appSettings) {
        $appSettings->addValue(Pap_Features_AffiliateNetwork_Definition::DEFAULT_NETWORK_MERCHANT_ROLE_SETTING_NAME, Gpf_Settings::get(Pap_Features_AffiliateNetwork_Definition::DEFAULT_NETWORK_MERCHANT_ROLE_SETTING_NAME));
    }
}
?>