<?php
/**
 *   @copyright Copyright (c) 2007 Quality Unit s.r.o.
 *   @package PostAffiliatePro
 *   @author Milos Jancovic
 *   @since Version 1.0.0
 *   $Id: CustomerForm.class.php 19572 2008-08-01 16:43:19Z mjancovic $
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.postaffiliatepro.com/licenses/license
 *
 */

/**
 * @package PostAffiliatePro
 */
class Pap_Features_AffiliateNetwork_NetworkInvoiceForm extends Gpf_View_FormService {

	/**
	 * @service invoice read_own
	 * @param Gpf_Rpc_Params $params
	 * @return Gpf_Rpc_Form
	 */
	public function load(Gpf_Rpc_Params $params) {
		return parent::load($params);
	}
	
    /**
     * @service invoice read_own
     * @param $fields
     * @return Gpf_Rpc_Form
     */
    public function send_mail(Gpf_Rpc_Params $params) {
        $action = new Gpf_Rpc_Action($params);
        $action->setErrorMessage($this->_("Error sending mail"));
        $action->setInfoMessage($this->_("Mail send"));
        try {
            $invoice = new Pap_Db_Invoice();
            $invoice->setId($action->getParam('id'));
            $this->loadRow($invoice);
            if ($invoice->getStatus() === Pap_Common_Constants::PSTATUS_PAID) {
                $this->sendInvoiceMail(new Pap_Features_AffiliateNetwork_Mail_InvoicePaid(), $invoice);
            } else {
                $this->sendInvoiceMail(new Pap_Features_AffiliateNetwork_Mail_RequestPayment(), $invoice);
            }
            $action->addOk();
        } catch (Exception $e) {
            $action->addError();
        }
        return $action;
    }
	
	/**
	 * @return Pap_Db_Invoice
	 */
	protected function createDbRowObject() {
		return new Pap_Db_Invoice();
	}

	/**
	 * @return string
	 */
	protected function getDbRowObjectName() {
		return $this->_("Invoice");
	}
	
    protected function loadRow(Gpf_DbEngine_RowBase $row) {
    	$row->setAccountId(Gpf_Session::getAuthUser()->getAccountId());
        try {
            $row->loadFromData(array(Pap_Db_Table_Invoices::ID, Pap_Db_Table_Invoices::ACCOUNTID));
        } catch (Gpf_DbEngine_NoRow $e) {
            throw new Gpf_Exception($this->_("%s does not exist", $this->getDbRowObjectName()));
        }
    }
    
    protected function sendInvoiceMail(Pap_Features_AffiliateNetwork_Mail_Invoice $mail, Pap_Db_Invoice $invoice) {
        try {
            $account = Pap_Account::getAccount($invoice->getAccountId());
            $mail->setAccount($account);
            $mail->setInvoice($invoice);
            $mail->addRecipient($account->getEmail());
            $mail->setFromEmail(Pap_Common_User::getMerchantEmail());
            $mail->send();
        } catch (Gpf_Exception $e) {
        }
    }
}