<?php
/**
 *   @copyright Copyright (c) 2007 Quality Unit s.r.o.
 *   @package PostAffiliatePro
 *   @author Milos Jancovic
 *   @since Version 1.0.0
 *   $Id: CustomerForm.class.php 19572 2008-08-01 16:43:19Z mjancovic $
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.postaffiliatepro.com/licenses/license
 *
 */

/**
 * @package PostAffiliatePro
 */
class Pap_Features_AffiliateNetwork_NetworkInvoicePreview extends Gpf_Object implements Gpf_Rpc_Serializable {

	private $invoice;

	/**
	 * @service network_invoice_format read
	 * @param $params
	 * @return Gpf_Rpc_Form
	 */
	public function preview(Gpf_Rpc_Params $params) {
		$form = new Gpf_Rpc_Form($params);
		$invoice = $this->createInvoice();
		try {
			$invoice->setAccount($this->getAccount($form->getFieldValue('accountid')));
		} catch (Gpf_Exception $e) {
			$this->invoice = $this->_("You have to select account");
			return $this;
		}
		$invoice->setInvoice($this->createExampleInvoice());
		$this->invoice = $invoice->generate($form->getFieldValue('invoice'));
		return $this;
	}

	public function toObject() {
		throw new Gpf_Exception('Unsupported');
	}

	public function toText() {
		return $this->invoice;
	}

	/**
	 * @return Pap_Features_AffiliateNetwork_Invoice
	 */
	protected function createInvoice() {
		return new Pap_Features_AffiliateNetwork_Invoice();
	}

	/**
	 * @throws Gpf_DbEngine_NoRowException
	 * @throws Gpf_DbEngine_TooManyRowsException
	 * @param $accountId
	 * @return Pap_Account
	 */
	protected function getAccount($accountId) {
		return Pap_Account::getAccount($accountId);
	}

	/**
	 * @return Pap_Db_Invoice
	 */
	private function createExampleInvoice() {
		$date = new Gpf_DateTime();
		$invoice = new Pap_Db_Invoice();
		$invoice->setDateTo($date->toLocaleDate());
		$date->addDay(-30);
		$invoice->setDateFrom($date->toLocaleDate());
		$date->addDay(44);
		$invoice->setDueDate($date->toLocaleDate());
		$invoice->setNumber('123456');
		$invoice->setAmount(1000);
		$invoice->setMerchantNote($this->_('Example merchant note'));
		return $invoice;
	}
}
?>