<?php
/**
 *   @copyright Copyright (c) 2007 Quality Unit s.r.o.
 *   @package PostAffiliatePro
 *   @author Milos Jancovic
 *   @since Version 1.0.0
 *   $Id: CustomerForm.class.php 19572 2008-08-01 16:43:19Z mjancovic $
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.postaffiliatepro.com/licenses/license
 *
 */

/**
 * @package PostAffiliatePro
 */
class Pap_Features_AffiliateNetwork_Stats_AccountingComputer extends Gpf_Object {

	/**
	 * @var Pap_Stats_Params
	 */
	private $params;
	private $count = null;
	private $amount = null;

	public function __construct(Pap_Stats_Params $params) {
		$this->params = $params;
	}

	/**
	 * @return Pap_Features_AffiliateNetwork_Stats_AccountingData
	 */
	public function getCount() {
		return $this->get($this->count);
	}

	/**
	 * @return Pap_Features_AffiliateNetwork_Stats_AccountingData
	 */
	public function getAmount() {
		return $this->get($this->amount);
	}

	private function get(&$variable) {
		if (is_null($variable)) {
			$this->init();
		}
		return $variable;
	}

	private function init() {
		$this->amount = new Pap_Features_AffiliateNetwork_Stats_AccountingData();
		$this->count = new Pap_Features_AffiliateNetwork_Stats_AccountingData();

		foreach ($this->createSelect()->getAllRowsIterator() as $row) {
			$this->processRow($row);
		}
		
		$this->processCommissionFromTransactions();
	}
	
	private function processCommissionFromTransactions() {	    
	    try {
	        $nextInvoiceDateFrom = new Gpf_DateTime(Pap_Db_Table_Invoices::getInstance()->getNextInvoiceDateFrom($this->params->getAccountId()));
	        if (!$this->params->isDateApprovedFromDefined() || $this->params->getDateApprovedFrom()->compare($nextInvoiceDateFrom) < 0) {        	
	        	$this->params->setDateApprovedFrom($nextInvoiceDateFrom);
	        }
	    } catch (Gpf_Exception $e) {
	        // no accounting record so far
	    }
        $transactions = new Pap_Stats_Transactions($this->params);
        $this->amount->addCommission($transactions->getCommission()->getApproved());
        $this->amount->addCommission($transactions->getCommission()->getPaid());
	}

	private function processRow(Gpf_Data_Record $row) {
	    $count = $row->get('count');
	    $amount = $row->get('amount');
	    $type = $row->get('type');
	    if ($type === Pap_Db_Accounting::TYPE_FEE_FIXED || $type === Pap_Db_Accounting::TYPE_FEE_PERCENTAGE) {
	        $this->count->addFee($count);
            $this->amount->addFee($amount);
        }
        if ($type === Pap_Db_Accounting::TYPE_COMMISSSION) {
            $this->count->addCommission($count);
            $this->amount->addCommission($amount);
        }
	}

	/**
	 * @return Gpf_SqlBuilder_SelectBuilder
	 */
	private function createSelect() {
		$select = new Gpf_SqlBuilder_SelectBuilder();
		$select->select->add('SUM(' . Pap_Db_Table_Accountings::AMOUNT . ')', 'amount');
		$select->select->add(Pap_Db_Table_Accountings::TYPE, 'type');
		$select->select->add('COUNT(' . Pap_Db_Table_Accountings::ID . ')', 'count');
		$select->from->add(Pap_Db_Table_Accountings::getName());
		$this->buildWhere($select->where);
		$this->buildGroupBy($select->groupBy);
		return $select;
	}

	private function buildWhere(Gpf_SqlBuilder_WhereClause $where) {
		if ($this->params->isAccountIdDefined()) {
			$where->add(Pap_Db_Table_Accountings::ACCOUNTID, '=', $this->params->getAccountId());
		}
		if ($this->params->isDateFromDefined()) {
			$where->add(Pap_Db_Table_Accountings::DATE_INSTERTED, '>=', $this->params->getDateFrom()->toDateTime());
		}
		if ($this->params->isDateToDefined()) {
			$where->add(Pap_Db_Table_Accountings::DATE_INSTERTED, '<=', $this->params->getDateTo()->toDateTime());
		}
	}

	private function buildGroupBy(Gpf_SqlBuilder_GroupByClause $groupBy) {
		$groupBy->add(Pap_Db_Table_Accountings::TYPE);
	}
}

?>