<?php
/**
 *   @copyright Copyright (c) 2007 Quality Unit s.r.o.
 *   @package PostAffiliatePro
 *   @author Milos Jancovic
 *   @since Version 1.0.0
 *   $Id: CustomerForm.class.php 19572 2008-08-01 16:43:19Z mjancovic $
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.postaffiliatepro.com/licenses/license
 *
 */

/**
 * @package PostAffiliatePro
 */
class Pap_Features_AffiliateNetwork_Stats_InvoiceComputer extends Gpf_Object {

	/**
	 * @var Pap_Stats_Params
	 */
	private $params;
	private $amount = null;

	public function __construct(Pap_Stats_Params $params) {
		$this->params = $params;
	}

	/**
	 * @return Pap_Features_AffiliateNetwork_Stats_InvoiceData
	 */
	public function getAmount() {
		return $this->get($this->amount);
	}

	private function get(&$variable) {
		if (is_null($variable)) {
			$this->init();
		}
		return $variable;
	}

	private function init() {
		$this->amount = new Pap_Features_AffiliateNetwork_Stats_InvoiceData();

		foreach ($this->createSelect()->getAllRowsIterator() as $row) {
			$this->amount->add($row, 'amount');
		}
	}

	/**
	 * @return Gpf_SqlBuilder_SelectBuilder
	 */
	private function createSelect() {
		$select = new Gpf_SqlBuilder_SelectBuilder();
		$select->select->add('SUM(' . Pap_Db_Table_Invoices::AMOUNT . ')', 'amount');
		$select->select->add(Pap_Db_Table_Invoices::STATUS, 'type');
		$select->from->add(Pap_Db_Table_Invoices::getName());
		$this->buildWhere($select->where);
		$this->buildGroupBy($select->groupBy);
		return $select;
	}

	private function buildWhere(Gpf_SqlBuilder_WhereClause $where) {
		if ($this->params->isAccountIdDefined()) {
			$where->add(Pap_Db_Table_Invoices::ACCOUNTID, '=', $this->params->getAccountId());
		}
		if ($this->params->isDateFromDefined()) {
			$where->add(Pap_Db_Table_Invoices::DATE_CREATED, '>=', $this->params->getDateFrom()->toDateTime());
		}
		if ($this->params->isDateToDefined()) {
			$where->add(Pap_Db_Table_Invoices::DATE_CREATED, '<=', $this->params->getDateTo()->toDateTime());
		}
	}

	private function buildGroupBy(Gpf_SqlBuilder_GroupByClause $groupBy) {
		$groupBy->add(Pap_Db_Table_Invoices::STATUS);
	}
}

?>