<?php
/**
 *   @copyright Copyright (c) 2007 Quality Unit s.r.o.
 *   @package PostAffiliatePro
 *   @author Matej Kendera
 *   @since Version 1.0.0
 *   $Id: Banner.class.php 16622 2008-03-21 09:39:50Z aharsani $
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.postaffiliatepro.com/licenses/license
 *
 */

/**
 * @package PostAffiliatePro
 */
class Pap_Features_CampaignScheduler_Action_ChangeCampaignStatusBaseAction extends Gpf_Object {
    
    /**
     * @var Gpf_Log_Logger
     */
    private $logger;

    protected $targetStatus;

    protected $campaignId;

    public function __construct($campaignId, $targetStatus) {
        $this->targetStatus = $targetStatus;
        $this->campaignId = $campaignId;
    }

    /**
     * @throws Gpf_Exception
     */
    public function execute() {
        $this->log('Executing action: Changing Campaign Status...');
        $this->changeCampaignStatus();
    }

    private function changeCampaignStatus() {
        $campaign = Pap_Common_Campaign::getCampaignById($this->campaignId);
        if ($campaign == null) {
            $this->log('Error loading campaig for transaction from campaignId: ' . $this->campaignId);
            return;
        }
        if ($campaign->getCampaignStatus() == $this->getStatusTo()) {
            $this->log('Campaign is in status: ' . $this->getFormattedStatusTo());
            return;
        }
        $campaign->setStatus($this->getStatusTo());
        $campaign->update(array(Pap_Db_Table_Campaigns::STATUS));
        $this->log('Campaign status changed to: ' . $this->getFormattedStatusTo());
    }

    private function getStatusTo() {
        return $this->targetStatus;
    }

    private function getFormattedStatusTo() {
        switch ($this->getStatusTo()) {
            case Pap_Db_Campaign::CAMPAIGN_STATUS_STOPPED_INVISIBLE:
                return $this->_('stopped');
            case Pap_Db_Campaign::CAMPAIGN_STATUS_STOPPED:
                return $this->_('paused');
            case Pap_Db_Campaign::CAMPAIGN_STATUS_ACTIVE:
                return $this->_('active');
        }
    }

    public static function toString() {
        return Gpf_Lang::_('change campaign status to');
    }

    public function getString() {
        return sprintf(self::toString() . ' %s', $this->getFormattedStatusTo());
    }

    /**
     * @return Gpf_Log_Logger
     */
    private function getLogger() {
        if ($this->logger === null) {
            $this->logger = Pap_Logger::create(Pap_Common_Constants::TYPE_ACTION);
        }
        return $this->logger;
    }

    private function setLogGroup($goupId) {
        $this->getLogger()->setGroup($goupId);
    }

    private function log($message, $logLevel = Gpf_Log::DEBUG) {
        $this->getLogger()->log($message, $logLevel);
    }
}
?>
