<?php
/**
 *   @copyright Copyright (c) 2007 Quality Unit s.r.o.
 *   @author Matej Kendera
 *   @package PostAffiliatePro
 *   @since Version 1.0.0
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.postaffiliatepro.com/licenses/license
 *
 */

/**
 * @package PostAffiliatePro
 */
class Pap_Features_CampaignScheduler_Main extends Gpf_Plugins_Handler {

    const TIME_TYPE_DAILY = 'D';
    const TIME_TYPE_MONTHLY = 'M';
    const TIME_TYPE_WEEKLY = 'W';
    const TIME_TYPE_SPECIFIC_TIME = 'T';

    const STATS_WHAT_TIME_CONDITION = 'D';

    /**
     * @var Gpf_Log_Logger
     */
    private $logger = null;

    public static function getHandlerInstance() {
        return new Pap_Features_CampaignScheduler_Main();
    }

    public function checkRules(Pap_Common_Transaction $transaction) {
        $this->setLogGroup($transaction->getLogGroupId());
        $this->log('Campaign Scheduler started');
        foreach($this->getRules($transaction->getCampaignId()) as $ruleRecord) {
            $campainDbRule = new Pap_Db_CampaignRule();
            $campainDbRule->fillFromRecord($ruleRecord);
            $modelRule = new Pap_Model_CampaignRule($campainDbRule, $transaction);
            $rule = new Pap_Features_CampaignScheduler_Rule($modelRule);
            try {
                $this->log('Processing campign rule ' . $rule->getId() . ': change campaign status to: ' . $rule->getAction());
                $rule->executeAction();
                $this->log('Rule completed');
            } catch (Exception $e) {
                $this->log(sprintf('Rule %s failed. Reason: %s', $rule->getId(), $e->getMessage()), Gpf_Log::ERROR);
            }
        }
        $this->log('Campaign Scheduler ended');
    }

    public function checkTimeRules(Pap_Common_TransactionCompoundContext $transactionCompoundContext) {
        $transaction = $transactionCompoundContext->getTransaction();
        $this->setLogGroup($transaction->getLogGroupId());
        $this->log('Campaign Scheduler time rules started');
        foreach($this->getRules($transaction->getCampaignId(), true) as $ruleRecord) {
            $campainDbRule = new Pap_Db_CampaignRule();
            $campainDbRule->fillFromRecord($ruleRecord);
            $timeRule = new Pap_Features_CampaignScheduler_TimeRule($campainDbRule);
            try {
                $this->log('Processing campign time rule ' . $timeRule->getId() . ': ' . $timeRule->getString());
                $timeRule->executeAction();
                $this->log('Rule completed');
            } catch (Exception $e) {
                $this->log(sprintf('Rule %s failed. Reason: %s', $timeRule->getId(), $e->getMessage()), Gpf_Log::ERROR);
            }
        }
        $this->log('Campaign Scheduler time rules ended');
    }

    /**
     * @return Gpf_SqlBuilder_SelectIterator
     */
    public function getRules($campaignId = null, $timeConditionsOnly = false) {
        $select = new Gpf_SqlBuilder_SelectBuilder();
        $select->select->addAll(Pap_Db_Table_CampaignRules::getInstance());
        $select->from->add(Pap_Db_Table_CampaignRules::getName());
        if ($campaignId != null) {
            $select->where->add(Pap_Db_Table_CampaignRules::CAMPAIGN_ID, '=', $campaignId);
        }
        if ($timeConditionsOnly) {
            $select->where->add(Pap_Db_Table_CampaignRules::STATS_WHAT, '=', self::STATS_WHAT_TIME_CONDITION);
        } else {
            $select->where->add(Pap_Db_Table_CampaignRules::STATS_WHAT, '!=', self::STATS_WHAT_TIME_CONDITION);
        }
        return $select->getAllRowsIterator();
    }

    /**
     * @return Gpf_SqlBuilder_SelectIterator
     */
    public function getTimeRules() {
        return $this->getRules(null, true);
    }
    
    /**
     * @return Gpf_Log_Logger
     */
    private function getLogger() {
        if ($this->logger === null) {
            $this->logger = Pap_Logger::create(Pap_Common_Constants::TYPE_ACTION);
        }
        return $this->logger;
    }

    private function setLogGroup($goupId) {
        $this->getLogger()->setGroup($goupId);
    }

    private function log($message, $logLevel = Gpf_Log::DEBUG) {
        $this->getLogger()->log($message, $logLevel);
    }
}
?>
