<?php
/**
 *   @copyright Copyright (c) 2007 Quality Unit s.r.o.
 *   @package PostAffiliatePro
 *   @author Matej Kendera
 *   @since Version 1.0.0
 *   $Id: CustomerForm.class.php 19572 2008-08-01 16:43:19Z mjancovic $
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.postaffiliatepro.com/licenses/license
 *
 */

/**
 * @package PostAffiliatePro
 */
class Pap_Features_CampaignScheduler_SchedulerForm extends Gpf_View_FormService {

    /**
     * @return Pap_Db_CampaignRule
     */
    protected function createDbRowObject() {
        return new Pap_Db_CampaignRule();
    }

    /**
     * @return string
     */
    protected function getDbRowObjectName() {
        return $this->_("Campaign Scheduling Rule");
    }

    /**
     *
     * @service campaign_scheduler add
     * @param $fields
     * @return Gpf_Rpc_Form
     */
    public function add(Gpf_Rpc_Params $params) {
        return parent::add($params);
    }

    /**
     *
     * @service campaign_scheduler read
     * @param $fields
     * @return Gpf_Rpc_Form
     */
    public function load(Gpf_Rpc_Params $params) {
        return parent::load($params);
    }

    protected function loadForm(Gpf_Rpc_Form $form, Gpf_DbEngine_RowBase $dbRow) {
        parent::loadForm($form, $dbRow);
        $form->setField(Pap_Db_Table_CampaignRules::TIME_HOUR, $dbRow->getTimeHourInClientTimezone());
        $form->setField(Pap_Db_Table_CampaignRules::TIME_DATE, $dbRow->getTimeDateInClientTimezone());
        if ($dbRow->getTimeType() == Pap_Features_CampaignScheduler_Main::TIME_TYPE_WEEKLY) {
            $form->setField('time_week_day', $dbRow->getTimeDayInClientTimezone());
        } else if ($dbRow->getTimeType() == Pap_Features_CampaignScheduler_Main::TIME_TYPE_MONTHLY) {
            $form->setField('time_month_day', $dbRow->getTimeDayInClientTimezone());
        }
    }

    /**
     *
     * @service campaign_scheduler write
     * @param $fields
     * @return Gpf_Rpc_Form
     */
    public function save(Gpf_Rpc_Params $params) {
        return parent::save($params);
    }

    protected function fillAdd(Gpf_Rpc_Form $form, Gpf_DbEngine_RowBase $dbRow) {
        parent::fillAdd($form, $dbRow);
        $this->fillTimeDay($form, $dbRow);
    }

    protected function fillSave(Gpf_Rpc_Form $form, Gpf_DbEngine_RowBase $dbRow) {
        parent::fillSave($form, $dbRow);
        $this->fillTimeDay($form, $dbRow);
    }

    private function fillTimeDay(Gpf_Rpc_Form $form, Gpf_DbEngine_RowBase $dbRow) {
        if ($form->getFieldValue('stats_what') == Pap_Features_CampaignScheduler_Main::STATS_WHAT_TIME_CONDITION) {
            $clientTimeHour = $form->getFieldValue(Pap_Db_Table_CampaignRules::TIME_HOUR);
            $dbRow->setTimeHour((int)$this->getServerHour($clientTimeHour));
            $dbRow->setTimeDate($this->getServerDate($form->getFieldValue(Pap_Db_Table_CampaignRules::TIME_DATE), $clientTimeHour));

            if ($form->getFieldValue('time_type') == Pap_Features_CampaignScheduler_Main::TIME_TYPE_WEEKLY) {
                $dbRow->setTimeDay($this->shiftWeekDay($form->getFieldValue('time_week_day'), $clientTimeHour));
            } else if ($form->getFieldValue('time_type') == Pap_Features_CampaignScheduler_Main::TIME_TYPE_MONTHLY) {
                $dbRow->setTimeDay($this->shiftMonthDay($form->getFieldValue('time_month_day'), $clientTimeHour));
            }
        }
    }

    private function getServerHour($clientTimeHour) {
        if ($clientTimeHour == null || $clientTimeHour == '') {
            return '';
        }
        $clientTimeHour = $clientTimeHour.':00';
        $date = new Gpf_DateTime(Gpf_Common_DateUtils::getTimestamp($clientTimeHour), false);
        return $date->getServerTime()->getHour();
    }

    private function getServerDate($clientDate, $clientTimeHour) {
        if ($clientDate == null || $clientDate == '') {
            return '';
        }
        $date = new Gpf_DateTime($clientDate);
        $date->addDay($this->getServerTimeDifferenceDays($clientTimeHour));
        return $date->toDate();
    }

    private function getServerTimeDifferenceDays($clientTimeHour) {
        if ($clientTimeHour == null || $clientTimeHour == '') {
            return 0;
        }
        $clientTimeHour = $clientTimeHour.':00';
        $date = new Gpf_DateTime(Gpf_Common_DateUtils::getTimestamp($clientTimeHour), false);
        if ($date->toDate() != $date->getServerTime()->toDate()) {
            return Gpf_Common_DateUtils::getExactDifference($date->toDate(), $date->getServerTime()->toDate(), Gpf_Common_DateUtils::DAY);
        }
        return 0;
    }

    private function shiftWeekDay($day, $clientTimeHour) {
        $day = $day+$this->getServerTimeDifferenceDays($clientTimeHour);
        return $day%7 < 0 ? 7 + $day%7 : $day%7;
    }

    private function shiftMonthDay($day, $clientTimeHour) {
        $day = $day+$this->getServerTimeDifferenceDays($clientTimeHour);
        return $day%31 < 1 ? 31 + $day%31 : $day%31;
    }

    protected function afterSave(Pap_Db_CampaignRule $dbRow, $saveType) {
        if ($saveType == Gpf_View_FormService::ADD) {
            //add
            if ($dbRow->getStatsWhat() == Pap_Features_CampaignScheduler_Main::STATS_WHAT_TIME_CONDITION) {
                $this->addPlannedTask($dbRow);
            }
        } else {
            //edit
            $this->removePlannedTask($dbRow);
            if ($dbRow->getStatsWhat() == Pap_Features_CampaignScheduler_Main::STATS_WHAT_TIME_CONDITION) {
                $this->addPlannedTask($dbRow);
            }
        }
    }

    /**
     * @param Pap_Db_CampaignRule $dbRow
     * @throws Pap_Features_CampaignScheduler_ExpiredTimeException
     */
    private function addPlannedTask(Pap_Db_CampaignRule $dbRow) {
        $plannedTask = $this->createPlannedTask();
        $plannedTask->setRecurrencePresetId($this->getRecurrencePresetForTimeType($dbRow->getTimeType()));
        $plannedTask->setParams(serialize(array(Pap_Features_CampaignScheduler_TimeConditionTask::TASK_PARAM_PLANNEDTASKID => $dbRow->getId())));
        $plannedTask->setLastPlanDate($this->getNextPlanDate($dbRow->getTimeType(), $dbRow->getTimeDay(), $dbRow->getTimeDate(), $dbRow->getTimeHour()));

        if ($dbRow->getTimeType() == Pap_Features_CampaignScheduler_Main::TIME_TYPE_SPECIFIC_TIME) {
            $task = new Gpf_Db_Task();
            $task->setClassName($plannedTask->getClassName());
            $task->setParams($plannedTask->getParams());
            $task->setAccountId($plannedTask->getAccountId());
            $task->setSleepUntil(new Gpf_DateTime($plannedTask->getLastPlanDate()));
            $task->save();
        } else {
            $plannedTask->insert();
        }
    }

    private function removePlannedTask(Pap_Db_CampaignRule $dbRow) {
        $plannedTask = $this->createPlannedTask();
        $plannedTask->setParams(serialize(array(Pap_Features_CampaignScheduler_TimeConditionTask::TASK_PARAM_PLANNEDTASKID => $dbRow->getId())));
        try {
            $plannedTask->loadFromData();
            $plannedTask->delete();
        } catch (Gpf_DbEngine_NoRowException $e) {
        }
        $task = new Gpf_Db_Task();
        $task->setClassName($plannedTask->getClassName());
        $task->setParams($plannedTask->getParams());
        try {
            $task->loadFromData(array(Gpf_Db_Table_Tasks::CLASSNAME, Gpf_Db_Table_Tasks::PARAMS));
            $task->delete();
        } catch (Gpf_DbEngine_NoRowException $e) {
        }
    }

    /**
     * @throws Pap_Features_CampaignScheduler_ExpiredTimeException
     */
    private function getNextPlanDate($timeType, $timeDay, $timeDate, $timeHour) {
        switch ($timeType) {
            case Pap_Features_CampaignScheduler_Main::TIME_TYPE_DAILY:
                $actualDate = new Gpf_DateTime();
                if ((int)$actualDate->getHour() >= (int)$timeHour) {
                    $actualDate->addDay();
                }
                $dateTime = new Gpf_DateTime($actualDate->toDate() .' '.$timeHour.':00:00');
                return $dateTime->toDateTime();
            case Pap_Features_CampaignScheduler_Main::TIME_TYPE_MONTHLY:
                $actualDate = new Gpf_DateTime();
                if ($actualDate->getDay() == $timeDay && (int)$actualDate->getHour() < (int)$timeHour) {
                    $dateTime = new Gpf_DateTime($actualDate->toDate() .' '.$timeHour.':00:00');
                    return $dateTime->toDateTime();
                }
                if ($actualDate->getDay() >= $timeDay) {
                    $actualDate->addMonth();
                }
                $dateTime = new Gpf_DateTime($actualDate->getYear().'-'.$actualDate->getMonth().'-'.$timeDay.' '.$timeHour.':00:00');
                return $dateTime->toDateTime();
            case Pap_Features_CampaignScheduler_Main::TIME_TYPE_WEEKLY:
                $actualDate = new Gpf_DateTime();
                if ($actualDate->getWeekDay() == $timeDay && (int)$actualDate->getHour() < (int)$timeHour) {
                    $dateTime = new Gpf_DateTime($actualDate->toDate() .' '.$timeHour.':00:00');
                    return $dateTime->toDateTime();
                }
                $dateTime = Gpf_DateTime::getNextWeekDay($timeDay);
                $dateTime = new Gpf_DateTime($dateTime->toDate() .' '.$timeHour.':00:00');
                return $dateTime->toDateTime();
            case Pap_Features_CampaignScheduler_Main::TIME_TYPE_SPECIFIC_TIME:
                $actualDate = new Gpf_DateTime();
                $dateTime = new Gpf_DateTime($timeDate.' '.$timeHour.':00:00');
                if ($actualDate->isAfter($dateTime)) {
                    throw new Pap_Features_CampaignScheduler_ExpiredTimeException();
                }

                return $dateTime->toDateTime();
        }

    }

    private function getRecurrencePresetForTimeType($timeType) {
        switch ($timeType) {
            case Pap_Features_CampaignScheduler_Main::TIME_TYPE_DAILY:
                return Pap_Db_CommissionType::RECURRENCE_DAILY;
            case Pap_Features_CampaignScheduler_Main::TIME_TYPE_MONTHLY:
                return Pap_Db_CommissionType::RECURRENCE_MONTHLY;
            case Pap_Features_CampaignScheduler_Main::TIME_TYPE_WEEKLY:
                return Pap_Db_CommissionType::RECURRENCE_WEEKLY;
            case Pap_Features_CampaignScheduler_Main::TIME_TYPE_SPECIFIC_TIME:
                return 'varied';
        }
    }

    /**
     * @service campaign_scheduler delete
     * @param $ids
     * @return Gpf_Rpc_Action
     */
    public function deleteRows(Gpf_Rpc_Params $params) {
        return parent::deleteRows($params);
    }

    protected function deleteRow(Gpf_DbEngine_RowBase $row) {
        parent::deleteRow($row);
        $this->removePlannedTask($row);
    }

    /**
     * @service campaign_scheduler read
     * @return Gpf_Data_RecordSet
     */
    public function loadConditions(Gpf_Rpc_Params $params) {
        return $this->createConditionsRecordSet();
    }

    private function createConditionsRecordSet() {
        $recordset = new Gpf_Data_RecordSet();
        $recordset->setHeader(array('id', 'name'));

        foreach($this->getConditions() as $code) {
            try {
                $recordset->add(array((string)$code,
                Pap_Common_Condition::toString($code)));
            } catch (Pap_Common_UnknownRuleException $e) {}
        }
        return $recordset;
    }

    private function getConditions() {
        $list = array();
        $list[] = Pap_Common_Condition::WHAT_NUMBER_OF_SALES;
        $list[] = Pap_Common_Condition::WHAT_NUMBER_OF_ACTIONS;
        $list[] = Pap_Common_Condition::WHAT_NUMBER_OF_RECURRING_SALES;
        $list[] = Pap_Common_Condition::WHAT_VALUE_OF_COMMISSIONS;
        $list[] = Pap_Common_Condition::WHAT_VALUE_OF_COMMISSIONS_FROM_FIRST_TIER;
        $list[] = Pap_Common_Condition::WHAT_VALUE_OF_TOTAL_COST_ALL;
        $list[] = Pap_Common_Condition::WHAT_VALUE_OF_TOTAL_COST;
        $list[] = Pap_Common_Condition::WHAT_VALUE_OF_ACTIONS_TOTAL_COST;
        $list[] = Pap_Common_Condition::WHAT_VALUE_OF_TOTAL_COST_IN_RECURRING_SALES;
        $list[] = Pap_Common_Condition::WHAT_VALUE_OF_TOTAL_COST_AND_TOTAL_COST_IN_RECURRING_SALES;
        $list[] = Pap_Common_Condition::WHAT_NUMBER_OF_SALES_AND_RECURRING_SALES;
        return $list;
    }

    /**
     * @return Gpf_Db_PlannedTask
     */
    private function createPlannedTask() {
        $plannedTask = new Gpf_Db_PlannedTask();
        $plannedTask->setClassName('Pap_Features_CampaignScheduler_TimeConditionTask');
        return $plannedTask;
    }
}

?>
