<?php
/**
 *   @copyright Copyright (c) 2007 Quality Unit s.r.o.
 *   @package PostAffiliatePro
 *   @author Matej Kendera
 *   @since Version 1.0.0
 *   $Id: SchedulerGrid.class.php 17234 2008-04-11 14:23:06Z mbebjak $
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.postaffiliatepro.com/licenses/license
 *
 */

/**
 * @package PostAffiliatePro
 */
class Pap_Features_CampaignScheduler_SchedulerGrid extends Gpf_View_GridService implements Gpf_View_Grid_HasRowFilter {

    protected function initViewColumns() {
        $this->addViewColumn(Pap_Db_Table_CampaignRules::STATUS_TO, $this->_("Change status to"), true);
        $this->addViewColumn('rule', $this->_("Rule"), true);
        $this->addViewColumn(parent::ACTIONS, $this->_("Actions"), false);
    }

    protected function initDataColumns() {
        $this->setKeyDataColumn('r.'.Pap_Db_Table_CampaignRules::ID);
        $this->addDataColumn('rule', 'NULL');
        $this->addDataColumn(Pap_Db_Table_CampaignRules::STATS_WHAT, 'r.'.Pap_Db_Table_CampaignRules::STATS_WHAT);
        $this->addDataColumn(Pap_Db_Table_CampaignRules::STATS_STATUS, 'r.'.Pap_Db_Table_CampaignRules::STATS_STATUS);
        $this->addDataColumn(Pap_Db_Table_CampaignRules::STATS_DATE, 'r.'.Pap_Db_Table_CampaignRules::STATS_DATE);
        $this->addDataColumn(Pap_Db_Table_CampaignRules::STATS_SINCE, 'r.'.Pap_Db_Table_CampaignRules::STATS_SINCE);
        $this->addDataColumn(Pap_Db_Table_CampaignRules::STATS_EQUATION, 'r.'.Pap_Db_Table_CampaignRules::STATS_EQUATION);
        $this->addDataColumn(Pap_Db_Table_CampaignRules::STATS_EQUATION_VALUE_1, 'r.'.Pap_Db_Table_CampaignRules::STATS_EQUATION_VALUE_1);
        $this->addDataColumn(Pap_Db_Table_CampaignRules::STATS_EQUATION_VALUE_2, 'r.'.Pap_Db_Table_CampaignRules::STATS_EQUATION_VALUE_2);
        $this->addDataColumn(Pap_Db_Table_CampaignRules::STATUS_TO, 'r.'.Pap_Db_Table_CampaignRules::STATUS_TO);
        $this->addDataColumn(Pap_Db_Table_CampaignRules::TIME_DATE, 'r.'.Pap_Db_Table_CampaignRules::TIME_DATE);
        $this->addDataColumn(Pap_Db_Table_CampaignRules::TIME_DAY, 'r.'.Pap_Db_Table_CampaignRules::TIME_DAY);
        $this->addDataColumn(Pap_Db_Table_CampaignRules::TIME_HOUR, 'r.'.Pap_Db_Table_CampaignRules::TIME_HOUR);
        $this->addDataColumn(Pap_Db_Table_CampaignRules::TIME_TYPE, 'r.'.Pap_Db_Table_CampaignRules::TIME_TYPE);
    }

    protected function initDefaultView() {
        $this->addDefaultViewColumn(Pap_Db_Table_CampaignRules::STATUS_TO, '', 'D');
        $this->addDefaultViewColumn('rule', '', 'N');
        $this->addDefaultViewColumn(parent::ACTIONS, '', 'N');
    }

    protected function buildWhere() {
        parent::buildWhere();
        $this->_selectBuilder->where->add('r.'.Pap_Db_Table_CampaignRules::CAMPAIGN_ID, '=', $this->getCampaignId());
    }

    protected function buildFrom() {
        $this->_selectBuilder->from->add(Pap_Db_Table_CampaignRules::getName(), 'r');
        $this->_selectBuilder->from->addInnerJoin(Pap_Db_Table_Campaigns::getName(), 'c',
		  'c.'.Pap_Db_Table_Campaigns::ID.'=r.'.Pap_Db_Table_CampaignRules::CAMPAIGN_ID);
    }

    /**
     * @param $row
     * @return DataRow or null
     */
    public function filterRow(Gpf_Data_Row $row) {
        $row->set('rule', 'Rule text');
        $campainDbRule = new Pap_Db_CampaignRule();
        $campainDbRule->fillFromRecord($row);
        if ($campainDbRule->getStatsWhat() != Pap_Features_CampaignScheduler_Main::STATS_WHAT_TIME_CONDITION) {
            $modelRule = new Pap_Model_CampaignRule($campainDbRule, new Pap_Common_Transaction());
            $rule = new Pap_Features_CampaignScheduler_Rule($modelRule);
            try {
                $row->set('rule', $rule->getString());
            } catch (Pap_Common_UnknownRuleException $e) {
                $row->set('rule', $this->_('Unknown rule (%s) - this rule will not be applied.', $rule->getId()));
            }
        } else {
            $timeRule = new Pap_Features_CampaignScheduler_TimeRule($campainDbRule);
            $row->set('rule', $timeRule->getString());
        }
        return $row;
    }

    private function getCampaignId() {
        if ($this->_params->exists('campaignid')) {
            return $this->_params->get('campaignid');
        }

        throw new Gpf_Exception($this->_('Campaign id is missing'));
    }

    /**
     * @service campaign_scheduler read
     * @return Gpf_Rpc_Serializable
     */
    public function getRows(Gpf_Rpc_Params $params) {
        return parent::getRows($params);
    }
}
?>
