<?php
/**
 *   @copyright Copyright (c) 2007 Quality Unit s.r.o.
 *   @package PostAffiliatePro
 *   @author Matej Kendera
 *   @since Version 1.0.0
 *   $Id: Banner.class.php 16622 2008-03-21 09:39:50Z aharsani $
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.postaffiliatepro.com/licenses/license
 *
 */

/**
 * @package PostAffiliatePro
 */
class Pap_Features_CampaignScheduler_TimeCondition extends Gpf_Object {

    /**
     *
     * @var Pap_Db_CampaignRule
     */
    private $rule;
    
    /**
     * @var Gpf_Log_Logger
     */
    private $logger;

    public function __construct(Pap_Db_CampaignRule $rule) {
        $this->rule = $rule;
        $this->logger = Pap_Logger::create(Pap_Common_Constants::TYPE_ACTION);
    }

    public function isValid() {
        return $this->evaluateDay($this->rule->getTimeType(), $this->rule->getTimeDay(), $this->rule->getTimeDate())
        && $this->evaluateHour($this->rule->getTimeHour());
    }

    private function evaluateDay($type, $day, $date) {
        switch ($type) {
            case Pap_Features_CampaignScheduler_Main::TIME_TYPE_DAILY:
                return true;
            case Pap_Features_CampaignScheduler_Main::TIME_TYPE_WEEKLY:
                $this->logger->log('Time rule condition actual day in week: '.$this->getActualDayInWeek().' EQUAL TO '.$day, Gpf_Log::INFO);
                return $this->getActualDayInWeek() == $day;
            case Pap_Features_CampaignScheduler_Main::TIME_TYPE_MONTHLY:
                $this->logger->log('Time rule condition actual day in month: '.$this->getActualDayInMonth().' EQUAL TO '.$day, Gpf_Log::INFO);
                return $this->getActualDayInMonth() == $day;
            case Pap_Features_CampaignScheduler_Main::TIME_TYPE_SPECIFIC_TIME:
                $this->logger->log('Time rule condition actual date: '.$this->getActualDatePart()->toLocaleDate().' EQUAL TO '.$date, Gpf_Log::INFO);
                $specificDate = new Gpf_DateTime($date);
                return $this->getActualDatePart()->toTimeStamp() == $specificDate->toTimeStamp();
        }
    }

    private function evaluateHour($hour) {
        $this->logger->log('Time rule condition actual hour: '.(int)$this->getActualHour().' EQUAL TO '.(int)$hour, Gpf_Log::INFO);
        return (int)$this->getActualHour() == (int)$hour;
    }
    /**
     * @return Gpf_DateTime
     */
    protected function creteActualDateTimeObject() {
        return new Gpf_DateTime();
    }

    private function getActualHour() {
        $actualDate = $this->creteActualDateTimeObject();
        return $actualDate->getHour();
    }

    private function getActualDayInWeek() {
        $actualDate = $this->creteActualDateTimeObject();
        return $actualDate->getWeekDay();
    }

    private function getActualDayInMonth() {
        $actualDate = $this->creteActualDateTimeObject();
        return $actualDate->getDay();
    }

    private function getActualDatePart() {
        $actualDate = $this->creteActualDateTimeObject();
        return new Gpf_DateTime($actualDate->toDate());
    }

    public function getText() {
        switch ($this->rule->getTimeType()) {
            case Pap_Features_CampaignScheduler_Main::TIME_TYPE_DAILY:
                return $this->_('Daily') . ' ' .  $this->getHourFormatted($this->rule->getTimeHourInClientTimezone());
            case Pap_Features_CampaignScheduler_Main::TIME_TYPE_WEEKLY:
                return $this->_('Weekly') . ' ' .$this->getDayInWeekFormatted($this->rule->getTimeDayInClientTimezone()). $this->getHourFormatted($this->rule->getTimeHourInClientTimezone());
            case Pap_Features_CampaignScheduler_Main::TIME_TYPE_MONTHLY:
                return $this->_('Monthly') . ' '.$this->getDayInMonthFormatted($this->rule->getTimeDayInClientTimezone()) . $this->getHourFormatted($this->rule->getTimeHourInClientTimezone());
            case Pap_Features_CampaignScheduler_Main::TIME_TYPE_SPECIFIC_TIME:
                return $this->_('At') . ' ' . $this->getDateFormatted($this->rule->getTimeDateInClientTimezone()) .' '. $this->getHour($this->rule->getTimeHourInClientTimezone());
        }
    }

    private function getDateFormatted($date) {
        return Gpf_Common_DateUtils::getDateInLocaleFormat(strtotime($date));
    }

    private function getHourFormatted($hour) {
        return $this->_('at %s', $this->getHour($hour));
    }

    private function getHour($hour) {
        return $hour.':00 ';
    }

    private function getDayInMonthFormatted($day) {
        return $this->_('on %s', $this->getOrdinalNumberName($day).' ');
    }

    private function getDayInWeekFormatted($day) {
        return $this->_('on %s', Gpf_Common_DateUtils::getDayName($day).' ');
    }

    private function getOrdinalNumberName($i) {
        switch ($i) {
            case 1:
                return $i . $this->_('st');
            case 2:
                return $i . $this->_('nd');
            case 3:
                return $i . $this->_('rd');
            default:
                return $i . $this->_('th');
        }
    }
}

?>
