<?php
/**
 *   @copyright Copyright (c) 2007 Quality Unit s.r.o.
 *   @author Matej Kendera
 *   @package PostAffiliatePro
 *   @since Version 1.0.0
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.postaffiliatepro.com/licenses/license
 *
 */

/**
 * @package PostAffiliatePro
 */
class Pap_Features_CampaignScheduler_TimeRule extends Gpf_Object {
    
    /**
     * @var Pap_Db_CampaignRule
     */
    private $campaignRule;
    
    /**
     * @var Pap_Features_CampaignScheduler_TimeCondition
     */
    private $condition = null;
    
    /**
     * @var Pap_Features_CampaignScheduler_Action_ChangeCampaignStatusBaseAction
     */
    private $action = null;
    
    public function __construct(Pap_Db_CampaignRule $campaignRule) {
        $this->campaignRule = $campaignRule;
    }

    public function executeAction($forceCondition = false) {
        if ($this->isConditionValid() || $forceCondition) {
            $this->getActionObject()->execute();
        } else {
            Gpf_Log::debug('Condition of rule is not valid - skipping');
        }
    }
    
    /**
     * @return Pap_Features_CampaignScheduler_Action_ChangeCampaignStatusBaseAction
     */
    private function getActionObject() {
        if ($this->action === null) {
            $this->action = new Pap_Features_CampaignScheduler_Action_ChangeCampaignStatusBaseAction($this->campaignRule->getCampaignId(), $this->campaignRule->getStatusTo());
        }
        return $this->action;
    }
    
	/**
     * @return Pap_Features_CampaignScheduler_TimeCondition
     */
    private function getCondition() {
        if ($this->condition === null) {
            $this->condition = new Pap_Features_CampaignScheduler_TimeCondition($this->campaignRule);
        }
        return $this->condition;
    }
    
    private function isConditionValid() {
        return $this->getCondition()->isValid();
    }
    
    public function getString() {
        return $this->getCondition()->getText() . $this->getActionObject()->getString();
    }
    
    public function getId() {
        return $this->campaignRule->getId();
    }
    
    public function getAction() {
        return $this->campaignRule->getStatusTo();
    }
}
?>
