<?php
/**
 *   @copyright Copyright (c) 2007 Quality Unit s.r.o.
 *   @author Matej Kendera
 *   @package PostAffiliatePro
 *   @since Version 1.0.0
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.postaffiliatepro.com/licenses/license
 *
 */

/**
 * @package PostAffiliatePro
 */
class Pap_Features_MultipleCurrency_ExchangeRateUpdater extends Gpf_Tasks_LongTask {

    public function getName() {
        return $this->_('Exchange Rate Updater');
    }

    protected function execute() {
        Gpf_Log::info('Starting updating currency exchange rates');

        $defaultCurrency = $this->loadDefaultCurrency();

        $currencyObject = new Gpf_Db_Currency();
        $currencyCollection = $currencyObject->loadCollection();
        foreach ($currencyCollection as $currency) {
            if ($currency->getIsDefault() != Gpf_Db_Currency::DEFAULT_CURRENCY_VALUE) {
                $newExchangeRate = $this->getActualCurrencyRate($currency->getName(), $defaultCurrency->getName());
                if ($newExchangeRate && $newExchangeRate != 0) {
                    $currency->setExchangeRate($newExchangeRate);
                    $currency->update(array(Gpf_Db_Table_Currencies::EXCHANGERATE));
                    Gpf_Log::info('Exchange rate for currency ' . $currency->getName() . ' updated: ' . $newExchangeRate);
                } else {
                    Gpf_Log::warning('New exchange rate for currency ' . $currency->getName() . ' cannot be loaded. Check if you are using existing and official currency code.');
                }
            }
        }

        $this->setDone();
    }

    private function loadDefaultCurrency() {
        $defaultCurrency = new Gpf_Db_Currency();
        $defaultCurrency->setIsDefault(Gpf_Db_Currency::DEFAULT_CURRENCY_VALUE);

        try {
            $defaultCurrency->loadFromData();
        } catch (Gpf_DbEngine_NoRowException $e) {
            throw new Exception($this->_("Default currency does not exist"));
        }

        return $defaultCurrency;
    }

    private function getActualCurrencyRate($currencyName, $originalCurrencyName) {
        $currencyHelper = new Gpf_Currency_Helper();
        return $currencyHelper->getCurrencyRate($currencyName, $originalCurrencyName);
    }

}
?>
