<?php
/**
 *   @copyright Copyright (c) 2007 Quality Unit s.r.o.
 *   @author Matej Kendera
 *   @package PostAffiliatePro
 *   @since Version 1.0.0
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.postaffiliatepro.com/licenses/license
 *
 */

/**
 * @package PostAffiliatePro
 */
class Pap_Features_NetworkAccountPrepaid_Config extends Gpf_Plugins_Config {

    const LOW_CREDIT_LIMI = 'NetworkPrepaid_LowCreditLimit';
    const LOW_CREDIT_NOTIFICATION_ON_EVERY_SALE = 'NetworkPrepaid_LowCreditNotificationOnEverySale';
    const SUSPEND_CREDIT_LIMIT = 'NetworkPrepaid_SuspendCreditLimit';

    protected function initFields() {
        $this->addTextBox($this->_('Low credit limit value'), self::LOW_CREDIT_LIMI, $this->_("If account balance is lower than this value, network account will get 'Account low prepaid credit' email."));
        $this->addCheckBox($this->_('Send low credit notification on every sale while ballance is low'), self::LOW_CREDIT_NOTIFICATION_ON_EVERY_SALE, $this->_('Network merchant receive low credit notification only once when ballane is lower than inserted value or after each sale while ballance is low.'));
        $this->addTextBox($this->_('Suspend credit limit value'), self::SUSPEND_CREDIT_LIMIT, $this->_('If account ballance is lower than this value, network account will be suspended.'));
    }

    /**
     * @anonym
     * @service custom_separator write
     * @param Gpf_Rpc_Params $params
     * @return Gpf_Rpc_Form
     */
    public function save(Gpf_Rpc_Params $params) {
        $form = new Gpf_Rpc_Form($params);
        Gpf_Settings::set(self::LOW_CREDIT_LIMI, $form->getFieldValue(self::LOW_CREDIT_LIMI));
        Gpf_Settings::set(self::SUSPEND_CREDIT_LIMIT, $form->getFieldValue(self::SUSPEND_CREDIT_LIMIT));
        Gpf_Settings::set(self::LOW_CREDIT_NOTIFICATION_ON_EVERY_SALE, $form->getFieldValue(self::LOW_CREDIT_NOTIFICATION_ON_EVERY_SALE));
        $form->setInfoMessage($this->_('Settings saved'));
        return $form;
    }

    /**
     * @anonym
     * @service custom_separator read
     * @param Gpf_Rpc_Params $params
     * @return Gpf_Rpc_Form
     */
    public function load(Gpf_Rpc_Params $params) {
        $form = new Gpf_Rpc_Form($params);
        $form->addField(self::LOW_CREDIT_LIMI, Gpf_Settings::get(self::LOW_CREDIT_LIMI));
        $form->addField(self::SUSPEND_CREDIT_LIMIT, Gpf_Settings::get(self::SUSPEND_CREDIT_LIMIT));
        $form->addField(self::LOW_CREDIT_NOTIFICATION_ON_EVERY_SALE, Gpf_Settings::get(self::LOW_CREDIT_NOTIFICATION_ON_EVERY_SALE));
        return $form;
    }
}

?>