<?php
/**
 *   @copyright Copyright (c) 2007 Quality Unit s.r.o.
 *   @author Matej Kendera
 *   @package PostAffiliatePro
 *   @since Version 1.0.0
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.postaffiliatepro.com/licenses/license
 *
 */

/**
 * @package PostAffiliatePro
 */
class Pap_Features_NetworkAccountPrepaid_Definition extends Gpf_Plugins_Definition {

    public function __construct() {
        $this->codeName =  'NetworkAccountPrepaid';
        $this->name = $this->_('Network Account Prepaid');
        $this->description = $this->_('Network Account Prepaid feature enables prepaid mode. Network merchants will need to have positive accounting balance, otherwise their accounts will be suspended (higher payments for account than their commissions).');
        $this->version = '1.0.0';
        $this->configurationClassName = 'Pap_Features_NetworkAccountPrepaid_Config';
        $this->pluginType = self::PLUGIN_TYPE_FEATURE;

        $this->addRequirement('AffiliateNetwork', '1.0.0');

        $this->addImplementation('Core.defineSettings', 'Pap_Features_NetworkAccountPrepaid_Main', 'initSettings');
        $this->addImplementation('PostAffiliate.Transaction.afterSave', 'Pap_Features_NetworkAccountPrepaid_Main', 'doAfterTransactionSave');
        $this->addImplementation('PostAffiliate.InvoiceForm.markaspaid', 'Pap_Features_NetworkAccountPrepaid_Main', 'doAfterInvoicePaid');
        $this->addImplementation('PostAffiliate.InvoiceForm.getNextInvoiceDateFrom', 'Pap_Features_NetworkAccountPrepaid_Main', 'getNextInvoiceDateFrom');
        $this->addImplementation('PostAffiliateNetwork.AccountsForm.load', 'Pap_Features_NetworkAccountPrepaid_Main', 'doLoadAccountsForm');
        $this->addImplementation('PostAffiliateNetwork.AccountsForm.afterSave', 'Pap_Features_NetworkAccountPrepaid_Main', 'doSaveAccountsForm');
    }

    public function onActivate() {
        $this->insertMailTemplateToDb();
    }

    public function onDeactivate() {
        $this->deleteMailTemplateFromDb();
    }

    private function insertMailTemplateToDb() {
        $template = new Pap_Features_NetworkAccountPrepaid_MailAccountLowCreditMerchant();
        $template->setup(Gpf_Session::getAuthUser()->getAccountId());
    }

    private function deleteMailTemplateFromDb() {
        $dbTemplate = new Gpf_Db_MailTemplate();
        $dbTemplate->setAccountId(Gpf_Session::getAuthUser()->getAccountId());
        $dbTemplate->setClassName('Pap_Features_NetworkAccountPrepaid_MailAccountLowCreditMerchant');
        try {
            $dbTemplate->loadFromData();
        } catch (Gpf_DbEngine_NoRowException $e) {
            Gpf_Log::warning('Mail template Pap_Features_NetworkAccountPrepaid_MailAccountLowCreditMerchant was not found during deactivation of '.$this->name.' feature. It should be there.');
            return;
        }
        $dbTemplate->delete();
    }
}

?>