<?php
/**
 *   @copyright Copyright (c) 2007 Quality Unit s.r.o.
 *   @author Milos Jancovic
 *   @package PostAffiliatePro
 *   @since Version 1.0.0
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.postaffiliatepro.com/licenses/license
 *
 */

/**
 * @package PostAffiliatePro
 */
class Pap_Features_PerformanceRewards_Main extends Gpf_Plugins_Handler {

    /**
     * @var Pap_Common_Transaction
     */
    private $transaction;

    /**
     * @var Gpf_Log_Logger
     */
    private $logger;

    /**
     * @return Pap_Features_PerformanceRewards_Main
     */
    public static function getHandlerInstance() {
        return new Pap_Features_PerformanceRewards_Main();
    }

    public function checkRules(Pap_Common_Transaction $transaction) {
        $this->setLogGroup($transaction->getLogGroupId());
        $this->log('Performance reward started');
        foreach($this->getRules($transaction->getCampaignId()) as $ruleRecord) {
            $dbRule = new Pap_Db_Rule();
            $dbRule->fillFromRecord($ruleRecord);
            $modelRule = new Pap_Model_PerformanceRule($dbRule, $transaction);
            $rule = new Pap_Features_PerformanceRewards_Rule($modelRule);
            try {
                $this->log('Processing rule ' . $rule->getId() . ': ' . $rule->getAction());
                $rule->executeAction();
                $this->log('Rule completed');
            } catch (Exception $e) {
                $this->log(sprintf('Rule %s failed. Reason: %s', $rule->getId(), $e->getMessage()), Gpf_Log::ERROR);
            }
        }
        $this->log('Performance reward ended');
    }
    
    /**
     * @param String $campaignId
     * @return Gpf_SqlBuilder_SelectIterator
     */
    private function getRules($campaignId) {
        $select = new Gpf_SqlBuilder_SelectBuilder();
        $select->select->addAll(Pap_Db_Table_Rules::getInstance());
        $select->from->add(Pap_Db_Table_Rules::getName());
        $select->where->add(Pap_Db_Table_Rules::CAMPAIGN_ID, '=', $campaignId);
        return $select->getAllRowsIterator();
    }

    /**
     * @return Gpf_Log_Logger
     */
    private function getLogger() {
        if ($this->logger === null) {
            $this->logger = Pap_Logger::create(Pap_Common_Constants::TYPE_ACTION);
        }
        return $this->logger;
    }

    private function setLogGroup($goupId) {
        $this->getLogger()->setGroup($goupId);
    }

    private function log($message, $logLevel = Gpf_Log::DEBUG) {
        $this->getLogger()->log($message, $logLevel);
    }
}
?>
