<?php

/**
 *   @copyright Copyright (c) 2007 Quality Unit s.r.o.
 *   @author Andrej Harsani
 *   @package ShopMachine
 *   @since Version 1.0.0
 *   $Id: ActionParser.class.php 16620 2008-03-21 09:21:07Z aharsani $
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.postaffiliatepro.com/licenses/license
 *
 */

/**
 * @package PostAffiliatePro
 */
class Pap_Mail_SaleMail extends Pap_Mail_UserMail {

    const TRANSACTION_USER_PREFIX = 'transaction_';

    /**
     * @var Pap_Common_Transaction
     */
    protected $transaction;

    public function __construct() {
        parent::__construct();
        $this->isHtmlMail = true;
    }

    protected function initTemplateVariables() {
        parent::initTemplateVariables();
        $transactionFields = Pap_Common_TransactionFields::getInstance();
        $fields = $transactionFields->getTransactionFields();
        foreach ($fields as $code => $title) {
            $this->addVariable($code, $title);
        }
        $this->addTransactionUserFields();
    }

    protected function setVariableValues() {
        parent::setVariableValues();

        $transactionFields = Pap_Common_TransactionFields::getInstance();
        $this->updateTransactionFields($transactionFields);
        $transactionValues = $transactionFields->getTransactionFieldsValues();
        foreach($transactionValues as $code => $value) {
            if ($code == Pap_Common_TransactionFields::CAMPAIGNNAME) {
                $value = Gpf_Lang::_localizeRuntime($value, $this->getRecipientLanguage());
            }
            if ($code == Pap_Common_TransactionFields::STATUS) {
                $value = Gpf_Lang::_($value, null, $this->getRecipientLanguage());
            }
            $this->setVariable($code, $value);
        }

        $this->setTransactionUserVariables();
    }

    private function addTransactionUserFields() {
        $formFields = Pap_Common_UserFields::getInstance()->getUserFields(array('M', 'O', 'R'), Gpf::YES);

        $this->addVariable(self::TRANSACTION_USER_PREFIX.'userid', $this->_('Transaction %s', 'userid'));

        foreach($formFields as $code => $name) {
            $this->addVariable(self::TRANSACTION_USER_PREFIX.$code, $this->_('Transaction %s', $this->_localize($name)));
        }
    }

    private function setTransactionUserVariables() {
        if ($this->transaction == null && $this->transaction->getUserId() == null) {
            return;
        }

        try {
            $user = Pap_Common_User::getUserById($this->transaction->getUserId());
        } catch (Gpf_DbEngine_NoRowException $e) {
            return;
        }

        $this->setTransactionUserFieldsValues($user, self::TRANSACTION_USER_PREFIX);

        $this->setVariable(self::TRANSACTION_USER_PREFIX.'userid', $user->getId());
    }

    private function setTransactionUserFieldsValues(Pap_Common_User $user, $preffix = '') {
        Pap_Common_UserFields::getInstance()->setUser($user);
        $formFields = Pap_Common_UserFields::getInstance()->getUserFieldsValues(Gpf::YES);

        foreach($formFields as $code => $value) {
            $this->setVariable($preffix.$code, $value);
        }
    }

    public function updateTransactionFields($transactionFields) {
        $transactionFields->setTransaction($this->transaction);
    }

    
    protected function setTimeVariableValues($timeOffset = 0) {
        parent::setTimeVariableValues($timeOffset);
        $firstClickTime = Gpf_Common_DateUtils::getTimestamp($this->transaction->get(Pap_Db_Table_Transactions::FIRST_CLICK_TIME));
        $lastClickTime = Gpf_Common_DateUtils::getTimestamp($this->transaction->get(Pap_Db_Table_Transactions::LAST_CLICK_TIME));
        $this->setVariable('firstclicktime', Gpf_Common_DateUtils::getDateTime($firstClickTime + $timeOffset));
        $this->setVariable('lastclicktime', Gpf_Common_DateUtils::getDateTime($lastClickTime + $timeOffset));
    }

    public function setTransaction(Pap_Common_Transaction $transaction) {
        $this->transaction = $transaction;
    }
}
