<?php
/**
 *   @copyright Copyright (c) 2007 Quality Unit s.r.o.
 *   @author Matej Kendera
 *   @package PostAffiliatePro
 *   @since Version 1.0.0
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.postaffiliatepro.com/licenses/license
 *
 */

/**
 * @package PostAffiliatePro
 */
class Pap_Stats_Trans_SalesStats extends Gpf_Object {

    /**
     * @var Pap_Stats_Trans_SalesStats
     */
    private static $instance;

    /**
     * @return Pap_Stats_Trans_SalesStats
     */
    public static function getInstance() {
        if (self::$instance == null) {
            self::$instance = new self();
        }
        return self::$instance;
    }

    public function onTransactionUpdate($transactionDateInserted) {
        if ($transactionDateInserted == null) {
            return;
        }
        $select = new Gpf_SqlBuilder_SelectBuilder();
        $select->select->add('\''.$transactionDateInserted.'\'', Pap_Db_Table_Transactions::DATE_INSERTED);

        $this->isertStatsDays($select);
    }

    public function onTransactionsPaid($userId, $payoutHistoryId) {
        $select = new Gpf_SqlBuilder_SelectBuilder();
        $select->select->add(Pap_Db_Table_Transactions::DATE_INSERTED);
        $select->from->add(Pap_Db_Table_Transactions::getName());
        $select->where->add(Pap_Db_Table_Transactions::PAYOUTHISTORY_ID, '=', $payoutHistoryId);
        $select->where->add(Pap_Db_Table_Transactions::USER_ID, '=', $userId);

        $this->isertStatsDays($select);
    }

    public function updateDay($dateInserted) {
        $insert = new Gpf_SqlBuilder_InsertBuilder();
        $insert->setIgnore();
        $insert->setTable(Pap_Db_Table_TransStatsDays::getInstance());
        $insert->add(Pap_Db_Table_TransStatsDays::DATEINSERTED, $dateInserted);
        try {
            $insert->execute();
        } catch (Gpf_Exception $e) {
            Gpf_Log::error('Error during inserting transaction date '.$transactionId.' into db table Pap_Db_Table_TransStatsDays ' . $e->getMessage());
        }
    }

    private function isertStatsDays(Gpf_SqlBuilder_SelectBuilder $select) {
        if ($this->isCronRunning()) {
            $this->insertFromSelect($select);
            return;
        }
        foreach ($select->getAllRowsIterator() as $row) {
            $salesStatsProcessorTask = new Pap_Stats_Trans_SalesStatsProcessorTask();
            $salesStatsProcessorTask->processManually(Gpf_Common_DateUtils::getOnlyDatePart($row->get(Pap_Db_Table_Transactions::DATE_INSERTED)));
        }
    }

    private function insertFromSelect(Gpf_SqlBuilder_SelectBuilder $select) {
        $insertString = 'INSERT IGNORE INTO '.Pap_Db_Table_TransStatsDays::getName().'
            ('.Pap_Db_Table_TransStatsDays::DATEINSERTED.') ' . $select->toString();

        try {
            Gpf_DbEngine_Database::getDatabase()->execute($insertString);
        } catch (Gpf_Exception $e) {
            Gpf_Log::error('Error during inserting transaction date into db table Pap_Db_Table_TransStatsDays: ' . $e->getMessage());
        }
    }

    private function isCronRunning() {
        $taskRunner = new Gpf_Tasks_Runner();
        return $taskRunner->isRunningOK();
    }
}
?>
