<?php
/**
 *   @copyright Copyright (c) 2007 Quality Unit s.r.o.
 *   @author Matej Kendera
 *   @package PostAffiliatePro
 *   @since Version 1.0.0
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.postaffiliatepro.com/licenses/license
 *
 */

/**
 * @package PostAffiliatePro
 */
class Pap_Stats_Trans_SalesStatsProcessorTask extends Gpf_Tasks_LongTask {

    const GROUP_BY_DAY = '%Y-%m-%d 00:00:00';
    const GROUP_BY_HOUR = '%Y-%m-%d %H:00:00';

    public function getName() {
        return $this->_('Sales Stats Processor');
    }

    protected function execute() {
        $select = new Gpf_SqlBuilder_SelectBuilder();
        $select->from->add(Pap_Db_Table_TransStatsDays::getName());
        $select->select->add(Pap_Db_Table_TransStatsDays::DATEINSERTED);
        $select->orderBy->add(Pap_Db_Table_TransStatsDays::DATEINSERTED, false);
        foreach ($select->getAllRowsIterator() as $row) {
            $dateInserted = $row->get(Pap_Db_Table_TransStatsDays::DATEINSERTED);
            $this->debug('SalesStatsProcessorTask: processing transactions for day: '.$dateInserted);
            try {
                $this->processTransactionsForDay($dateInserted);
            } catch (Gpf_Exception $e) {
                Gpf_Log::error('SalesStatsProcessorTask: error during processing transactions for day: '.$dateInserted.' '.$e->getMessage());
                $this->interrupt();
            }
            $this->checkInterruption();
        }

        $this->interrupt(Gpf_Settings::get(Pap_Settings::SALES_STATS_PROCESSOR_INTERVAL) * 60);
    }

    public function processManually($dateInserted) {
        $this->fillTransactionsStatsAff($dateInserted);
        $this->fillTransactionsStats($dateInserted);
    }

    protected function doAfterLongTaskInterrupt() {
    }

    private function processTransactionsForDay($dateInserted) {
        $transStatsDay = new Pap_Db_TransStatsDay();
        $transStatsDay->setDateInserted($dateInserted);
        $transStatsDay->delete();

        $this->fillTransactionsStatsAff($dateInserted);
        $this->fillTransactionsStats($dateInserted);
    }

    private function fillTransactionsStatsAff($dateInserted) {
        $this->deleteGroupedStatsForDays(Pap_Db_Table_TransactionsStatsAff::getName(), $dateInserted);

        $insertString = 'INSERT INTO '.Pap_Db_Table_TransactionsStatsAff::getName().'
            ('.Pap_Db_Table_TransactionsStatsAff::COMMISSIONCOUNT.',
            '.Pap_Db_Table_TransactionsStatsAff::TOTAL_COST.',
            '.Pap_Db_Table_TransactionsStatsAff::COMMISSION.',
            '.Pap_Db_Table_TransactionsStatsAff::PARRENT_BANNER_ID.',
            '.Pap_Db_Table_TransactionsStatsAff::CHANNEL.',
            '.Pap_Db_Table_TransactionsStatsAff::COMMISSIONTYPEID.',
            '.Pap_Db_Table_TransactionsStatsAff::TIER.',
            '.Pap_Db_Table_TransactionsStatsAff::COUNTRY_CODE.',
            '.Pap_Db_Table_TransactionsStatsAff::PAYOUT_STATUS.',
            '.Pap_Db_Table_TransactionsStatsAff::DATE_INSERTED.',
            '.Pap_Db_Table_TransactionsStatsAff::R_TYPE.',
            '.Pap_Db_Table_TransactionsStatsAff::R_STATUS.',
            '.Pap_Db_Table_TransactionsStatsAff::CAMPAIGN_ID.',
            '.Pap_Db_Table_TransactionsStatsAff::BANNER_ID.',
            '.Pap_Db_Table_TransactionsStatsAff::USER_ID.',
            '.Pap_Db_Table_TransactionsStatsAff::ACCOUNT_ID.') ' . $this->getGroupedTransactionsSelect($dateInserted)->toString();

        Gpf_DbEngine_Database::getDatabase()->execute($insertString);
    }

    private function fillTransactionsStats($dateInserted) {
        $this->deleteGroupedStatsForDays(Pap_Db_Table_TransactionsStats::getName(), $dateInserted);

        $insertString = 'INSERT INTO '.Pap_Db_Table_TransactionsStats::getName().'
            ('.Pap_Db_Table_TransactionsStats::COMMISSIONCOUNT.',
            '.Pap_Db_Table_TransactionsStats::TOTAL_COST.',
            '.Pap_Db_Table_TransactionsStats::COMMISSION.',
            '.Pap_Db_Table_TransactionsStats::PARRENT_BANNER_ID.',
            '.Pap_Db_Table_TransactionsStats::CHANNEL.',
            '.Pap_Db_Table_TransactionsStats::COMMISSIONTYPEID.',
            '.Pap_Db_Table_TransactionsStats::TIER.',
            '.Pap_Db_Table_TransactionsStats::COUNTRY_CODE.',
            '.Pap_Db_Table_TransactionsStats::PAYOUT_STATUS.',
            '.Pap_Db_Table_TransactionsStats::DATE_INSERTED.',
            '.Pap_Db_Table_TransactionsStats::R_TYPE.',
            '.Pap_Db_Table_TransactionsStats::R_STATUS.',
            '.Pap_Db_Table_TransactionsStats::CAMPAIGN_ID.',
            '.Pap_Db_Table_TransactionsStats::BANNER_ID.',
            '.Pap_Db_Table_TransactionsStats::ACCOUNT_ID.') ' . $this->getGroupedTransactionsSelect($dateInserted, false)->toString();

        Gpf_DbEngine_Database::getDatabase()->execute($insertString);
    }

    private function deleteGroupedStatsForDays($tableName, $dateInserted) {
        $delete = new Gpf_SqlBuilder_DeleteBuilder();
        $delete->from->add($tableName);
        $delete->where->add(Pap_Stats_Table::DATEINSERTED, 'BETWEEN', '\''.$dateInserted . ' 00:00:00\'' . ' AND \'' . $dateInserted.' 23:59:59\'', 'AND', false);
        $delete->execute();
    }

    /**
     * @param string $dateInserted
     * @return Gpf_SqlBuilder_SelectBuilder
     */
    private function getGroupedTransactionsSelect($dateInserted, $withAffiliate = true) {
        $ageDays = Gpf_Common_DateUtils::getDifference($dateInserted, Gpf_Common_DateUtils::now(), Gpf_Common_DateUtils::DAY);
        $select = new Gpf_SqlBuilder_SelectBuilder();
        $select->select->add('count(*)');
        $select->select->add('SUM('.Pap_Db_Table_Transactions::TOTAL_COST.')');
        $select->select->add('SUM('.Pap_Db_Table_Transactions::COMMISSION.')');
        $select->select->add(Pap_Db_Table_Transactions::PARRENT_BANNER_ID);
        $select->select->add(Pap_Stats_Table::CHANNEL);
        $select->select->add(Pap_Db_Table_Transactions::COMMISSIONTYPEID);
        $select->select->add(Pap_Db_Table_Transactions::TIER);
        $select->select->add(Pap_Db_Table_Transactions::COUNTRY_CODE);
        $select->select->add(Pap_Db_Table_Transactions::PAYOUT_STATUS);
        $select->select->add('DATE_FORMAT('.Pap_Db_Table_Transactions::DATE_INSERTED.', \''.
        ($ageDays < Gpf_Settings::get(Pap_Settings::SALES_HOURS_STATS_MAX_DAYS) ? self::GROUP_BY_HOUR : self::GROUP_BY_DAY).
        '\')', 'dateins');
        $select->select->add(Pap_Db_Table_Transactions::R_TYPE);
        $select->select->add(Pap_Db_Table_Transactions::R_STATUS);
        $select->select->add(Pap_Db_Table_Transactions::CAMPAIGN_ID);
        $select->select->add(Pap_Db_Table_Transactions::BANNER_ID);
        if ($withAffiliate) {
            $select->select->add(Pap_Db_Table_Transactions::USER_ID);
        }
        $select->select->add(Pap_Db_Table_Transactions::ACCOUNT_ID);
        $select->from->add(Pap_Db_Table_Transactions::getName());

        $select->where->add(Pap_Db_Table_Transactions::DATE_INSERTED, 'BETWEEN', '\''.$dateInserted . ' 00:00:00\'' . ' AND \'' . $dateInserted.' 23:59:59\'', 'AND', false);
        $select->groupBy->add(Pap_Db_Table_Transactions::PARRENT_BANNER_ID);
        $select->groupBy->add(Pap_Stats_Table::CHANNEL);
        $select->groupBy->add(Pap_Db_Table_Transactions::COMMISSIONTYPEID);
        $select->groupBy->add(Pap_Db_Table_Transactions::TIER);
        $select->groupBy->add(Pap_Db_Table_Transactions::COUNTRY_CODE);
        $select->groupBy->add(Pap_Db_Table_Transactions::PAYOUT_STATUS);
        $select->groupBy->add('dateins');
        $select->groupBy->add(Pap_Db_Table_Transactions::R_TYPE);
        $select->groupBy->add(Pap_Db_Table_Transactions::R_STATUS);
        $select->groupBy->add(Pap_Db_Table_Transactions::CAMPAIGN_ID);
        $select->groupBy->add(Pap_Db_Table_Transactions::BANNER_ID);
        if ($withAffiliate) {
            $select->groupBy->add(Pap_Db_Table_Transactions::USER_ID);
        }
        $select->groupBy->add(Pap_Db_Table_Transactions::ACCOUNT_ID);

        if (!$withAffiliate) {
            Gpf_Plugins_Engine::extensionPoint('PostAffiliate.Pap_Stats_Trans_SalesStatsProcessorTask.getGroupedTransactionsSelectWithoutAff', $select);
        }

        return $select;
    }
}
?>
