<?php
/**
 *   @copyright Copyright (c) 2007 Quality Unit s.r.o.
 *   @author Matej Kendera
 *   @package PostAffiliatePro
 *   @since Version 1.0.0
 *   $Id: UpdateManager.class.php 18026 2008-05-14 08:07:20Z mbebjak $
 *
 *   Licensed under the Quality Unit, s.r.o. Dual License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.qualityunit.com/licenses/gpf
 *
 */

/**
 * @package GwtPhpFramework
 */
class Pap_Tasks_RecomputeRawClicksTask extends Gpf_Tasks_LongTask {

    const RAW_CLICKS_TEMP_TABLE_NAME = 'qu_pap_rawclicks_update';
    
    private $campaignAccount = array();

    public function getName() {
        return $this->_('Recompute Raw Clicks');
    }

    /**
     *
     * @return Gpf_SqlBuilder_SelectBuilder
     */
    private function getRawClicksSelect() {
        $select = new Gpf_SqlBuilder_SelectBuilder();
        $select->select->add('*');
        $select->from->add(self::RAW_CLICKS_TEMP_TABLE_NAME);
        return $select;
    }

    protected function execute() {
        $result = Gpf_DbEngine_Database::getDatabase()->execute('show tables like \''.self::RAW_CLICKS_TEMP_TABLE_NAME.'\';');
        if ($result->affectedRows() == 0) {
            return;
        }

        foreach($this->getRawClicksSelect()->getAllRowsIterator() as $row) {
            if($this->isPending('copy_' . $row->get('clickid'))) {
                $this->insertRow($row);
                $this->setDone();
            }
        }
        
        $this->truncateRawClicksTempTable();
    }

    private function insertRow($row) {
        $rawClick = new Pap_Db_RawClick();
        $rawClick->setAccountId($this->getAccount($row->get('campaignid')));
        $rawClick->setUserId($row->get('userid'));
        $rawClick->setBannerId($row->get('bannerid'));
        $rawClick->setCampaignId($row->get('campaignid'));
        $rawClick->setParentBannerId($row->get('parentbannerid'));
        $rawClick->setCountryCode($row->get('countrycode'));
        $rawClick->setType($row->get('rtype'));
        $rawClick->setDateTime($row->get('datetime'));
        $rawClick->setRefererUrl($row->get('refererurl'));
        $rawClick->setIp($row->get('ip'));
        $rawClick->setBrowser($row->get('browser'));
        $rawClick->setData1($row->get('cdata1'));
        $rawClick->setData2($row->get('cdata2'));
        $rawClick->setChannel($row->get('channel'));
        $rawClick->set(Pap_Db_Table_RawClicks::RSTATUS, $row->get('rstatus'));
        $rawClick->insert();
    }
    
    private function truncateRawClicksTempTable() {
        $result = Gpf_DbEngine_Database::getDatabase()->execute('SELECT count(*) FROM '.self::RAW_CLICKS_TEMP_TABLE_NAME.';');
        $row = $result->fetchRow();
        $ClicksTempCount = $row[0];
        
        $result = Gpf_DbEngine_Database::getDatabase()->execute('SELECT count(*) FROM '.Pap_Db_Table_RawClicks::getName().';');
        $row = $result->fetchRow();
        $ClicksCount = $row[0];
        
        if ($ClicksCount < $ClicksTempCount) {
            Gpf_Log::error('Recompute Raw Clicks: not all records from qu_pap_rawclicks_update was inserted into table qu_pap_rawclicks');
            return;
        }
        Gpf_DbEngine_Database::getDatabase()->execute('DROP TABLE '.self::RAW_CLICKS_TEMP_TABLE_NAME.';');
    }

    private function getAccount($campaignId) {
        if (!array_key_exists($campaignId, $this->campaignAccount)) {
            $campaign = Pap_Common_Campaign::getCampaignById($campaignId);
            $this->campaignAccount[$campaignId] = $campaign->getAccountId();
        }

        return $this->campaignAccount[$campaignId];
    }
}
