<?php
/**
 *   @copyright Copyright (c) 2011 Quality Unit s.r.o.
 *   @author Matej Kendera
 *   @package PostAffiliatePro
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.postaffiliatepro.com/licenses/license
 */

class AffiliateHomeStatsAdditionalData_Main extends Gpf_Plugins_Handler {

    private static $instance = false;

    public function __construct() {
    }

    /**
     * @return AffiliateHomeStatsAdditionalData_Main
     */
    public static function getHandlerInstance() {
        if(!self::$instance)  {
            return self::$instance = new AffiliateHomeStatsAdditionalData_Main();
        }
        return self::$instance;
    }

    public function fillAffiliateDataToTemplate(Gpf_Templates_Template $tmpl) {
        $allTimeStatsParams = $this->getStatsParams();
        
        $tmpl->assign('allTimeClicks', new Pap_Stats_Clicks($allTimeStatsParams));
        $tmpl->assign('allTimeImpressions', new Pap_Stats_Impressions($allTimeStatsParams));
        $tmpl->assign('allTimeSales', $s = new Pap_Stats_Sales($allTimeStatsParams));
        $tmpl->assign('allTimeCommissions', new Pap_Stats_Transactions($allTimeStatsParams));
        $tmpl->assign('allTimeRefunds', new Pap_Stats_Refunds($allTimeStatsParams));
        $tmpl->assign('allTimeChargebacks', new Pap_Stats_Chargebacks($allTimeStatsParams));
        $tmpl->assign('allTimeActionCommissions', new Pap_Stats_Actions($allTimeStatsParams));
        
        
        $transactionsStatsAllTime = new Pap_Stats_Transactions($allTimeStatsParams);
        $transactionsStatsAllTime->setTier(1);
        $tmpl->assign('allTimeCommissionsFirstTier', $transactionsStatsAllTime);

        $transactionsStatsToday = new Pap_Stats_Transactions($this->getStatsParams(Gpf_Data_Filter::RANGE_TODAY));
        $transactionsStatsToday->setTier(1);
        $tmpl->assign('todayCommissionsFirstTier', $transactionsStatsToday);

        $transactionsStatsThisMonth = new Pap_Stats_Transactions($this->getStatsParams(Gpf_Data_Filter::RANGE_THIS_MONTH));
        $transactionsStatsThisMonth->setTier(1);
        $tmpl->assign('thisMonthCommissionsFirstTier', $transactionsStatsThisMonth);
    }

    /**
     * @return Pap_Stats_Params
     */
    public function getStatsParams($datePreset = '') {
        $statsParams = new Pap_Stats_Params();

        if ($datePreset != '') {
            $date = $this->getDateArray($datePreset);
            $statsParams->setDateFrom(new Gpf_DateTime($date["dateFrom"]));
            $statsParams->setDateTo(new Gpf_DateTime($date["dateTo"]));
        }

        $statsParams->setAffiliateId(Gpf_Session::getAuthUser()->getPapUserId());

        return $statsParams;
    }

    private function getDateArray($datePreset) {
        $filter = new Gpf_SqlBuilder_Filter(array('', 'DP', $datePreset));
        return $filter->addDateValueToArray(array());
    }
}
?>
