<?php
/**
 *   @copyright Copyright (c) 2007 Quality Unit s.r.o.
 *   @author Martin Pullmann
 *   @package GwtPhpFramework
 *   @since Version 1.0.0
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.postaffiliatepro.com/licenses/license
 *
 */

/**
 * @package GwtPhpFramework
 */
class CustomRecurringLevel_Config extends Gpf_Plugins_Config {

    const CRL_FIELD1 = 'CustomRecurringLevelField1';
    const CRL_FIELD2 = 'CustomRecurringLevelField2';
    const CRL_FIELD3 = 'CustomRecurringLevelField3';
    const CRL_FIELD4 = 'CustomRecurringLevelField4';
    const CRL_FIELD5 = 'CustomRecurringLevelField5';
    const CRL_EXCLUSION = 'CustomRecurringExcludeCampaign';

    protected function initFields() {
        $this->addTextBox($this->_("First sale custom commission"), self::CRL_FIELD1, $this->_("Custom value of the first commission. Each commission have to be defined as a number or %number e.g. %10 means 10% commission, otherwise flat commission will be used."));
        $this->addTextBox($this->_("Second sale custom commission"), self::CRL_FIELD2, $this->_("Custom value of the second (recurring) commission. First recurring sale comes with the same order ID, so we can easily recognize it. You can set up to 5 different recurring values, that means each recurrent payment can have a different commission."));
        $this->addTextBox($this->_("Third sale custom commission"), self::CRL_FIELD3, $this->_("Zero means to stop processing any next recurrings. E.g. if you want to limit commission to 3 recurrences only, you have to define 3 levels and 4th and 5th set as zero - '0'."));
        $this->addTextBox($this->_("Fourth sale custom commission"), self::CRL_FIELD4);
        $this->addTextBox($this->_("Fifth sale custom commission"), self::CRL_FIELD5);
        $this->addTextBox($this->_("Campaigns to be excluded"), self::CRL_EXCLUSION, $this->_("If you want some campaigns to be excluded from custom level commissions, just write down all campaigns IDs here, semicolon separated."));
    }
    
    /**
     * @anonym
     * @service lifetime_comm_settings write
     * @param Gpf_Rpc_Params $params
     * @return Gpf_Rpc_Form
     */
    public function save(Gpf_Rpc_Params $params) {
        $form = new Gpf_Rpc_Form($params);
        Gpf_Settings::set(self::CRL_FIELD1, $form->getFieldValue(self::CRL_FIELD1));
        Gpf_Settings::set(self::CRL_FIELD2, $form->getFieldValue(self::CRL_FIELD2));
        Gpf_Settings::set(self::CRL_FIELD3, $form->getFieldValue(self::CRL_FIELD3));
        Gpf_Settings::set(self::CRL_FIELD4, $form->getFieldValue(self::CRL_FIELD4));
        Gpf_Settings::set(self::CRL_FIELD5, $form->getFieldValue(self::CRL_FIELD5));
        Gpf_Settings::set(self::CRL_EXCLUSION, $form->getFieldValue(self::CRL_EXCLUSION));
        
        $form->setInfoMessage($this->_('Plugin settings saved'));
        return $form;
    }

    /**
     * @anonym
     * @service lifetime_comm_settings read
     * @param Gpf_Rpc_Params $params
     * @return Gpf_Rpc_Form
     */
    public function load(Gpf_Rpc_Params $params) {
        $form = new Gpf_Rpc_Form($params);
        $form->addField(self::CRL_FIELD1, Gpf_Settings::get(self::CRL_FIELD1));
        $form->addField(self::CRL_FIELD2, Gpf_Settings::get(self::CRL_FIELD2));
        $form->addField(self::CRL_FIELD3, Gpf_Settings::get(self::CRL_FIELD3));
        $form->addField(self::CRL_FIELD4, Gpf_Settings::get(self::CRL_FIELD4));
        $form->addField(self::CRL_FIELD5, Gpf_Settings::get(self::CRL_FIELD5));
        $form->addField(self::CRL_EXCLUSION, Gpf_Settings::get(self::CRL_EXCLUSION));
        
        return $form;
    }
}

?>
