<?php
/**
 *   @copyright Copyright (c) 2007 Quality Unit s.r.o.
 *   @author Martin Pullmann & Matej Kendera
 *   @package PostAffiliatePro
 *   @since Version 1.0.0
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.postaffiliatepro.com/licenses/license
 *
 */

/**
 * @package PostAffiliate
 */
class CustomRecurringLevel_Main extends Gpf_Plugins_Handler {
    
  	public static function getHandlerInstance() {
   		return new CustomRecurringLevel_Main();
  	}

    public function initSettings($context) {
        $context->addDbSetting(CustomRecurringLevel_Config::CRL_FIELD1, '');
        $context->addDbSetting(CustomRecurringLevel_Config::CRL_FIELD2, '');
        $context->addDbSetting(CustomRecurringLevel_Config::CRL_FIELD3, '');
        $context->addDbSetting(CustomRecurringLevel_Config::CRL_FIELD4, '');
        $context->addDbSetting(CustomRecurringLevel_Config::CRL_FIELD5, '');
        $context->addDbSetting(CustomRecurringLevel_Config::CRL_EXCLUSION, '');
    }

    public function processLevels(Pap_Common_TransactionCompoundContext $transactionCompoundContext) {
        $context = $transactionCompoundContext->getContext();
        $transaction = $transactionCompoundContext->getTransaction();
        
        $campaignIdsExclusion = explode(';',Gpf_Settings::get(CustomRecurringLevel_Config::CRL_EXCLUSION));
        if (!empty($campaignIdsExclusion)) {
            if (in_array($transaction->getCampaignId(),$campaignIdsExclusion)) {
                $context->debug('PLUGIN CustomRecurringLevel - The campaign is defined in plugin to be excluded so no custom level commission to be applied.');
                return;
            }
        }
        
        if (Gpf_Settings::get(CustomRecurringLevel_Config::CRL_FIELD1) == '') {
            $context->debug('PLUGIN CustomRecurringLevel - No custom levels defined, ending.');
            return;
        }
        
        if ($transaction->getTier() > '1') {
            $context->debug('PLUGIN CustomRecurringLevel - No first tier, ending.');
            return;
        }

        $numberOfRecurrences = $this->getTransactionsCount($transaction->getOrderId(), array(Pap_Common_Constants::STATUS_APPROVED, Pap_Common_Constants::STATUS_PENDING));
        
        if ($numberOfRecurrences > 4) {
            $context->debug('PLUGIN CustomRecurringLevel - There were more recurrent orders found ('.$numberOfRecurrences.') than the plugin can handle, stopping.');
            return;
        }
        
        switch ($numberOfRecurrences) {
            case 0:
                $context->debug('PLUGIN CustomRecurringLevel - This is initial sale, commission won\'t be modified.');
                break;
            case 1:
                $this->processNewCustomCommission($transactionCompoundContext, Gpf_Settings::get(CustomRecurringLevel_Config::CRL_FIELD1));
                break;
            case 2:
                $this->processNewCustomCommission($transactionCompoundContext, Gpf_Settings::get(CustomRecurringLevel_Config::CRL_FIELD2));
                break;
            case 3:
                $this->processNewCustomCommission($transactionCompoundContext, Gpf_Settings::get(CustomRecurringLevel_Config::CRL_FIELD3));
                break;
            case 4:
                $this->processNewCustomCommission($transactionCompoundContext, Gpf_Settings::get(CustomRecurringLevel_Config::CRL_FIELD4));
                break;
            case 5:
                $this->processNewCustomCommission($transactionCompoundContext, Gpf_Settings::get(CustomRecurringLevel_Config::CRL_FIELD5));
                break;
        }
    }
    
    private function processNewCustomCommission(Pap_Common_TransactionCompoundContext $transactionCompoundContext, $commissionValue) {
        $context = $transactionCompoundContext->getContext();

        $context->debug('PLUGIN CustomRecurringLevel - Custom commission value for this level: ' . $commissionValue);

        if ($commissionValue == 0) {
            $transactionCompoundContext->setSaveTransaction(false);
            $context->debug('PLUGIN CustomRecurringLevel - Custom commission for this level is set to ZERO, transaction won\'t be saved.');
            return;
        }
        
        $type = $this->getParameterType($commissionValue);
        $commissionValue = $this->makeCorrections($commissionValue);
        
        if (is_numeric($commissionValue) && $commissionValue > 0) {
            $transaction = $transactionCompoundContext->getTransaction();
            $commissionNew = new Pap_Tracking_Common_Commission($transaction->getTier(), $type, $commissionValue);
            $commissionValue = $commissionNew->getCommission($transaction->getTotalCost()-$transaction->getFixedCost());
            $transaction->setCommission($commissionValue);
            $context->debug('PLUGIN CustomRecurringLevel - Commission successfully changed to ' . $commissionValue);
            return;
        }

        $context->debug('PLUGIN CustomRecurringLevel - empty or wrong custom commission value, ending.');

    }

  	private function getTransactionsCount($value, $status) {
  	    $select = new Gpf_SqlBuilder_SelectBuilder();

        $select->select->add('count(*)', 'count');
        $select->from->add(Pap_Db_Table_Transactions::getName());
        $select->where->add(Pap_Db_Table_Transactions::ORDER_ID, 'LIKE', '%'.$value.'%');

        $select->where->add(Pap_Db_Table_Transactions::R_TYPE, "IN", array(Pap_Common_Constants::TYPE_SALE, Pap_Common_Constants::TYPE_ACTION));
        $select->where->add(Pap_Db_Table_Transactions::TIER, "=", "1");
        if (is_array($status)) {
            $select->where->add(Pap_Db_Table_Transactions::R_STATUS, "IN", $status);
        }
        else{
            $select->where->add(Pap_Db_Table_Transactions::R_STATUS, "=", $status);
        }

        return $select->getOneRow()->get('count');
  	}
  	
    private function getParameterType($value) {
        $type = '$';
        if(strpos($value, '%') !== false) {
            $type = '%';
        }
        return $type;
    }
    
    private function makeCorrections($value) {
        $value = str_replace('%', '', $value);
        $value = str_replace('$', '', $value);
        $value = str_replace(',', '.', $value);
        $value = str_replace(' ', '', $value);
        return $value;
    }
}
?>
