<?php
/**
 *   @copyright Copyright (c) 2007 Quality Unit s.r.o.
 *   @author Milos Jancovic & Martin Pullmann
 *   @package GwtPhpFramework
 *   @since Version 1.0.0
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.postaffiliatepro.com/licenses/license
 *
 */

/**
 * @package GwtPhpFramework
 */
class LifetimeCommissions_Config extends Gpf_Plugins_Config {

    const LIFETIME_COMMISSION_COOKIE_LIMIT_ACTIVE = 'LifetimeCommissionCookieLimitActive';
    const CUSTOM_COMMISSION_LEVEL = 'LifetimeCommissionLevel';
    const LIFETIME_COMMISSION_AS_DEFAULT_AFFILIATE = 'LifetimeCommissionAsDefaultAffiliate';
    const LIFETIME_COMMISSION_CAMPAIGN_BANNER = 'LifetimeCommissionCampaignBanner';
    const LIFETIME_COMMISSION_CHANNEL = 'LifetimeCommissionChannel';

    const AFTER_COOKIE_LIMIT_LIFETIME_REFERRER = 'N';
    const AFTER_COOKIE_LIMIT_DO_NOT_SAVE = 'Y';
    const AFTER_COOKIE_LIMIT_UNREFERRED_AFFILIATE = 'U';

    protected function initFields() {
        $this->addCheckBox($this->_('Load by lifetime referrer also campaign and banner'), self::LIFETIME_COMMISSION_CAMPAIGN_BANNER, $this->_('Campaign and banner will be loaded from Lifetime referral.'));
        $this->addCheckBox($this->_('Load by lifetime referrer also channel'), self::LIFETIME_COMMISSION_CHANNEL, $this->_('Channel will be loaded from Lifetime referral.'));
        $radioValues = array(
            self::AFTER_COOKIE_LIMIT_LIFETIME_REFERRER => $this->_('Save commissions to lifetime referrer'),
            self::AFTER_COOKIE_LIMIT_DO_NOT_SAVE => $this->_('Do not save commissions'),
            self::AFTER_COOKIE_LIMIT_UNREFERRED_AFFILIATE => $this->_('Save as unreferred (if set in tracking settings)'),
        );
        $this->addRadioBox($this->_('When cookies expired:'), self::LIFETIME_COMMISSION_COOKIE_LIMIT_ACTIVE, $radioValues);
        $this->addCheckBox($this->_('Use lifetime referrer only for unreferred sales'), self::LIFETIME_COMMISSION_AS_DEFAULT_AFFILIATE, $this->_('Commissions will be saved to Lifetime referral affiliate if affiliate is not recognized from cookies.'));
        $this->addTextBox($this->_('Custom commission'), self::CUSTOM_COMMISSION_LEVEL, $this->_('Custom commission value for lifetime referrals - if the transaction is tracked for a lifetime referral affiliate, you can set a custom commission here, e.g. %20 or just 20 for flat commission.'));
    }
    
    /**
     * @anonym
     * @service lifetime_comm_settings write
     * @param Gpf_Rpc_Params $params
     * @return Gpf_Rpc_Form
     */
    public function save(Gpf_Rpc_Params $params) {
        $form = new Gpf_Rpc_Form($params);
        Gpf_Settings::set(self::LIFETIME_COMMISSION_COOKIE_LIMIT_ACTIVE, $form->getFieldValue(self::LIFETIME_COMMISSION_COOKIE_LIMIT_ACTIVE));
        Gpf_Settings::set(self::CUSTOM_COMMISSION_LEVEL, $form->getFieldValue(self::CUSTOM_COMMISSION_LEVEL));
        Gpf_Settings::set(self::LIFETIME_COMMISSION_AS_DEFAULT_AFFILIATE, $form->getFieldValue(self::LIFETIME_COMMISSION_AS_DEFAULT_AFFILIATE));
        Gpf_Settings::set(self::LIFETIME_COMMISSION_CAMPAIGN_BANNER, $form->getFieldValue(self::LIFETIME_COMMISSION_CAMPAIGN_BANNER));
        Gpf_Settings::set(self::LIFETIME_COMMISSION_CHANNEL, $form->getFieldValue(self::LIFETIME_COMMISSION_CHANNEL));
        $form->setInfoMessage($this->_('Lifetime commission settings saved'));
        return $form;
    }

    /**
     * @anonym
     * @service lifetime_comm_settings read
     * @param Gpf_Rpc_Params $params
     * @return Gpf_Rpc_Form
     */
    public function load(Gpf_Rpc_Params $params) {
        $form = new Gpf_Rpc_Form($params);
        $form->addField(self::LIFETIME_COMMISSION_COOKIE_LIMIT_ACTIVE, Gpf_Settings::get(self::LIFETIME_COMMISSION_COOKIE_LIMIT_ACTIVE));
        $form->addField(self::CUSTOM_COMMISSION_LEVEL, Gpf_Settings::get(self::CUSTOM_COMMISSION_LEVEL));
        $form->addField(self::LIFETIME_COMMISSION_AS_DEFAULT_AFFILIATE, Gpf_Settings::get(self::LIFETIME_COMMISSION_AS_DEFAULT_AFFILIATE));
        $form->addField(self::LIFETIME_COMMISSION_CAMPAIGN_BANNER, Gpf_Settings::get(self::LIFETIME_COMMISSION_CAMPAIGN_BANNER));
        $form->addField(self::LIFETIME_COMMISSION_CHANNEL, Gpf_Settings::get(self::LIFETIME_COMMISSION_CHANNEL));
        return $form;
    }
}

?>
