<?php
/**
 *   @copyright Copyright (c) 2007 Quality Unit s.r.o.
 *   @author Maros Fric
 *   @package PostAffiliatePro
 *   @since Version 1.0.0
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.postaffiliatepro.com/licenses/license
 *
 */

/**
 * @package PostAffiliate
 */
class LifetimeCommissions_Main extends Gpf_Plugins_Handler {
    
	public static function getHandlerInstance() {
 		return new LifetimeCommissions_Main();
	}

 	/**
 	 * checks if lifetime referral exists and if user (affiliate) is still valid
 	 *
 	 * @param Pap_Contexts_Action $context
 	 */
 	public function checkLifetimeReferral(Pap_Common_VisitorAffiliateCacheCompoundContext $visitorAffiliateCompoundContext) {
 	    $context = $visitorAffiliateCompoundContext->getContext();
    	$context->debug(Gpf_Lang::_runtime('PLUGIN LifetimeCommissions checkLifetimeReferral started'));

    	if (Gpf_Settings::get(LifetimeCommissions_Config::LIFETIME_COMMISSION_AS_DEFAULT_AFFILIATE) == Gpf::YES) {
            $context->debug(Gpf_Lang::_runtime('PLUGIN LifetimeCommissions ended - checking will be during loading default affiliate'));
            return;
        }

        $this->setLifetimeReferral($context);

        $context->debug(Gpf_Lang::_runtime('PLUGIN LifetimeCommissions ended'));
        return Gpf_Plugins_Engine::PROCESS_CONTINUE;
    }

    /**
     * @param Pap_Contexts_Action $context
     */
    public function setLifetimeReferralAsDefaultUser(Pap_Contexts_Action $context) {
        if (Gpf_Settings::get(LifetimeCommissions_Config::LIFETIME_COMMISSION_AS_DEFAULT_AFFILIATE) != Gpf::YES) {
            $context->debug(Gpf_Lang::_runtime('PLUGIN LifetimeCommissions set LifelimeReferral as default affiliate - disabled'));
            return;
        }

        $context->debug(Gpf_Lang::_runtime('PLUGIN LifetimeCommissions set LifelimeReferral as default affiliate'));

        $this->setLifetimeReferral($context);

        $context->debug(Gpf_Lang::_runtime('PLUGIN LifetimeCommissions ended'));
        return;
    }

    /**
     * @param Pap_Contexts_Action $context
     */
    private function setLifetimeReferral(Pap_Contexts_Action $context) {
        $identifier = $this->getIdentifier($context);
        if ($identifier == null || $identifier == '') {
            $context->debug("  Identifier (data1) is empty, stopping");
        } else {
            $lifetimeCommission = $this->findLifetimeCommissionIdentifier($context, $identifier);
            if ($lifetimeCommission == null) {
                return;
            }
            $user = $this->checkUserIsCorrect($context, $lifetimeCommission->getUserId());

            if ($user != null) {
                $context->setUserObject($user);
                
                if (Gpf_Settings::get(LifetimeCommissions_Config::LIFETIME_COMMISSION_CAMPAIGN_BANNER) == Gpf::YES) {
                    $context->debug('  LifetimeCommissions: loading campaign by lifetime referral');
                    $campaign = $this->getCheckCampaign($context, $lifetimeCommission->getCampaignId());
                    if ($campaign != null) {
                        $context->debug('  LifetimeCommissions: campaign loaded '. $lifetimeCommission->getCampaignId());
                        $context->setCampaignObject($campaign);
                        
                        $context->debug('  LifetimeCommissions: loading banner by lifetime referral');
                        $banner = $this->getCheckBanner($lifetimeCommission->getBannerId());
                        if ($banner != null && $banner->getCampaignId() == $campaign->getId()) {
                            $context->debug('  LifetimeCommissions: banner loaded '. $lifetimeCommission->getBannerId());
                            $context->setBannerObject($banner);
                        }
                    }
                }
                if (Gpf_Settings::get(LifetimeCommissions_Config::LIFETIME_COMMISSION_CHANNEL) == Gpf::YES) {
                    $context->debug('  LifetimeCommissions: loading channel by lifetime referral');
                    $channel = $this->getCheckChannel($context, $lifetimeCommission->getChannelId());
                    if ($channel != null) {
                        $context->debug('  LifetimeCommissions: channel loaded '. $lifetimeCommission->getChannelId());
                        $context->setChannelObject($channel);
                    }
                }

                $commissionValue = Gpf_Settings::get(LifetimeCommissions_Config::CUSTOM_COMMISSION_LEVEL);
                $commissionValue = $this->doCommissionCorrections($commissionValue);

                if ($commissionValue != '') {
                    $context->debug("  LifetimeCommissions: Setting custom commission: $commissionValue");
                    $context->setCustomCommission($commissionValue);
                }
            }
        }
    }
    
    private function getCheckCampaign(Pap_Contexts_Action $context, $campaignId) {
        if ($campaignId == null) {
            return null;
        }
        $recognizeCampaign = new Pap_Tracking_Action_RecognizeCampaign();
        try {
            $campaign = $recognizeCampaign->getCampaignById($context, $campaignId);
        } catch (Exception $e) {
            return null;
        }
        return $campaign;
    }
    
    private function getCheckBanner($bannerId) {
        if ($bannerId == null) {
            return null;
        }
        $bannerFactory = new Pap_Common_Banner_Factory();
        try {
            $banner = $bannerFactory->getBanner($bannerId);
        } catch (Exception $e) {
            return null;
        }
        return $banner;
    }
    
    private function getCheckChannel(Pap_Contexts_Action $context, $channelId) {
        if ($channelId == null) {
            return null;
        }
        $recognizeChannel = new Pap_Tracking_Action_RecognizeChannel();
        try {
            $channel = $recognizeChannel->getChannelById($context, $channelId);
        } catch (Exception $e) {
            return null;
        }
        return $channel;
    }

    public function checkLifitimeLimit(Pap_Contexts_Action $context) {
        if (Gpf_Settings::get(LifetimeCommissions_Config::LIFETIME_COMMISSION_COOKIE_LIMIT_ACTIVE) == LifetimeCommissions_Config::AFTER_COOKIE_LIMIT_LIFETIME_REFERRER) {
            return;
        }
        $context->debug('PLUGIN LifetimeCommissions - Checking LifeTime cookie limit started');
        if (!$this->isInCookieLifetimeLimit($context)) {
            if (Gpf_Settings::get(LifetimeCommissions_Config::LIFETIME_COMMISSION_COOKIE_LIMIT_ACTIVE) == LifetimeCommissions_Config::AFTER_COOKIE_LIMIT_UNREFERRED_AFFILIATE) {
                $defaultUserObject = $this->getDefaultAffiliate($context);
                if($defaultUserObject != null) {
                    $context->setUserObject($defaultUserObject);
                } else {
                    $context->setDoCommissionsSave(false);
                    $context->debug('    Transaction not saved. Unreferred user cannot be loaded.');
                }
            } else {
                $context->setDoCommissionsSave(false);
                $context->debug('    Transaction not saved. After cookie lifetime limit.');
            }
        }
        $context->debug('PLUGIN LifetimeCommissions - Checking LifeTime cookie limit ended');
    }
    
    /**
     * @param Pap_Contexts_Tracking $context
     * @return boolean
     */
    private function isInCookieLifetimeLimit(Pap_Contexts_Tracking $context) {
        $select = new Gpf_SqlBuilder_SelectBuilder();
        $select->select->add(Pap_Db_Table_Transactions::DATE_INSERTED);
        $select->from->add(Pap_Db_Table_Transactions::getName());
        $select->where->add(Pap_Db_Table_Transactions::DATA1, '=', $this->getIdentifier($context));
        $select->where->add(Pap_Db_Table_Transactions::R_TYPE, '=', Pap_Db_Transaction::TYPE_SALE);
        $select->orderBy->add(Pap_Db_Table_Transactions::DATE_INSERTED);
        $select->limit->set(0, 1);
        try {
            $firstSale = $select->getOneRow()->get(Pap_Db_Table_Transactions::DATE_INSERTED);
            $cookieLifeTime = Pap_Tracking_Cookie::getCookieLifeTimeInDays($context);
            if (Gpf_Common_DateUtils::getDifference($firstSale, Gpf_Common_DateUtils::getDateTime(time()), 'day') > $cookieLifeTime) {
                $context->debug('    Transaction is not in cookie limit. Date of first transaction: ' . $firstSale . ' is not in cookie limit: ' . $cookieLifeTime . ' days.');
                return false;
            }
        } catch (Gpf_DbEngine_NoRowException $e) {
        }
        return true;
    }

    /**
     * @param Pap_Contexts_Action $context
     * @param string $identifier
     * @return Pap_Db_LifetimeCommission|NULL
     */
 	private function findLifetimeCommissionIdentifier(Pap_Contexts_Action $context, $identifier) {
 		$lifetimeCommission = new Pap_Db_LifetimeCommission();
 		$lifetimeCommission->setIdentifier($identifier);

 		try {
 			$lifetimeCommission->loadFromData(array(Pap_Db_Table_LifetimeCommissions::IDENTIFIER));
 			$context->debug("  Found user '".$lifetimeCommission->getUserId()."' for identifier '$identifier', checking if user is valid");
 			return $lifetimeCommission;
 		} catch (Gpf_DbEngine_NoRowException $e) {
 			$context->debug("  New lifetime referral: identifier '$identifier' <-> user '".$lifetimeCommission->getUserId()."' inserted");
 			return null;
 		} catch (Gpf_DbEngine_TooManyRowsException $e) {
 			$context->debug("  Too many rows for ths identifier '$identifier'");
 			return null;
 		}
 	}

    /**
     * checks that user with this ID exists and is correct
     *
     * @param Pap_Contexts_Action $context
     * @param string $userId
     * @param string $trackingMethod
     * @return Pap_Common_User|NULL
     */
    private function checkUserIsCorrect(Pap_Contexts_Action $context, $userId) {
    	$userObj = $this->getUserById($context, $userId);
    	if($userObj != null) {
    		$context->setTrackingMethod(Pap_Common_Transaction::TRACKING_METHOD_LIFETIME_REFERRAL);
    	}
    	return $userObj;
    }

	/**
	 * gets user by user id
	 * @param $userId
	 * @return Pap_Common_User
	 */
	protected function getUserById($context, $id) {
		if($id == '') {
			return null;
		}

		// try to get user by userid
		$user = new Pap_Common_User();
		$user->setPrimaryKeyValue($id);
		try {
			$user->load();
			if($user->getStatus() == Pap_Common_Constants::STATUS_DECLINED) {
				$context->debug("    User with UserId: $id found, but he is declined. We'll not use him.");
				return null;
			}

			$context->debug("    User with UserId: $id found and valid (approved or pending)");
			return $user;
		} catch (Gpf_DbEngine_NoRowException $e) {
			$context->debug("    User with UserId: $id doesn't exist");
			return null;
		}
	}

 	/**
 	 * saves lifetime referral.
 	 * It requires the sale parameter data1 to be filler. This is the identifier
 	 * that identifies the customer, for example his email.
 	 *
 	 * @param Pap_Contexts_Action $context
 	 */
 	public function saveLifetimeReferral(Pap_Contexts_Action $context) {
    	$context->debug(Gpf_Lang::_runtime('PLUGIN LifetimeCommissions saveLifetimeReferral started'));

    	if($context->getDoCommissionsSave()) {
    		$this->saveReferral($context);
    	} else {
    		$context->debug('  Commissions are not saved, stopping');
    	}

    	$context->debug(Gpf_Lang::_runtime('PLUGIN LifetimeCommissions ended'));
    	$context->debug('');

    	return Gpf_Plugins_Engine::PROCESS_CONTINUE;
 	}

 	private function saveReferral(Pap_Contexts_Action $context) {
 		$transactionObject = $context->getTransactionObject();
 		if($transactionObject == null) {
 			$context->debug('  Transaction object is null');
 			return;
 		}

 		$identifier = $this->getIdentifier($context);
 		if($identifier == null || $identifier == '') {
 			$context->debug("  Identifier (data1) is empty, stopping");
 			return;
 		}

 		$userId = $transactionObject->get(Pap_Db_Table_Transactions::USER_ID);
 		$campaignId = $transactionObject->get(Pap_Db_Table_Transactions::CAMPAIGN_ID);
 		$bannerId = $transactionObject->get(Pap_Db_Table_Transactions::BANNER_ID);
 		$channelId = $transactionObject->get(Pap_Db_Table_Transactions::CHANNEL);

        $context->debug("  User is '$userId'");
        $context->debug("  Campaign is '$campaignId'");
        $context->debug("  Banner is '$bannerId'");
        $context->debug("  ChannelId is '$channelId'");

 		$context->debug("  Identifier (data1) is '$identifier'");

 		$lifetimeCommission = new Pap_Db_LifetimeCommission();
 		$lifetimeCommission->setIdentifier($identifier);
 		$lifetimeCommission->setUserId($userId);
 		$lifetimeCommission->setCampaignId($campaignId);
 		if ($bannerId != null) {
 		    $lifetimeCommission->setBannerId($bannerId);
 		}
 		if ($channelId != null) {
 		    $lifetimeCommission->setChannelId($channelId);
 		}

 		try {
 			$lifetimeCommission->loadFromData(array(Pap_Db_Table_LifetimeCommissions::IDENTIFIER));
 			$context->debug("  Lifetime referral: identifier '$identifier' already exists for user: ". $lifetimeCommission->getUserId());
 		} catch (Gpf_DbEngine_NoRowException $e) {
 			$lifetimeCommission->insert();
 			$context->debug("  New lifetime referral: identifier '$identifier' <-> user '$userId' inserted");
 		}
 	}

 	private function getIdentifier(Pap_Contexts_Tracking $context) {
 		if ($context->getExtraDataFromRequest(1) != '') {
 		    return $context->getExtraDataFromRequest(1);
 		}
 		if (!is_null($context->getTransactionObject())) {
            return $context->getTransactionObject()->getData1();
 		}
 		return '';
 	}

 	/**
 	 * add custom screen to menu
 	 *
 	 * @param Gpf_Menu $context
 	 */
 	public function addToMenu(Gpf_Menu $menu) {
//    	$menu->debug('##PLUGIN LifetimeCommissions addToMenu started##');
// 	    $menu->getItem('Affiliates-Overview')->addItem('lifetimeCommissions', $this->_('Lifetime commissions'));
//    	$menu->debug('##PLUGIN LifetimeCommissions addToMenu ended##');
    	return Gpf_Plugins_Engine::PROCESS_CONTINUE;
 	}
 	
    public function initSettings($context) {
        $context->addDbSetting(LifetimeCommissions_Config::LIFETIME_COMMISSION_COOKIE_LIMIT_ACTIVE, Gpf::NO);
        $context->addDbSetting(LifetimeCommissions_Config::CUSTOM_COMMISSION_LEVEL, '');
        $context->addDbSetting(LifetimeCommissions_Config::LIFETIME_COMMISSION_AS_DEFAULT_AFFILIATE, Gpf::NO);
        $context->addDbSetting(LifetimeCommissions_Config::LIFETIME_COMMISSION_CAMPAIGN_BANNER, Gpf::YES);
        $context->addDbSetting(LifetimeCommissions_Config::LIFETIME_COMMISSION_CHANNEL, Gpf::YES);
    }

    /**
     * returns user object from user ID stored in default affiliate
     *
     * @return string
     */
    private function getDefaultAffiliate(Pap_Contexts_Tracking $context) {
        $context->debug("    Trying to get default affiliate");
        if (Gpf_Settings::get(Pap_Settings::SAVE_UNREFERED_SALE_LEAD_SETTING_NAME) != Gpf::YES) {
            $context->debug("      Save unreferred sale is not enabled");
            return null;
        }
        $userId = Gpf_Settings::get(Pap_Settings::DEFAULT_AFFILIATE_SETTING_NAME);
        if($userId == '') {
            $context->debug("      No default affiliate defined");
            return null;
        }

        $userObj = Pap_Affiliates_User::loadFromId($userId);
        if($userObj == null) {
            return null;
        }

        return $userObj;
    }

    private function doCommissionCorrections($value) {
        $value = str_replace('$', '', $value);
        $value = str_replace(',', '.', $value);
        $value = str_replace(' ', '', $value);
        return $value;
    }
}
?>
