<?php
/**
 *   @copyright Copyright (c) 2007 Quality Unit s.r.o.
 *   @author Martin Pullmann
 *   @package PostAffiliatePro
 *   @since Version 1.0.0
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.postaffiliatepro.com/licenses/license
 *
 */

/**
 * @package PostAffiliatePro
 */
class MinTotalCost_Main extends Gpf_Plugins_Handler {

    const MIN_TOTAL_COST = 'MinTotalCostValue';

    private static $instance = false;

    /**
     * @return MinTotalCost_Main
     */
    public static function getHandlerInstance() {
        if(!self::$instance)  {
            return self::$instance = new MinTotalCost_Main();
        }
        return self::$instance;
    }

    public function initFields(Pap_Merchants_Campaign_CommissionTypeEditAdditionalForm $additionalDetails) {
        $additionalDetails->addTextBox($this->_('Minimum total cost value'), self::MIN_TOTAL_COST, $this->_('Minimum total cost value for commission to be created. If lower, no commission will be generated.'));
    }

    public function save(Gpf_Rpc_Form $form) {
        $minTotal = $form->getFieldValue(self::MIN_TOTAL_COST);

        if (!is_numeric($minTotal) && ($minTotal != '')) {
            $form->setErrorMessage($this->_('Wrong format used for \'Minimum total cost\'. The value has to be numeric.'));
            return;
        }

        if (is_numeric($minTotal) && $minTotal < 0) {
            $form->setErrorMessage($this->_('\'Minimum total cost\' value can not be a negative.'));
            return;
        }

        $commTypeAttr = Pap_Db_Table_CommissionTypeAttributes::getInstance();
        $commTypeAttr->setCommissionTypeAttributeValue($form->getFieldValue('Id'), self::MIN_TOTAL_COST, $minTotal);
    }

    public function load(Gpf_Rpc_Form $form) {
        $form->setField(self::MIN_TOTAL_COST, $this->getMinTotalCostValue($form->getFieldValue('Id')));
    }

    private function getMinTotalCostValue($commTypeId) {
        $commTypeAttr = Pap_Db_Table_CommissionTypeAttributes::getInstance();
        try {
            return $commTypeAttr->getCommissionTypeAttribute($commTypeId, self::MIN_TOTAL_COST)->getValue();
        } catch (Gpf_DbEngine_NoRowException $e) {
            return '';
        }
    }

    public function saveCommission(Pap_Common_TransactionCompoundContext $transactionCompoundContext) {
        $context = $transactionCompoundContext->getContext();
        $transaction = $transactionCompoundContext->getTransaction();

        $context->debug('MinTotalCost: Starting the plugin...');

        $minTotal = $this->getMinTotalCostValue($transaction->getCommissionTypeId());

        if ($minTotal == '') {
            $context->debug('MinTotalCost: no value defined! STOPPING...');
        }

        if ($transaction->getTotalCost() < $minTotal) {
            $transactionCompoundContext->setSaveTransaction(false);
            $context->debug('MinTotalCost: commission will NOT be saved, total cost is LOWER than the minumum. Ending...');
        } else {
            $context->debug('MinTotalCost: total cost is higher than the minimum, commission will be saved normally. Ending...');
        }
    }
}

?>
