<?php
/**
 *   @copyright Copyright (c) 2007 Quality Unit s.r.o.
 *   @author Martin Pullmann
 *   @package PostAffiliatePro
 *   @since Version 1.0.0
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.postaffiliatepro.com/licenses/license
 *
 */

/**
 * @package PostAffiliatePro
 */
class RestCommissions_Definition extends Gpf_Plugins_Definition {

    public function __construct() {
        $this->codeName =  'RestCommissions';
        $this->name = $this->_('Rest Commissions');
        $this->description = $this->_('Rest Commissions enables dynamic flow of commissions for tiers, defined per campaign. E.g. you would like to pay 5% commission three levels deep. But in case there are only two tiers, pay the rest 5% to parent of the missing affiliate. In case there is just one affiliate, pay full 15% commission to the affiliate.');
        $this->version = '2.1.0';

        $this->help = $this->_('If main commission structure should be set to 1st tier 5%; 2nd tier 4% and 3rd tier 3% the formula would look like %3-%4-%5. In case you want to payout flat commission, the formula requires only numbers: 3-4-5. This structure defines tiers from right to left, meaning the first one from left is top tier, then their child, then grandchild etc... The missing tier commission <strong>always goes to the parent affiliate</strong> (from right to left).<br /><br />E.g. in case of five levels %2-%2-%2-%4-%5 the commission structure changes like this, if the last affiliate is missing:<br />
                All 5 tiers: %2-%2-%2-%4-%5<br />
                4 tiers: %2-%2-%2-%9<br />
                3 tiers: %2-%2-%11<br />
                2 tiers: %2-%13<br />
                1 tier: %15<br />');

        $this->addRequirement('PapCore', '4.2.10.4');
        $this->addRefuse('SplitCommissions');

        // additional options for each campaign
        $this->addImplementation('PostAffiliate.CommissionTypeEditAdditionalForm.initFields', 'RestCommissions_Main', 'initFields');
        $this->addImplementation('PostAffiliate.CommissionTypeEditAdditionalForm.save', 'RestCommissions_Main', 'save');
        $this->addImplementation('PostAffiliate.CommissionTypeEditAdditionalForm.load', 'RestCommissions_Main', 'load');

        // events
        $this->addImplementation('Tracker.saveCommissions.beforeSaveTransaction', 'RestCommissions_Main', 'beforeSave');
        $this->addImplementation('Tracker.saveAllCommissions', 'RestCommissions_Main', 'modifyCommission');
    }
}

?>
