<?php
/**
 *   @copyright Copyright (c) 2007 Quality Unit s.r.o.
 *   @author Martin Pullmann
 *   @package PostAffiliatePro
 *   @since Version 1.0.0
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.qualityunit.com/licenses/license
 *
 */

/**
 * @package PostAffiliatePro plugins
 */
class Stripe_Tracker extends Pap_Tracking_CallbackTracker {
    const TRANSACTION_TYPE_REFUND = 'charge.refunded';
    const TRANSACTION_TYPE_APPROVED = 'invoice.payment_succeeded';

    /**
     * @return Stripe_Tracker
     */
    public function getInstance() {
        $tracker = new Stripe_Tracker();
        $tracker->setTrackerName("Stripe");
        return $tracker;
    }

    protected function refundChargeback() {
        $transaction = new Pap_Common_Transaction();
        $transaction->processRefundChargeback($this->getOrderID(), Pap_Db_Transaction::TYPE_REFUND, '', $this->getOrderID(), 0, true);
    }

    public function checkStatus() {
        $this->sendBackVerification();

        if ($this->getType() == self::TRANSACTION_TYPE_REFUND) {
            $this->debug('Transaction '.$this->getOrderID().' will be refunded');
            try {
                $this->refundChargeback();
                $this->debug('Refund completed, ending process.');
            } catch (Gpf_Exception $e) {
                $this->debug('Error occurred during transaction refund:' . $e->getMessage());
            }
            return false;
        }

        // check transaction type
        if ($this->getType() != self::TRANSACTION_TYPE_APPROVED) {
            $this->debug("Ignoring type '".$this->getType()."'");
            return false;
        }

        return true;
    }

    protected function sendBackVerification() {
        return "200";
    }

    /**
     *  @return Pap_Tracking_Request
     */
    protected function getRequestObject() {
        $response = @file_get_contents('php://input');
        $response_object = json_decode($response);

        $this->debug(" callback data: ".print_r($response_object,true));
        return $response_object;
    }

    public function readRequestVariables() {
        $request = $this->getRequestObject();
        if (empty($request)) {
            return;
        }

        $this->setType((string)$request->type);
        if (($this->getType() != self::TRANSACTION_TYPE_APPROVED) && ($this->getType() != self::TRANSACTION_TYPE_REFUND)) {
            return;
        }

        $request = $request->data->object;

        $visitorID = $this->getCustomerDescription($request->customer);
        if (!$visitorID) {
            return;
        }

        $cookieValue = $visitorID;

        try {
            $customSeparator = Gpf_Settings::get(Stripe_Config::CUSTOM_SEPARATOR);
            if ($customSeparator != '') {
                $explodedCookieValue = explode($customSeparator, $cookieValue, 2);
                if (count($explodedCookieValue) == 2) {
                    $cookieValue = $explodedCookieValue[1];
                }
            }
        } catch (Gpf_Exception $e) {
            //
        }

        $this->setCookie($cookieValue);
        if (!empty($request->discount)) {
            $coupon = (string)$request->discount->coupon->id;
            if (!empty($coupon)) {
                if (strpos($coupon,"_") !== false) {
                    $this->setCoupon(substr($coupon,0,strpos($coupon,"_")));
                } else {
                    $this->setCoupon($coupon);
                }
            }
        }

        $this->setTransactionID((string)$request->id);

        if ($this->getType() == self::TRANSACTION_TYPE_REFUND) {
            $this->setTotalCost(((float)$request->amount_refunded/100));
            $this->setTransactionID((string)$request->invoice);
        } else {
            $this->setTotalCost(((float)$request->amount_due/100));
        }

        $this->setProductID((string)$request->lines->subscriptions[0]->plan->name);
        $this->setCurrency((string)$request->currency);
    }

    public function getCustomerDescription($cust_id) {
        $this->debug(' Loading customer '.$cust_id);
        $key = Gpf_Settings::get(Stripe_Config::API_KEY);
        require_once('lib/Stripe.php');

        try {
            $customer = Stripe_Customer::retrieve($cust_id, $key);
            $customer = json_decode($customer);

            $this->debug(' Customer found: '.print_r($customer,true));

            $this->setEmail($customer->email);
            $this->setData1($customer->email);

            return $customer->description;
        } catch (Stripe_InvalidRequestError $e) {
            $this->debug('Error loading customer: '.$e->json_body['error']['message']);
            return false;
        }
    }

    public function getOrderID() {
        return $this->getTransactionID();
    }
}
?>
