<?php
/**
 *   @copyright Copyright (c) 2007 Quality Unit s.r.o.
 *   @author Michal Bebjak
 *   @package GwtPhpFramework
 *   @since Version 1.0.0
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.postaffiliatepro.com/licenses/license
 *
 */

/**
 * @package GwtPhpFramework
 */
class TopLevelAffiliateCommision_Config extends Gpf_Plugins_Config {
    const COMMISSION_KEY = 'TopAffiliateCommission';
    const COMMISSION_CAMPAIGNS_SKIPPED = 'TopAffiliateCommission_CampaignsSkipped';
    const COMMISSION_CAMPAIGNS = 'TopAffiliateCommission_Campaigns';

    protected function initFields() {
        $this->addTextBox($this->_("Commission Percent"), self::COMMISSION_KEY, $this->_('Top affiliate commission percentage'));
        $this->addCheckBox($this->_("Skip the Campaigns below"), self::COMMISSION_CAMPAIGNS_SKIPPED, $this->_('If it is checked, the plugin will not apply to the campaigns defined below. Otherwise the plugin applies only to the campaigns defined below.'));
        $this->addTextBox($this->_("Campaigns"), self::COMMISSION_CAMPAIGNS, $this->_('Insert Campaign IDs (comma separated) of the campaigns to which the plugin should or should not apply. If you leave this field empty, the plugin will apply to all the campaigns.'));
    }

    /**
     * @anonym
     * @service custom_separator write
     * @param Gpf_Rpc_Params $params
     * @return Gpf_Rpc_Form
     */
    public function save(Gpf_Rpc_Params $params) {
        $form = new Gpf_Rpc_Form($params);
        $form->addValidator(new Gpf_Rpc_Form_Validator_MandatoryValidator(), self::COMMISSION_KEY);
        $form->addValidator(new Gpf_Rpc_Form_Validator_NumberRangeValidator(0, 100), self::COMMISSION_KEY);
        $form->validate();
        if($form->isSuccessful()){
            Gpf_Settings::set(self::COMMISSION_KEY, $form->getFieldValue(self::COMMISSION_KEY));
            Gpf_Settings::set(self::COMMISSION_CAMPAIGNS_SKIPPED, $form->getFieldValue(self::COMMISSION_CAMPAIGNS_SKIPPED));
            Gpf_Settings::set(self::COMMISSION_CAMPAIGNS, $form->getFieldValue(self::COMMISSION_CAMPAIGNS));
            $form->setInfoMessage($this->_('Plugin configuration saved'));
        }
        return $form;
    }

    /**
     * @anonym
     * @service custom_separator read
     * @param Gpf_Rpc_Params $params
     * @return Gpf_Rpc_Form
     */
    public function load(Gpf_Rpc_Params $params) {
        $form = new Gpf_Rpc_Form($params);
        $form->addField(self::COMMISSION_KEY, Gpf_Settings::get(self::COMMISSION_KEY));
        $form->addField(self::COMMISSION_CAMPAIGNS_SKIPPED, Gpf_Settings::get(self::COMMISSION_CAMPAIGNS_SKIPPED));
        $form->addField(self::COMMISSION_CAMPAIGNS, Gpf_Settings::get(self::COMMISSION_CAMPAIGNS));
        return $form;
    }
}

?>
