<?php
/**
 *   @copyright Copyright (c) 2007 Quality Unit s.r.o.
 *   @author Maros Fric
 *   @package PostAffiliatePro
 *   @since Version 1.0.0
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.postaffiliatepro.com/licenses/license
 *
 */

/**
 * @package PostAffiliatePro plugins
 */
class UltraCart_Tracker extends Pap_Tracking_CallbackTracker {

    protected $xml;

    /**
     * @return UltraCart_Tracker
     */
    public function getInstance() {
        $tracker = new UltraCart_Tracker();
        $tracker->setTrackerName("UltraCart");
        return $tracker;
    }

    protected function refundChargeback() {
        $transaction = new Pap_Common_Transaction();
        $transaction->processRefundChargeback($this->getOrderID(), Pap_Db_Transaction::TYPE_REFUND, '', $this->getOrderID(), 0, true);
    }

    public function checkStatus() {
        if ($this->getPaymentStatus() == "Refunded") {
            $this->debug('Transaction '.$this->getOrderID().' will be marked as refunded.');
            try {
                $this->refundChargeback();
                $this->debug('Refund complete, ending...');
            } catch (Gpf_Exception $e) {
                $this->debug('Error occurred during transaction refund: '.$e->getMessage());
            }
            return false;
        }

        if($this->getPaymentStatus() != "Processed") {  // Declined, Unprocessed
            $this->debug('STOPPING, Payment status is not Processed. Status: '.$this->getPaymentStatus().'. Transaction: '.$this->getOrderID().', payer email: '.$this->getEmail());
            return false;
        }
        
        $this->debug("Payment successful.");
        return true;
    }

    protected function readXmlData() {
        $post_data = file_get_contents('php://input');
        return $post_data;
    }

    protected function outputError() {
        echo '99';
    }

    protected function outputSuccess() {
        echo '200';
    }

    private function computeTotalCost(SimpleXMLElement $xml) {        
        if (Gpf_Settings::get(UltraCart_Config::SHIPPING_HANDLING_SUBSTRACT) == Gpf::YES) {
            return (string)$xml->order->subtotal;
        }
        return (string)$xml->order->total;
    }

    public function readRequestVariables() {
        $input = $this->readXmlData();
        $this->debug("Input get: ".$input);
        try {
            $xml = new SimpleXMLElement($input);
            $this->xml = $xml;
        } catch (Exception $e) {
            $this->setPaymentStatus("Failed");
            $this->debug('Wrong XML format.');
            $this->outputError();
            return;
        }

        $_SERVER['REMOTE_ADDR'] = (string)$xml->order->customer_ip_address;

        // assign posted variables to local variables
        $customField = 'custom_field_'.Gpf_Settings::get(UltraCart_Config::CUSTOM_FIELD_NUMBER);
        $this->debug("Custom field number: ".Gpf_Settings::get(UltraCart_Config::CUSTOM_FIELD_NUMBER));
        $cookieValue = (string)$xml->order->$customField;
        
        $this->setCookie($cookieValue);

        $this->setCoupon((string)$xml->order->coupon->coupon_code);

        $this->setTotalCost($this->computeTotalCost($xml));
        $this->setCurrency((string)$xml->order->currency_code);
        $this->setEmail((string)$xml->order->email);
        $this->setTransactionID((string)$xml->order->order_id);
        $this->setProductID((string)$xml->order->item->item_id);
        $this->setData1((string)$xml->order->email);
        $this->setPaymentStatus((string)$xml->order->payment_status);

        if ($this->isAffiliateRegisterAllowed()) {
            $this->readRequestAffiliateVariables($xml->order);
        }

        $this->outputSuccess();
    }

    public function readRequestAffiliateVariables($xml) {
        $this->setUserFirstName((string)$xml->bill_to_first_name);
        $this->setUserLastName((string)$xml->bill_to_last_name);
        $this->setUserEmail((string)$xml->email);
        $this->setUserCity((string)$xml->bill_to_city);
        $this->setUserAddress((string)$xml->bill_to_address1);
    }

    protected function isAffiliateRegisterAllowed() {
        return (Gpf_Settings::get(UltraCart_Config::REGISTER_AFFILIATE) == Gpf::YES);
    }

    protected function prepareSales(Pap_Tracking_ActionTracker $saleTracker) {
        if (Gpf_Settings::get(UltraCart_Config::WHOLE_CART_AS_ONE) == GPF::YES) {
            parent::prepareSales($saleTracker);
        } else {
            $this->prepareSeparateCartItems($saleTracker);
        }
    }

    private function prepareSeparateCartItems(Pap_Tracking_ActionTracker $saleTracker) {
        $items = $this->xml->order->item;

        $i = 1;
        foreach($items as $item) {
            $sale = $saleTracker->createSale();
            $sale->setTotalCost((string)$item->total_cost_with_discount);
            $sale->setOrderID($this->getOrderID().'('.$i.')');
            $sale->setProductID((string)$item->item_id);
            $sale->setData1($this->getData1());
            $sale->setData2($this->getData2());
            $sale->setData3($this->getData3());
            $sale->setData4($this->getData4());
            $sale->setData5($this->getData5());
            $sale->setCoupon($this->getCouponCode());
            $sale->setCurrency($this->getCurrency());
            $sale->setChannelId($this->getChannelId());
            if ($this->getStatus()!='') {
                $sale->setStatus($this->getStatus());
            }
            if($this->getAffiliateID() != '' && $this->getCampaignID() != '') {
                $sale->setAffiliateID($this->getAffiliateID());
                $sale->setCampaignID($this->getCampaignID());
            }

            $this->setVisitorAndAccount($saleTracker, $this->getAffiliateID(), $this->getCampaignID(), $this->getCookie());
            $i++;
        }
    }

    public function getOrderID() {
        return $this->getTransactionID();
    }
}
?>
