<?php
/**
 *   @copyright Copyright (c) 2007 Quality Unit s.r.o.
 *   @author Martin Pullmann
 *   @package PostAffiliatePro
 *   @since Version 1.0.0
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.postaffiliatepro.com/licenses/license
 *
 */

/**
 * @package PostAffiliatePro plugins
 */
class s2Member_Tracker extends Pap_Tracking_CallbackTracker {

    protected $userPass;
    protected $secret;
    protected $refid;
    protected $parentid;

    public function setSecret($secret) {
        $this->secret = $secret;
    }

    public function getSecret() {
        return $this->secret;
    }

    /**
     * @return s2Member_Tracker
     */
    public function getInstance() {
        $tracker = new s2Member_Tracker();
        $tracker->setTrackerName("s2Member");
        return $tracker;
    }

    public function checkStatus() {
        if ($this->getTransactionID() != '') { // if this is order, ignore secret word
            return true;
        }
        if (Gpf_Settings::get(s2Member_Config::SECRET_WORD) == '') {
            $this->debug(" Secret word is missing! Configure it in your plugin configuration section first!");
            return false;
        }

        if (Gpf_Settings::get(s2Member_Config::SECRET_WORD)!= $this->getSecret()) {
            $this->debug(" Secret word received does not match the one from settings! Probably a fraud, stopping!");
            return false;
        }
        return true;
    }

    /**
     *  @return Pap_Tracking_Request
     */
    protected function getRequestObject() {
        return Pap_Contexts_Action::getContextInstance()->getRequestObject();
    }

    public function readRequestVariables() {
        $this->debug("  Data received: ".print_r($_REQUEST,true));

        $request = $this->getRequestObject();

        // fix IP
        if ($request->getRequestParameter('user_ip') != '') {
            $_SERVER["REMOTE_ADDR"] = $request->getRequestParameter('user_ip');
        }

        if ($request->getRequestParameter('secret') != '') {
            $this->setSecret($request->getRequestParameter('secret'));
        }

        $this->setCookie($request->getPostParam('visitorID'));
        $this->setVisitorID(substr($request->getPostParam('visitorID'),-32));
        $this->setAccountID(substr($request->getPostParam('visitorID'),0,8));
        $this->setAffiliateID($this->computeAffiliateId($this->getVisitorID(), $this->getAccountID()));

        if ($request->getRequestParameter('fname') != '') {
            $this->setUserFirstName($request->getRequestParameter('fname'));
        }
        if ($request->getRequestParameter('lname') != '')  {
            $this->setUserLastName($request->getRequestParameter('lname'));
        }
        if ($request->getRequestParameter('email') != '') {
            $this->setUserEmail($request->getRequestParameter('email'));
        }

        if ($request->getRequestParameter('order') != '') {
            $this->setTransactionID($request->getRequestParameter('order'));
            $this->setTotalCost($request->getRequestParameter('total'));
        }
    }

    protected function fillSignupParams(Gpf_Data_IndexedRecordSet $fields) {
        $this->addField($fields, 'username', $this->getUserEmail());
        $this->addField($fields, 'Id', '');
        for ($i=1; $i<=25; $i++) {
            $this->addField($fields, 'data' . $i, '');
        }

        $this->addField($fields, 'rpassword', $this->getCustomUserPass());
        $this->addField($fields, 'parentuserid', $this->getParentAffiliateId());
        $this->addField($fields, 'firstname', $this->getUserFirstName());
        $this->addField($fields, 'lastname', $this->getUserLastName());
        $this->addField($fields, 'refid', $this->getCustomRefid());
        $this->addField($fields, 'agreeWithTerms', Gpf::YES);
        $this->addField($fields, 'visitorId', $this->getVisitorID());
        if ($this->getUserNotificationEmail() != "") {
            $this->addField($fields, 'notificationemail', $this->getUserEmail());
        }
        Gpf_Plugins_Engine::extensionPoint('Pap_Tracking_CallbackTracker.fillSignupParams', $fields);
    }

    public function registerCommission() {
        if ($this->getUserEmail()) {
            // no commission registration needed here
            return true;
        } else {
            $this->debug('Registering a recurring commission, params TotalCost='.$this->getTotalCost().'; OrderID='.$this->getOrderID());

            $recurringCommission = new Pap_Features_RecurringCommissions_RecurringCommissionsForm();
            try {
                if ($this->getTotalCost()) {
                    $recurringCommission->createCommissionsNoRpc($this->getOrderID(), null, $this->getTotalCost());
                } else {
                    $recurringCommission->createCommissionsNoRpc($this->getOrderID());
                }
            } catch (Exception $e) {
                $this->debug("Can not process recurring commission: ".$e->getMessage());
            }
        }
    }

    protected function isAffiliateRegisterAllowed() {
        return Gpf_Settings::get(s2Member_Config::REGISTER_AFFILIATE) == Gpf::YES;
    }
}
?>
