<?php
interface Lib_ConvertableToArray {
    public function toArray();
}

class Lib_ImpParams implements Lib_ConvertableToArray {
    private $date = '';
    private $rtype = '';
    private $userid = '';
    private $bannerid = '';
    private $parentbannerid = '';
    private $channel = '';
    private $ip = '';
    private $data1 = '';
    private $data2 = '';
    private $impressionhash = '';

    private function getHash() {
        $hash = $this->getUserid();
        $hash .= $this->getBannerid();
        $hash .= $this->getRtype();
        $hash .= $this->getChannel();
        $hash .= $this->getData1();
        $hash .= $this->getData2();
        return sprintf('%d', abs(crc32($hash))) % 8;
    }

    public function setData2($data2) {
        $this->data2 = $data2;
    }

    public function getData2() {
        return $this->data2;
    }

    public function setData1($data1) {
        $this->data1 = $data1;
    }

    public function getData1() {
        return $this->data1;
    }

    public function setIp($ip) {
        $this->ip = $ip;
    }

    public function getIp() {
        return $this->ip;
    }

    public function setChannel($channel) {
        $this->channel = $channel;
    }

    public function getChannel() {
        return $this->channel;
    }

    public function setParentbannerid($parentbannerid) {
        $this->parentbannerid = $parentbannerid;
    }

    public function getParentbannerid() {
        return $this->parentbannerid;
    }

    public function setBannerid($bannerid) {
        $this->bannerid = $bannerid;
    }

    public function getBannerid() {
        return $this->bannerid;
    }

    public function setUserid($userid) {
        $this->userid = $userid;
    }

    public function getUserid() {
        return $this->userid;
    }

    public function setRtype($rtype) {
        $this->rtype = $rtype;
    }

    public function getRtype() {
        return $this->rtype;
    }

    public function setDate($date) {
        $this->date = $date;
    }

    public function getDate() {
        return $this->date;
    }

    public function toArray() {
        $variables = array();
        foreach ($this as $var => $value) {
            $variables[$var] = $value;
        }
        $variables['impressionhash'] = $this->getHash();
        return $variables;
    }
}

class Lib_Server {
    public static function getRemoteIp() {
        if (isset($_SERVER['HTTP_X_FORWARDED_FOR'])) {
            $ip = @$_SERVER['HTTP_X_FORWARDED_FOR'];
            $ipAddresses = explode(',', $ip);   //HTTP_X_FORWARDED_FOR returns multiple IP addresses
            foreach ($ipAddresses as $ipAddress) {
                $ipAddress = trim($ipAddress);
                if (self::isValidIp($ipAddress)) {
                    return $ipAddress;
                }
            }
        }
        if (isset($_SERVER['REMOTE_ADDR'])) {
            return @$_SERVER['REMOTE_ADDR'];
        }
        return '';
    }

    public static function getUserAgent() {
        return @$_SERVER['HTTP_USER_AGENT'];
    }

    public static function getReferer() {
        if (@$_GET['refx2s6d'] != '') {
            return @$_GET['refx2s6d'];
        }
        return @$_SERVER['HTTP_REFERER'];
    }

    private static function isValidIp($ip) {
        if(filter_var($ip, FILTER_VALIDATE_IP, FILTER_FLAG_NO_PRIV_RANGE | FILTER_FLAG_NO_RES_RANGE)) {
            return true;
        }
        return false;
    }
}

class Lib_SettingFile {
    const PARAM_NAME_BANNER_ID = 'param_name_banner_id';
    const PARAM_NAME_USER_ID = 'param_name_user_id';
    const PARAM_NAME_ROTATOR_ID = 'param_name_rotator_id';
    const PARAM_NAME_EXTRA_DATA1 = 'param_name_extra_data1';
    const PARAM_NAME_EXTRA_DATA2 = 'param_name_extra_data2';

    /**
     * @var Lib_Db
     */
    private $db;
    protected $fileName;
    static protected $values = null;

    public function __construct($fileName = null) {
    	if ($fileName != null) {
    		$this->fileName = $fileName;
    		return;
    	}
    	if (@$_SERVER['PROJECT_ACCOUNTS_PATH'] != '') {
    		$this->fileName = $_SERVER['LOCAL_ACCOUNTS_PATH'] . '/accounts/settings.php';
    		if (!@file_exists($this->fileName)) {
    			$this->fileName = $_SERVER['PROJECT_ACCOUNTS_PATH'] . '/accounts/settings.php';
    		}
    	} else {
    		$this->fileName = '../accounts/settings.php';
    	}
    }

    public function setValues($values) {
        self::$values = $values;
    }

    public function load() {
        if(self::$values !== null) {
            return;
        }
        self::$values = array();
        $lines = $this->getFileContent();

        foreach($lines as $line) {
            if(false !== strpos($line, '<?') || false !== strpos($line, '?>')) {
                continue;
            }
            $pos = strpos($line, '=');
            if($pos === false) {
                continue;
            }
            $name = substr($line, 0, $pos);
            $value = substr($line, $pos + 1);
            self::$values[$name] = rtrim($value);
        }

        $this->loadDbSettings();

        $this->setTimezone();
    }

    private function loadDbSettings() {
    	$sql = "SELECT name, value from qu_g_settings where name IN
    			('cronLastRun', 'offlineVisitProcessingDisabled', 'onlineSaleProcessing',
    			'TIMEZONE',	'param_name_campaign_id', 'param_name_rotator_id',
    			'serverName', 'serverNameResolveFrom', 'baseServerUrl', 'serverPort', 'param_name_user_id', 'param_name_banner_id',
    			'param_name_extra_data','param_name_extra_data1','param_name_extra_data2', 'param_name_dest_url',
    			'debug_types', 'delete_cookie', 'p3p_policy_compact', 'url_to_p3p', 'overwrite_cookie', 'cookie_domain',
    			'impTableInput', 'visitsTableInput', 'bannedips_clicks_from_iframe')";

    	$result = $this->getDb()->query($sql);
    	if (false === $result) {
    		throw new Exception('Error in query.');
    	}

		while ($row = mysql_fetch_array($result, MYSQL_ASSOC)) {
		    self::$values[$row['name']] = $row['value'];
		}
		mysql_free_result($result);
    }

    private function loadDbSetting($name) {
    	$sql = "SELECT name, value from qu_g_settings where name='$name'";

    	$result = $this->getDb()->query($sql);
    	if (false === $result) {
    		throw new Exception('Error getting one row. Error in query.');
    	}

    	while ($row = mysql_fetch_array($result, MYSQL_ASSOC)) {
    		print_r($row);
    		self::$values[$row['name']] = $row['value'];
    	}
    	mysql_free_result($result);
    }

    public function get($name) {
    	if (!isset(self::$values[$name]))  {
    		$this->loadDbSetting($name);
    	}
        return @self::$values[$name];
    }

    public function set($name, $value) {
        self::$values[$name] = $value;
    }

    public function isOfflineVisitProcessing(){
        if($this->isOfflineVisitDisabled()){
            return false;
        }
        return $this->isCronRunning();
    }

    public function isOfflineImpressionProcessing(){
        return $this->isCronRunning();
    }

    private function isCronRunning() {
        return (@self::$values['cronLastRun'] != '' && time() < strtotime(self::$values['cronLastRun']) + 900);
    }

    private function isOfflineVisitDisabled() {
        if(isset(self::$values['offlineVisitProcessingDisabled'])){
            return self::$values['offlineVisitProcessingDisabled'] == 'Y';
        }
        return false;
    }

    public function isOnlineSaleProcessingEnabled() {
        if(isset(self::$values['onlineSaleProcessing'])){
            return self::$values['onlineSaleProcessing'] == 'Y';
        }
        return false;
    }

    /**
     *
     * @return Lib_Db
     */
    public function getDb() {
        if ($this->db == null) {
            $this->db = new Lib_Db($this);
        }
        return $this->db;
    }

    public function saveVisit(Lib_VisitParams $visitParams, $tableIndex) {
        $this->getDb()->saveToDb($visitParams, 'qu_pap_visits' . $tableIndex);
    }

    protected function getFileContent() {
        for ($i = 1; $i <= 5; $i++) {
            $lines = file($this->fileName);
            if (!empty($lines)) {
                return $lines;
            }
            usleep(round(rand(0, 100)*1000));
        }
        throw new Exception('Could not read settings file: ' . ' ' . $this->fileName);
    }

    private function setTimezone() {
        if (@self::$values['TIMEZONE'] != '') {
            @date_default_timezone_set(self::$values['TIMEZONE']);
        } else {
            @date_default_timezone_set('America/Los_Angeles');
        }
    }
}
class Lib_Db {
    private $db;
    /**
     *
     * @var Lib_SettingFile
     */
    private $settings;

    public function __construct(Lib_SettingFile $settings) {
        $this->settings = $settings;
        $this->initDatabase();
        $this->query("SET NAMES utf8");
        $this->query("SET CHARACTER SET utf8");
    }

    public function query($sql) {
        $res = @mysql_query($sql, $this->db);
        return $res;
    }

    public function getOneRow($sql) {
        $result = $this->query($sql);
        if (false === $result) {
            throw new Exception('Error getting one row. Error in query.');
        }
        if (@mysql_num_rows($result) != 1) {
            throw new Exception('Error getting one row. Got ' . mysql_num_rows($result) . ' rows.');
        }
        return @mysql_fetch_array($result);
    }

    public function escape($value) {
        return @mysql_real_escape_string($value, $this->db);
    }

    public function saveToDb(Lib_ConvertableToArray $params, $tableName) {
        $columns = '';
        $values = '';
        foreach ($params->toArray() as $key => $value) {
            $columns .= $key.', ';
            $values .= '\''. $this->escape($value).'\', ';
        }
        $insert = 'INSERT INTO '.$tableName.' ('.rtrim($columns, ' ,').')'.
                  ' VALUES ('.rtrim($values, ' ,').')';
        $this->query($insert);
    }

    protected function initDatabase() {
        $this->db = @mysql_connect($this->settings->get('DB_HOSTNAME'), $this->settings->get('DB_USERNAME'), $this->settings->get('DB_PASSWORD'));

        @mysql_select_db($this->settings->get('DB_DATABASE'), $this->db);
    }
}
?>
